package de.starface.com.rpc.client;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.common.RpcTransportToken;
import de.starface.com.rpc.common.valuetranslation.FileTranslator;
import de.starface.com.rpc.services.filetransfer.FileInfo;
import de.starface.com.rpc.services.filetransfer.FileTransferEventListener;
import de.starface.com.rpc.services.filetransfer.FileTransferFailedException;
import de.starface.com.rpc.services.filetransfer.RpcFile;
import de.starface.com.rpc.services.filetransfer.RpcFileDownloadAgent;
import de.starface.com.rpc.services.filetransfer.RpcFileUploadAgent;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.Validate;

import static de.starface.com.rpc.server.AutomaticFileTransfer.isFileTransferEnabledForMethod;

/**
 * This class is used by the {@link RpcClient}/{@link RpcRequestCaller} to provide automatic file
 * transfer during RPC value translation.
 *
 * @author tilmann
 */
public class AutomaticFileTransfer implements FileTranslator {

	private final RpcFileUploadAgent fileUploadAgent;

	private final RpcFileDownloadAgent fileDownloadAgent;

	public static AutomaticFileTransfer createAutomaticFileTransferForRequest(OutgoingRequest<?> request,
																			  FileTransferEventListener downloadListener,
																			  FileTransferEventListener uploadListener)
		throws RpcException {
		if (isFileTransferEnabledForMethod(request.getRpcInterface(), request.getRpcMethod())) {
			AutomaticFileTransfer automaticFileTransfer = new AutomaticFileTransfer(request);
			automaticFileTransfer.addFileDownloadEventListener(downloadListener);
			automaticFileTransfer.addFileUploadEventListener(uploadListener);
			return automaticFileTransfer;
		}
		return null;
	}

	private <TransportToken extends RpcTransportToken> AutomaticFileTransfer(OutgoingRequest<TransportToken> request)
		throws RpcException {
		String namespace = request.getRpcMethodName();
		TransportToken transportToken = request.getTargetTransportToken();
		RpcClient<TransportToken> rpcClient = request.getRpcClient();

		fileDownloadAgent = rpcClient.createRpcAgent(
			de.starface.com.rpc.server.AutomaticFileTransfer.getFileDownloadNamespace(namespace),
			RpcFileDownloadAgent.class, transportToken);
		fileUploadAgent = rpcClient.createRpcAgent(
			de.starface.com.rpc.server.AutomaticFileTransfer.getFileUploadNamespace(namespace),
			RpcFileUploadAgent.class, transportToken);
	}

	public void removeFileUploadEventListener(FileTransferEventListener listener) {
		Validate.notNull(listener, "listener=null");

		fileUploadAgent.removeEventListener(listener);
	}

	public void addFileUploadEventListener(FileTransferEventListener listener) {
		Validate.notNull(listener, "listener=null");

		fileUploadAgent.addEventListener(listener);
	}

	public void addFileDownloadEventListener(FileTransferEventListener listener) {
		Validate.notNull(listener, "listener=null");

		fileDownloadAgent.addEventListener(listener);
	}

	public void removeFileDownloadEventListener(FileTransferEventListener listener) {
		Validate.notNull(listener, "listener=null");

		fileDownloadAgent.removeEventListener(listener);
	}

	/**
	 * @see de.starface.com.rpc.common.valuetranslation.FileTranslator#translateFileToFileId(de.starface.com.rpc.services.filetransfer.RpcFile)
	 */
	public String translateFileToFileId(RpcFile rpcFile) {
		// Null files will be transferred as empty String
		if (rpcFile == null) {
			return "";
		}

		FileInfo fileInfo = rpcFile.getFileInfo();
		try {
			rpcFile = fileUploadAgent.uploadFile(fileInfo.getDescription(), fileInfo.getFileName(),
				fileInfo.getMimeType(), rpcFile.getFile());
			return rpcFile.getId();
		} catch (FileTransferFailedException e) {
			throw new IllegalArgumentException("Could not upload file in method parameter to the server.", e);
		}
	}

	/**
	 * @see de.starface.com.rpc.common.valuetranslation.FileTranslator#translateFileIdToFile(String)
	 */
	public RpcFile translateFileIdToFile(String fileId) {
		// Null files are transferred as empty String
		if (StringUtils.isEmpty(fileId)) {
			return null;
		}

		try {
			return fileDownloadAgent.downloadFile(fileId);
		} catch (FileTransferFailedException e) {
			throw new IllegalArgumentException("A file with the id " + fileId
				+ " could not be retrieved from the server.", e);
		}
	}
}
