package de.starface.com.rpc.client;

import de.starface.com.rpc.common.RpcRequest;
import de.starface.com.rpc.common.RpcTransportToken;
import de.starface.com.rpc.server.RpcObjectRegistry;

import org.apache.commons.lang3.Validate;

import java.lang.reflect.Method;

/**
 * An Outgoing {@link RpcRequest} in the client implementation
 *
 * @author tilmann
 *
 * @param <TransportToken>
 *        the transport token type of the request
 */
public class OutgoingRequest<TransportToken extends RpcTransportToken> extends RpcRequest<TransportToken> {
	private final TransportToken targetTransportToken;
	private final RpcClientBase<TransportToken> rpcClient;

	/**
	 * Creates a new RpcRequest with an {@link RpcTransportToken} identifying the target of the
	 * request
	 *
	 * @param prefix
	 *        The prefix for the method name used for the remote call
	 * @param params
	 *        The parameters used in the remote call (may be null if no arguments are provided)
	 * @param targetTransportToken
	 *        the {@link RpcTransportToken} identifying the target of this request
	 * @param <TransportToken>
	 *        the transport token type of the request
	 * @param rpcInterface
	 *        the RPC interface for which a request should be created
	 * @param rpcMethod
	 *        the RPC method for which a request should be created
	 * @return The newly created RpcRequest
	 *
	 * @throws IllegalArgumentException
	 *         if no method rpcMethodName was provided or the given transport token is null
	 */
	public static <TransportToken extends RpcTransportToken> OutgoingRequest<TransportToken> createRequestWithPrefix(String prefix,
																													 Class<?> rpcInterface,
																													 Method rpcMethod,
																													 Object[] params,
																													 TransportToken targetTransportToken,
																													 RpcClientBase<TransportToken> rpcClient)
		throws IllegalArgumentException {
		String rpcMethodName = RpcObjectRegistry.computeRpcMethodName(prefix, rpcInterface, rpcMethod);
		OutgoingRequest<TransportToken> request = new OutgoingRequest<>(rpcMethodName, rpcInterface,
			rpcMethod, params, targetTransportToken, rpcClient);
		return request;
	}

	private OutgoingRequest(String rpcMethodName, Class<?> rpcInterface, Method rpcMethod, Object[] params,
							TransportToken targetTransportToken, RpcClientBase<TransportToken> rpcClient) {
		super(rpcMethodName, rpcInterface, rpcMethod, params);
		Validate.notNull(targetTransportToken, "targetTransportToken=null");
		this.targetTransportToken = targetTransportToken;
		this.rpcClient = rpcClient;
	}

	public TransportToken getTargetTransportToken() {
		return targetTransportToken;
	}

	public RpcClientBase<TransportToken> getRpcClient() {
		return rpcClient;
	}
}
