package de.starface.com.rpc.client.interceptor;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.client.AutomaticFileTransfer;
import de.starface.com.rpc.client.OutgoingRequest;
import de.starface.com.rpc.common.RpcRequest;
import de.starface.com.rpc.common.RpcTransportToken;
import de.starface.com.rpc.common.interceptor.RequestInterceptorChainLink;
import de.starface.com.rpc.common.valuetranslation.ValueTranslator;
import de.starface.com.rpc.services.filetransfer.FileTransferEventListener;

import java.lang.reflect.Type;
import java.util.Arrays;

import static de.starface.com.rpc.client.AutomaticFileTransfer.createAutomaticFileTransferForRequest;
import static de.starface.com.rpc.common.valuetranslation.ValueTranslator.createValueTranslatorForRequest;

/**
 * This request interceptor implements the aspect of value translation during RPC requests.
 *
 * @author tilmann
 */
public class ValueTranslationRequestInterceptor implements OutgoingRequestInterceptor<RpcTransportToken> {
	private final ValueTranslator valueTranslator;

	public static OutgoingRequestInterceptor<RpcTransportToken> createInterceptorForRequest(OutgoingRequest<? extends RpcTransportToken> request,
																							FileTransferEventListener downloadListener,
																							FileTransferEventListener uploadListener)
		throws RpcException {
		AutomaticFileTransfer fileTransfer = createAutomaticFileTransferForRequest(request, downloadListener,
			uploadListener);

		ValueTranslator valueTranslator = createValueTranslatorForRequest(request, fileTransfer);

		if (valueTranslator == null) {
			return NoOperationRequestInterceptor.INSTANCE;
		}

		return new ValueTranslationRequestInterceptor(valueTranslator);
	}

	private ValueTranslationRequestInterceptor(ValueTranslator valueTranslator) {
		this.valueTranslator = valueTranslator;
	}

	/**
	 * @see de.starface.com.rpc.common.interceptor.RpcRequestInterceptor#interceptRequest(de.starface.com.rpc.common.RpcRequest,
	 *      de.starface.com.rpc.common.interceptor.RequestInterceptorChainLink)
	 */
	public void interceptRequest(OutgoingRequest<RpcTransportToken> request,
								 RequestInterceptorChainLink interceptorChain) throws RpcException {
		translateParameters(request, valueTranslator);

		interceptorChain.processRequest();

		translateReturnValue(request, valueTranslator);
	}

	private void translateReturnValue(RpcRequest<RpcTransportToken> request, ValueTranslator valueTranslator)
		throws RpcException {
		try {
			Object returnValue = request.getReturnValue();
			Type returnType = request.getRpcMethod().getGenericReturnType();

			returnValue = valueTranslator.rpcValueToJavaValue(returnValue, returnType);

			request.setReturnValue(returnValue);
		} catch (Exception e) {
			throw new RpcException(RpcException.Codes.Client.VALUE_TRANSLATION_FAILED,
				"Could not translate return value from XML-RPC format to Java objects. " + requestInfo(request), e);
		}
	}

	private static String requestInfo(RpcRequest<RpcTransportToken> request) {
		if (request == null) {
			return "Request was null";
		}
		String methodString = String.valueOf(request.getRpcMethod());
		String interfaceString = request.getRpcInterface() == null ? "null" : request.getRpcInterface().getSimpleName();

		return "Request method=" + methodString
			+ " interface=" + interfaceString
			+ " parameters=" + Arrays.toString(request.getParameters())
			+ " returning=" + request.getReturnValue();
	}

	private void translateParameters(RpcRequest<RpcTransportToken> request, ValueTranslator valueTranslator)
		throws RpcException {
		try {
			Object[] parameters = request.getParameters();
			Type[] parameterTypes = request.getRpcMethod().getGenericParameterTypes();

			if (parameters != null) {
				for (int i = 0; i < parameters.length; i++) {
					parameters[i] = valueTranslator.javaValueToRpcValue(parameters[i], parameterTypes[i]);
				}
			}
		} catch (Exception e) {
			throw new RpcException(RpcException.Codes.Client.VALUE_TRANSLATION_FAILED,
				"Could not translate given parameter values from Java objects to XML-RPC format. " + requestInfo(request), e);
		}
	}
}
