/*
 * $Id$
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on 15.02.2011 by tilmann
 * 
 * $HeadURL$
 * 
 * $LastChangedDate$ $LastChangedBy$ $LastChangedRevision$
 */
package de.starface.com.rpc.common.events;

import org.apache.commons.lang3.Validate;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

/**
 * A class that helps other classes to provide events to listeners. A single EventsProviderSupport
 * can be used for a single event listener interface.
 *
 * @author tilmann
 * @param <EventsInterface>
 *        the type of the event listeners
 *
 */
public class EventsProviderSupport<EventsInterface> implements EventsProvider<EventsInterface> {
	private final Log log;

	private final Set<EventsInterface> listeners = new CopyOnWriteArraySet<>();

	private final EventsInterface distributor;

	private final EventsProviderExceptionHandler<EventsInterface> exceptionHandler;

	private final Class<EventsInterface> eventsInterface;

	/**
	 * Creates a new EventsProviderSupport that provides events for the given interface.
	 *
	 * @param eventsInterface
	 *        the events interface
	 */
	public EventsProviderSupport(Class<EventsInterface> eventsInterface) {
		this(eventsInterface, null, null);
	}

	/**
	 * Creates a new EventsProviderSupport that provides events for the given interface and logs to
	 * the given log.
	 *
	 * @param eventsInterface
	 *        the events interface
	 * @param log
	 *        the log for logging method calls
	 */
	public EventsProviderSupport(Class<EventsInterface> eventsInterface, Log log) {
		this(eventsInterface, null, log);
	}

	/**
	 * Creates a new EventsProviderSupport that provides events for the given interface and informs
	 * the given exception handler if an exception occurs during event processing.
	 *
	 * @param eventsInterface
	 *        the events interface
	 * @param exceptionHandler
	 *        the exception handler to be when an event receiver causes an exception
	 */
	public EventsProviderSupport(Class<EventsInterface> eventsInterface,
								 EventsProviderExceptionHandler<EventsInterface> exceptionHandler) {
		this(eventsInterface, exceptionHandler, null);
	}

	/**
	 * Creates a new EventsProviderSupport that provides events for the given interface logs to the
	 * given log and informs the given exception handler if an exception occurs during event
	 * processing.
	 *
	 * @param eventsInterface
	 *        the events interface
	 * @param exceptionHandler
	 *        the exception handler to be when an event receiver causes an exception
	 * @param log
	 *        the log for logging method calls
	 */
	public EventsProviderSupport(Class<EventsInterface> eventsInterface,
								 EventsProviderExceptionHandler<EventsInterface> exceptionHandler, Log log) {
		this.eventsInterface = eventsInterface;
		this.distributor = createProxy(eventsInterface, createInvocationHandler());
		this.exceptionHandler = exceptionHandler;
		if (log == null) {
			this.log = LogFactory.getLog(eventsInterface);
		} else {
			this.log = log;
		}
	}

	/**
	 * Gets an implementation of the events interface that upon invocation distributes the event to
	 * all listeners
	 *
	 * @return the events distributor
	 */
	public EventsInterface getEventsDistributor() {
		return distributor;
	}

	/**
	 * @see de.starface.com.rpc.common.events.EventsProvider#subscribeEvents(Object)
	 */
	public void subscribeEvents(EventsInterface listener) {
		Validate.notNull(listener, "listener=null");
		listeners.add(listener);
	}

	/**
	 * @see de.starface.com.rpc.common.events.EventsProvider#unsubscribeEvents(Object)
	 */
	public void unsubscribeEvents(EventsInterface listener) {
		Validate.notNull(listener, "listener=null");
		listeners.remove(listener);
	}

	/**
	 * Unsubscribes all subscribers from this events provider
	 */
	public void unsubscribeAll() {
		listeners.clear();
	}

	protected InvocationHandler createInvocationHandler() {
		return new EventsInvocationHandler();
	}

	@SuppressWarnings("unchecked")
	protected static <Interface> Interface createProxy(Class<Interface> interfaceClass,
													   InvocationHandler invocationHandler) {
		Validate.notNull(interfaceClass, "eventsInterface=null");
		Validate.isTrue(interfaceClass.isInterface(), interfaceClass.getName() + " is not an interface");
		ClassLoader classLoader = interfaceClass.getClassLoader();
		if (classLoader == null) {
			classLoader = Thread.currentThread().getContextClassLoader();
		}
		return (Interface) Proxy.newProxyInstance(classLoader, new Class<?>[]{interfaceClass},
			invocationHandler);
	}

	/**
	 * Checks whether this events provider has any subscribers
	 *
	 * @return {@code true} if any listeners are subscribed to the events, {@code false} otherwise
	 */
	public boolean hasSubscribers() {
		return !listeners.isEmpty();
	}

	protected class EventsInvocationHandler implements InvocationHandler {
		/**
		 * @see InvocationHandler#invoke(Object,
		 *      Method, Object[])
		 */
		public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
			sendEvents(method, args);
			return null;
		}

		protected void sendEvents(Method method, Object[] args) {
			log.trace("Distributing event " + method.getName() + " to subscribers.");
			for (EventsInterface listener : listeners) {
				try {
					method.invoke(listener, args);
				} catch (Exception e) {
					if (exceptionHandler != null) {
						exceptionHandler.handleException(listener, method, args, e);
					} else {
						log.warn("Exception while invoking method " + method.getName() + " of event subscriber.",
							e);
					}
				}
			}
		}
	}

	/**
	 * Returns the interface of the events distributed by this events provider
	 *
	 * @return the events interface
	 */
	public Class<EventsInterface> getEventsInterface() {
		return eventsInterface;
	}
}
