package de.starface.com.rpc.common.valuetranslation;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueType;
import de.starface.com.rpc.annotation.RpcValues;

import org.apache.commons.lang3.StringUtils;

import java.lang.reflect.Field;
import java.lang.reflect.Type;
import java.util.LinkedList;
import java.util.List;

/**
 * This type holds information about a field of an RPC value object that takes part in RPC value
 * translation. It holds a reflection reference to the field as well as the information given in the
 * {@link RpcValue} annotation of the field.
 *
 * @author tilmann
 *
 */
public class ValueField {
	private Field field;

	private String rpcName;

	private int minVersion;

	private int maxVersion;

	RpcValueType rpcType;

	private ValueField(Field field, RpcValue rpcValueAnnotation) {
		initField(field);
		initRpcName(rpcValueAnnotation, field);
		initVersionBoundaries(rpcValueAnnotation);
		initRpcType(rpcValueAnnotation);
	}

	private void initRpcType(RpcValue rpcValueAnnotation) {
		rpcType = rpcValueAnnotation.rpcType();
	}

	private void initField(Field field) {
		this.field = field;
		this.field.setAccessible(true);
	}

	private void initRpcName(RpcValue rpcValueAnnotation, Field field) {
		rpcName = rpcValueAnnotation.value();
		if (StringUtils.isEmpty(rpcName)) {
			rpcName = field.getName();
		}
	}

	private void initVersionBoundaries(RpcValue rpcValueAnnotation) {
		minVersion = rpcValueAnnotation.minVersion();
		maxVersion = rpcValueAnnotation.maxVersion();
	}

	public String getRpcName() {
		return rpcName;
	}

	public String getName() {
		return field.getName();
	}

	/**
	 * Retrieve the fields value in the given instance.
	 *
	 * @param instance
	 *        the instance to retrieve the value of
	 * @return the value
	 */
	public Object getValueOfInstance(Object instance) {
		try {
			return field.get(instance);
		} catch (Exception e) {
			throw new IllegalArgumentException("Could not retrieve value of field " + field.getName()
				+ " of type " + field.getDeclaringClass().getName());
		}
	}

	/**
	 * Set the field value in the given instance
	 *
	 * @param instance
	 *        the instance to set the value in
	 * @param value
	 *        the value to be set
	 */
	public void setValueOfInstance(Object instance, Object value) {
		try {
			field.set(instance, value);
		} catch (Exception e) {
			throw new IllegalArgumentException("Could not set value of field " + field.getName() + " of type "
				+ field.getDeclaringClass().getName());
		}
	}

	public Type getGenericType() {
		return field.getGenericType();
	}

	/**
	 * Retrieve the value fields of the given type that allow the given version.
	 *
	 * @param type
	 *        the type to retrieve the value fields of.
	 * @param version
	 *        the value translation version that the fields are checked against
	 * @param limitToOnePerField
	 *        if set to true only the first value field found for each field that allows the given
	 *        version is returned if the filed has multiple {@link RpcValue} annotations.
	 * @return a list containing the value fields of the type
	 */
	public static List<ValueField> findRpcValueFieldsForVersion(Class<?> type, int version,
																boolean limitToOnePerField) {
		List<ValueField> resultList = new LinkedList<>();
		while (!Object.class.equals(type)) {
			for (Field field : type.getDeclaredFields()) {
				RpcValue rpcValueAnnotation = field.getAnnotation(RpcValue.class);
				boolean added = false;
				if (rpcValueAnnotation != null) {
					added = addTranslatableFieldToList(field, rpcValueAnnotation, resultList, version);
				}
				if (!limitToOnePerField || !added) {
					RpcValues rpcValuesAnnotation = field.getAnnotation(RpcValues.class);
					if (rpcValuesAnnotation != null) {
						RpcValue[] rpcValueMembers = rpcValuesAnnotation.value();
						for (int i = rpcValueMembers.length - 1; i >= 0; i--) {
							added = addTranslatableFieldToList(field, rpcValueMembers[i], resultList, version);
							if (limitToOnePerField && added) {
								break;
							}
						}
					}
				}
			}
			type = type.getSuperclass();
		}
		if (resultList.isEmpty()) {
			throw new IllegalArgumentException("The type " + type.getName()
				+ " cannot be mapped. It has no fields annotated with RpcValue");
		} else {
			return resultList;
		}
	}

	private static boolean addTranslatableFieldToList(Field field, RpcValue rpcValueAnnotation,
													  List<ValueField> list, int version) {
		ValueField valueField = new ValueField(field, rpcValueAnnotation);
		if (valueField.shouldBeTranslatedInVersion(version)) {
			list.add(valueField);
			return true;
		}
		return false;
	}

	public boolean shouldBeTranslatedInVersion(int version) {
		return minVersion <= version && version <= maxVersion;
	}

	public RpcValueType getRpcType() {
		return rpcType;
	}
}
