package de.starface.com.rpc.direct;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.server.RequestExecutorFactory;

import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

/**
 * This class provides a registry for all all started DirectRpcServers inside a single class loading
 * namespace. The servers are identified by their server name.
 *
 * @author tilmann
 *
 */
class DirectRpcRequestExecutorFactoryRegistry {
	// The registry is implemented as a true Java singleton.
	private static final DirectRpcRequestExecutorFactoryRegistry singletonInstance = new DirectRpcRequestExecutorFactoryRegistry();

	static DirectRpcRequestExecutorFactoryRegistry getInstance() {
		return singletonInstance;
	}

	// Allow concurrent access using ConcurrentHashMap
	private ConcurrentMap<String, RequestExecutorFactory<DirectTransportToken>> serverNameToExecutor = new ConcurrentHashMap<>();

	void registerExecutorFactoryWithName(RequestExecutorFactory<DirectTransportToken> requestExecutor,
										 String serverName) throws RpcException {
		RequestExecutorFactory<DirectTransportToken> alreadyRegisteredExecutor = serverNameToExecutor.putIfAbsent(
			serverName, requestExecutor);

		if (alreadyRegisteredExecutor != null) {
			throw new RpcException(RpcException.Codes.Server.DIRECT_SERVER_ALREADY_REGISTERED,
				"A server with the name " + serverName + " is already registered.");
		}
	}

	RequestExecutorFactory<DirectTransportToken> lookupExecutorFactoryWithName(String serverName)
		throws RpcException {
		RequestExecutorFactory<DirectTransportToken> requestExcutor = serverNameToExecutor.get(serverName);

		if (requestExcutor == null) {
			throw new RpcException(RpcException.Codes.Client.DIRECT_SERVER_NOT_REGISTERED,
				"A server with the name " + serverName + " is not available.");
		}

		return requestExcutor;
	}

	void unregisterExecutorFactoryWithName(RequestExecutorFactory<DirectTransportToken> requestExecutor,
										   String serverName) throws RpcException {
		boolean executorRemovedSuccessfully = serverNameToExecutor.remove(serverName, requestExecutor);

		if (!executorRemovedSuccessfully) {
			throw new RpcException(RpcException.Codes.Server.DIRECT_SERVER_NOT_REGISTERED,
				"A server with the name " + serverName + " has not been registered.");
		}
	}
}
