package de.starface.com.rpc.direct;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.server.RequestExecutor;
import de.starface.com.rpc.server.RequestExecutorFactory;
import de.starface.com.rpc.server.RequestExecutorImpl;
import de.starface.com.rpc.server.RpcServer;
import de.starface.com.rpc.server.RpcServerBase;

/**
 * This is a {@link RpcServer} implementation that serves the registered RPC methods for direct java
 * calls inside a single class loader namespace.
 *
 * @author tilmann
 *
 */
public class DirectRpcServer extends RpcServerBase<DirectTransportToken> {
	/**
	 * This is a proxy {@link RequestExecutor} implementation that will be stored in the
	 * {@link DirectRpcRequestExecutorFactoryRegistry} and uses the currently set processorRegistry
	 * and authTokenConverter for the execution of the RPC calls.
	 *
	 * @author tilmann
	 */
	private class DirectRpcRequestExecutorFactory implements RequestExecutorFactory<DirectTransportToken> {
		public RequestExecutor<DirectTransportToken> createRpcRequestExecutor() {
			RequestExecutor<DirectTransportToken> directRpcRequestExecutor = new RequestExecutorImpl<>();
			directRpcRequestExecutor.setServerObjectRegistry(processorRegistry);
			directRpcRequestExecutor.setAuthTokenConverter(authTokenConverter);
			return directRpcRequestExecutor;
		}
	}

	private DirectRpcRequestExecutorFactory requestExecutorFactory = new DirectRpcRequestExecutorFactory();

	private String serverName;

	public static DirectRpcServer createWithServerName(String serverName) {
		return new DirectRpcServer(serverName);
	}

	private DirectRpcServer(String serverName) {
		this.serverName = serverName;
	}

	/**
	 * @see de.starface.com.rpc.server.RpcServerBase#startupImpl()
	 */
	@Override
	protected void startupImpl() throws RpcException {
		registerRequestExecutorFactory();
	}

	private void registerRequestExecutorFactory() throws RpcException {
		DirectRpcRequestExecutorFactoryRegistry registry = DirectRpcRequestExecutorFactoryRegistry.getInstance();

		registry.registerExecutorFactoryWithName(requestExecutorFactory, serverName);
	}

	/**
	 * @see de.starface.com.rpc.server.RpcServerBase#shutdownImpl()
	 */
	@Override
	protected void shutdownImpl() throws RpcException {
		unregisterRequestExecutorFactory();
	}

	private void unregisterRequestExecutorFactory() throws RpcException {
		DirectRpcRequestExecutorFactoryRegistry registry = DirectRpcRequestExecutorFactoryRegistry.getInstance();

		registry.unregisterExecutorFactoryWithName(requestExecutorFactory, serverName);
	}

	/**
	 * @see de.starface.com.rpc.server.RpcServer#getServerAddress()
	 */
	public DirectTransportToken getServerAddress() {
		return new DirectTransportToken(serverName);
	}
}
