package de.starface.com.rpc.direct;

import de.starface.com.rpc.common.RpcTransportToken;

import org.apache.commons.lang3.Validate;

/**
 * This is the {@link RpcTransportToken} implementation used for the {@link DirectRpcServer} and
 * {@link DirectRpcClient}. The token addresses the server using its unique server name and can be
 * used to handle authentication using an auth String.
 *
 * @author tilmann
 */
public class DirectTransportToken implements RpcTransportToken {
	private String serverName;

	private String auth;

	DirectTransportToken() {
		// Reduced visibility because this constructor should only be used internally
	}

	public DirectTransportToken(String serverName) {
		Validate.notEmpty(serverName, "serverName=empty");
		this.serverName = serverName;
	}

	public DirectTransportToken(String serverName, String auth) {
		this(serverName);
		Validate.notEmpty(auth, "auth=empty");
		this.auth = auth;
	}

	public String getServerName() {
		return serverName;
	}

	public String getAuth() {
		return auth;
	}

	/**
	 * Computes the hash code of this transport token depending on its properties
	 *
	 * @see Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((auth == null) ? 0 : auth.hashCode());
		result = prime * result + ((serverName == null) ? 0 : serverName.hashCode());
		return result;
	}

	/**
	 * Compares this transport token depending on its properties
	 *
	 * @see Object#equals(Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		DirectTransportToken other = (DirectTransportToken) obj;
		if (auth == null) {
			if (other.auth != null) {
				return false;
			}
		} else if (!auth.equals(other.auth)) {
			return false;
		}
		if (serverName == null) {
			if (other.serverName != null) {
				return false;
			}
		} else if (!serverName.equals(other.serverName)) {
			return false;
		}
		return true;
	}

	/**
	 * Returns a string representation of this transport token in the form "direct://" + serverName
	 *
	 * @see Object#toString()
	 */
	@Override
	public String toString() {
		return "direct://" + serverName;
	}

	public String getIdentifierString() {
		return "direct://" + serverName;
	}
}
