package de.starface.com.rpc.server;

import de.starface.com.rpc.annotation.RpcValueTranslation;
import de.starface.com.rpc.common.ReflectionUtils;
import de.starface.com.rpc.common.authentication.RpcAuthToken;
import de.starface.com.rpc.common.valuetranslation.FileTranslator;
import de.starface.com.rpc.common.valuetranslation.ValueTranslator;
import de.starface.com.rpc.services.RpcService;
import de.starface.com.rpc.services.filetransfer.FileTransfer;
import de.starface.com.rpc.services.filetransfer.FileTransferEventListener;
import de.starface.com.rpc.services.filetransfer.FileTransferFailedEvent;
import de.starface.com.rpc.services.filetransfer.RpcFile;
import de.starface.com.rpc.services.filetransfer.RpcFileDownloadService;
import de.starface.com.rpc.services.filetransfer.RpcFileUploadService;
import de.starface.com.rpc.services.filetransfer.service.RpcFileRegistry;
import de.starface.com.rpc.services.filetransfer.service.RpcFileRegistry.FileTimeoutListener;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.Validate;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.io.File;
import java.lang.reflect.Method;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

/**
 * This class is used by the {@link RpcServer}/{@link RequestExecutor} to provide automatic file
 * transfer during RPC value translation.
 *
 * @author tilmann
 */
public class AutomaticFileTransfer implements FileTranslator {
	private static final Log log = LogFactory.getLog(AutomaticFileTransfer.class);

	/**
	 * The default timeout that is used for the file uploads that are transfered via automatic file
	 * transfer. The default value is {@code 1000L * 60 * 30} which is 30 minutes.
	 */
	public static long DEFAULT_UPLOAD_TIMEOUT = 1000L * 60 * 30;

	/**
	 * The default timeout that is used for the file downloads that are transfered via automatic
	 * file transfer. The default value is {@code 1000L * 60 * 30} which is 30 minutes.
	 */
	public static long DEFAULT_DOWNLOAD_TIMEOUT = 1000L * 60 * 30;

	/**
	 * The default delete behavior for download files. Default value is {@code false}
	 */
	public static boolean DEFAULT_DELETE_DOWNLOAD_FILES = false;

	/**
	 * The default delete behaviour for upload files. Default value is {@code true}
	 */
	public static boolean DEFAULT_DELETE_UPLOAD_FILES = true;

	private boolean deleteDownloadFiles = DEFAULT_DELETE_DOWNLOAD_FILES;

	private boolean deleteUploadFiles = DEFAULT_DELETE_UPLOAD_FILES;

	private final RpcFileUploadService fileUploadService;

	private final RpcFileDownloadService fileDownloadService;

	private final RpcFileRegistry fileRegistry;

	private long uploadTimeout = DEFAULT_UPLOAD_TIMEOUT;

	private long downloadTimeout = DEFAULT_DOWNLOAD_TIMEOUT;

	public static AutomaticFileTransfer createAutomaticFileTransferForCall(Class<?> rpcInterface,
																		   Method rpcMethod,
																		   String namespace,
																		   Class<? extends RpcAuthToken> requiredAuthType) {
		if (isFileTransferEnabledForMethod(rpcInterface, rpcMethod)) {
			int version = getValueTranslationVersionForMethod(rpcInterface, rpcMethod);
			boolean downloadRequired = isFileTranslationRequiredForReturnType(rpcMethod, version);
			boolean uploadRequired = isFileTranslationRequiredForParameters(rpcMethod, version);
			return new AutomaticFileTransfer(namespace, requiredAuthType, downloadRequired, uploadRequired);
		} else {
			return null;
		}
	}

	private AutomaticFileTransfer(String namespace, Class<? extends RpcAuthToken> requiredAuthType,
								  boolean downloadRequired, boolean uploadRequired) {
		fileRegistry = new RpcFileRegistry();
		fileRegistry.addFileTimeoutListener(new TimeoutListener());

		if (downloadRequired) {
			fileDownloadService = new RpcFileDownloadService(getFileDownloadNamespace(namespace), requiredAuthType);
			fileDownloadService.addEventListener(new DownloadListener());
			fileDownloadService.setFileRegisty(fileRegistry);
		} else {
			fileDownloadService = null;
		}

		if (uploadRequired) {
			fileUploadService = new RpcFileUploadService(getFileUploadNamespace(namespace), requiredAuthType);
			fileUploadService.addEventListener(new UploadListener());
		} else {
			fileUploadService = null;
		}
	}

	public void addFileDownloadEventListener(FileTransferEventListener listener) {
		Validate.notNull(listener, "listener=null");

		if (fileDownloadService != null) {
			fileDownloadService.addEventListener(listener);
		}
	}

	public void removeFileDownloadEventListener(FileTransferEventListener listener) {
		Validate.notNull(listener, "listener=null");

		if (fileDownloadService != null) {
			fileDownloadService.removeEventListener(listener);
		}
	}

	public void addFileUploadEventListener(FileTransferEventListener listener) {
		Validate.notNull(listener, "listener=null");

		if (fileUploadService != null) {
			fileUploadService.addEventListener(listener);
		}
	}

	public void removeFileUploadEventListener(FileTransferEventListener listener) {
		Validate.notNull(listener, "listener=null");

		if (fileUploadService != null) {
			fileUploadService.removeEventListener(listener);
		}
	}

	public void addFileTimeoutListener(FileTimeoutListener listener) {
		Validate.notNull(listener, "listener=null");

		fileRegistry.addFileTimeoutListener(listener);
	}

	public void removeFileTimeoutListener(FileTimeoutListener listener) {
		Validate.notNull(listener, "listener=null");

		fileRegistry.removeFileTimeoutListener(listener);
	}

	public static String getFileDownloadNamespace(String namespace) {
		return namespace + ".rpcFileDownload";
	}

	public static String getFileUploadNamespace(String namespace) {
		return namespace + ".rpcFileUpload";
	}

	public List<RpcService> getRpcServices() {
		List<RpcService> list = new ArrayList<>(2);
		if (fileUploadService != null) {
			list.add(fileUploadService);
		}
		if (fileDownloadService != null) {
			list.add(fileDownloadService);
		}
		return list;
	}

	/**
	 * @see de.starface.com.rpc.common.valuetranslation.FileTranslator#translateFileToFileId(de.starface.com.rpc.services.filetransfer.RpcFile)
	 */
	public String translateFileToFileId(RpcFile rpcFile) {
		// Null files will be transferred as empty String
		if (rpcFile == null) {
			return "";
		}

		fileRegistry.registerRpcFile(rpcFile, downloadTimeout, 1);
		return rpcFile.getId();
	}

	/**
	 * @see de.starface.com.rpc.common.valuetranslation.FileTranslator#translateFileIdToFile(String)
	 */
	public RpcFile translateFileIdToFile(String fileId) {
		// Null files are transferred as empty String
		if (StringUtils.isEmpty(fileId)) {
			return null;
		}

		RpcFile rpcFile = fileRegistry.unregisterFile(fileId);
		if (rpcFile == null) {
			throw new IllegalArgumentException("A file with the id " + fileId
				+ " has not been uploaded or the upload has expired. Current expiration time is " + uploadTimeout
				+ "ms");
		}
		return rpcFile;
	}

	public static boolean isFileTransferEnabledForMethod(Class<?> rpcInterface, Method rpcMethod) {
		RpcValueTranslation annotation = ReflectionUtils.findRpcAnnotation(RpcValueTranslation.class,
			rpcInterface, rpcMethod);

		if (annotation != null) {
			return annotation.automaticFileTransfer();
		} else {
			return RpcValueTranslation.DEFAULT_AUTOMATIC_FILE_TRANSFER;
		}
	}

	private static int getValueTranslationVersionForMethod(Class<?> rpcInterface, Method rpcMethod) {
		RpcValueTranslation annotation = ReflectionUtils.findRpcAnnotation(RpcValueTranslation.class,
			rpcInterface, rpcMethod);

		if (annotation != null) {
			return annotation.version();
		} else {
			throw new IllegalArgumentException("RPC value translation is not enabled for the given method.");
		}
	}

	private static boolean isFileTranslationRequiredForReturnType(Method rpcMethod, int version) {
		return ValueTranslator.isFileTranslationRequiredForType(rpcMethod.getGenericReturnType(), version);
	}

	private static boolean isFileTranslationRequiredForParameters(Method rpcMethod, int version) {
		boolean hasAnyFileTransferParameters = false;

		for (Type parameterType : rpcMethod.getGenericParameterTypes()) {
			hasAnyFileTransferParameters = hasAnyFileTransferParameters
				|| ValueTranslator.isFileTranslationRequiredForType(parameterType, version);
		}

		return hasAnyFileTransferParameters;
	}

	private void registerUploadedFile(RpcFile rpcFile) {
		fileRegistry.registerRpcFile(rpcFile, uploadTimeout, null);
	}

	private class TimeoutListener implements FileTimeoutListener {
		public void fileRemvedByTimeout(RpcFile rpcFile, Integer remainingCount) {
			// Delete timed out uploaded files
			if (remainingCount == null && deleteUploadFiles) {
				deleteFile(rpcFile.getFile());
				return;
			}

			// Delete timed out download files
			if (deleteDownloadFiles) {
				deleteFile(rpcFile.getFile());
				return;
			}
		}
	}

	private class UploadListener implements FileTransferEventListener {
		public void fileTransferInitialized(FileTransfer fileTransfer) {
			if (log.isTraceEnabled()) {
				log.trace("Initialized file upload: " + fileTransfer);
			}
		}

		public void fileTransferProgress(FileTransfer fileTransfer) {
			if (log.isTraceEnabled()) {
				log.trace("Progress " + fileTransfer.getProgress() + " in file upload " + fileTransfer.getId());
			}
		}

		public void fileTransferFailed(FileTransferFailedEvent event) {
			log.info("File upload failded: " + event);
		}

		public void fileTransferCompleted(FileTransfer fileTransfer) {
			if (log.isDebugEnabled()) {
				log.debug("Completed file upload: " + fileTransfer);
			}
			registerUploadedFile(fileTransfer.getRpcFile());
		}
	}

	private class DownloadListener implements FileTransferEventListener {
		public void fileTransferInitialized(FileTransfer fileTransfer) {
			if (log.isTraceEnabled()) {
				log.trace("Initialized file download: " + fileTransfer);
			}
		}

		public void fileTransferProgress(FileTransfer fileTransfer) {
			if (log.isTraceEnabled()) {
				log.trace("Progress " + fileTransfer.getProgress() + " in file download " + fileTransfer.getId());
			}
		}

		public void fileTransferFailed(FileTransferFailedEvent event) {
			log.info("File download failded: " + event);
			deleteDownloadFileIfRequested(event.getFileTransfer());
		}

		public void fileTransferCompleted(FileTransfer fileTransfer) {
			if (log.isDebugEnabled()) {
				log.debug("Completed download: " + fileTransfer);
			}
			deleteDownloadFileIfRequested(fileTransfer);
		}

		private void deleteDownloadFileIfRequested(FileTransfer fileTransfer) {
			if (deleteDownloadFiles) {
				File file = fileTransfer.getRpcFile().getFile();
				deleteFile(file);
			}
		}
	}

	/**
	 * The upload timeout is maximum time that all uploads required for the current RPC call can
	 * take together.
	 *
	 * @return the upload timeout in use by this automatic file transfer
	 */
	public long getUploadTimeout() {
		return uploadTimeout;
	}

	/**
	 * Set the upload timout that is the maximum time that all uploads required for the current RPC
	 * call can take together.
	 *
	 * @param uploadTimeout
	 *        the new upload timeout to be used in milliseconds
	 */
	public void setUploadTimeout(long uploadTimeout) {
		this.uploadTimeout = uploadTimeout;
	}

	/**
	 * The download timeout is maximum time that all downloads required for the current RPC call can
	 * take together.
	 *
	 * @return the download timeout in use by this automatic file transfer
	 */
	public long getDownloadTimeout() {
		return downloadTimeout;
	}

	/**
	 * Set the download timout that is the maximum time that all downloads required for the current
	 * RPC call can take together.
	 *
	 * @param downloadTimeout
	 *        the new download timeout to be used in milliseconds
	 */
	public void setDownloadTimeout(long downloadTimeout) {
		this.downloadTimeout = downloadTimeout;
	}

	private static final boolean deleteFile(File file) {
		if (file == null) {
			return true;
		}
		log.debug("Deleting file " + file.getAbsolutePath() + " after file transfer");
		return file.delete();
	}

	/**
	 * Retrieves the delete behavior for download files. A value of {@code true} means that all
	 * files that have been provided for download by this transfer are deleted if the download
	 * completes or fails or times out.
	 *
	 * @return the current delete behavior for download files
	 */
	public boolean getDeleteDownloadFiles() {
		return deleteDownloadFiles;
	}

	/**
	 * Sets the delete behavior for download files. A value of {@code true} means that all files
	 * that have been provided for download by this transfer are deleted if the download completes
	 * or fails or times out.
	 *
	 * @param deleteDownloadFiles
	 *        the new delete behavior for download files
	 */
	public void setDeleteDownloadFiles(boolean deleteDownloadFiles) {
		this.deleteDownloadFiles = deleteDownloadFiles;
	}

	/**
	 * Retrieves the delete behavior for upload files. A value of {@code true} means that files that
	 * have been uploaded to this transfer are deleted if the upload times out.
	 *
	 * @return the current delete behavior for upload files
	 */
	public boolean getDeleteUploadFiles() {
		return deleteUploadFiles;
	}

	/**
	 * Sets the delete behavior for upload files. A value of {@code true} means that files that have
	 * been uploaded to this transfer are deleted if the upload times out.
	 *
	 * @param deleteUploadFiles
	 *        the new delete behavior for upload files
	 */
	public void setDeleteUploadFiles(boolean deleteUploadFiles) {
		this.deleteUploadFiles = deleteUploadFiles;
	}
}
