package de.starface.com.rpc.server;

import de.starface.com.rpc.common.authentication.RpcAuthToken;
import de.starface.com.rpc.common.valuetranslation.FileTranslator;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

/**
 * This interface defines the common properties and methods of RPC methods that can be registered in
 * the server to be alled from a client.
 *
 * @author tilmann
 */
public interface RpcMethod {
	/**
	 * The underlying Java method if present (may be null)
	 *
	 * @return the underlying Java method
	 */
	public Method getMethod();

	/**
	 * The required auth type for this method to be called. Null indicates no authentication
	 * required
	 *
	 * @return the required auth type
	 */
	public Class<? extends RpcAuthToken> getRequiredAuthType();

	/**
	 * The name of this RPC method. This is the name the method has wenn called through RPC
	 *
	 * @return the name of this RPC method
	 */
	public String getName();

	/**
	 * Invokes this RPC method with the given parameters and returns the invocation's return value
	 *
	 * @param parameters
	 *        the method parameters
	 * @return the method's return value
	 *
	 * @throws IllegalArgumentException
	 *         if the given parameters count/types don't fit for the method
	 * @throws IllegalAccessException
	 *         if the method cannot be accessed
	 * @throws InvocationTargetException
	 *         if the method itself threw an exception during invocation
	 */
	public Object invoke(Object[] parameters) throws IllegalArgumentException, IllegalAccessException,
		InvocationTargetException;

	/**
	 * The underlying Java interface the method is defined in if present (may be null)
	 *
	 * @return the underlying Java interface
	 */
	public Class<?> getRpcInterface();

	/**
	 * Retrieve a file translator that can be used to handle file translation when this method uses
	 * value translation
	 *
	 * @return the file translator for calls to this method
	 */
	public FileTranslator getFileTranslator();
}
