package de.starface.com.rpc.server;

import de.starface.com.rpc.common.ReflectionUtils;
import de.starface.com.rpc.common.authentication.RpcAuthToken;
import de.starface.com.rpc.common.valuetranslation.FileTranslator;
import de.starface.com.rpc.services.RpcService;

import org.apache.commons.lang3.Validate;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

/**
 * An RpcObject holds information of and a reference to a Java object that is actually implementing
 * the RPC methods to be registered for remote access in the {@link RpcObjectRegistry}.
 *
 * @author tilmann
 *
 * @param <RpcInterface>
 *        the RPC interface implemented by the RPC object
 */
public class RpcObject<RpcInterface> {
	private final Class<RpcInterface> rpcInterface;

	private final RpcInterface rpcInstance;

	private final String namePrefix;

	private final Class<? extends RpcAuthToken> requiredAuthType;

	private final String name;

	private final List<RpcMethod> rpcMethods;

	/**
	 * Creates a new RpcObject with the methods from the given RPC interface.
	 *
	 * @param <RpcInterface>
	 *        the RPC interface type
	 * @param rpcInterface
	 *        the RPC interface
	 * @param rpcInstance
	 *        the RPC interface instance that RPC calls are delegated to
	 * @param rpcNamePrefix
	 *        a prefix for the RPC method names
	 * @param requiredAuthType
	 *        the type of authentication required for all methods that don't specify their own auth
	 *        type
	 * @return the newly created RpcObject instance
	 */
	public static <RpcInterface> RpcObject<RpcInterface> createRpcObjectWithInterfaceMethods(Class<RpcInterface> rpcInterface,
																							 RpcInterface rpcInstance,
																							 String rpcNamePrefix,
																							 Class<? extends RpcAuthToken> requiredAuthType) {
		RpcObject<RpcInterface> rpcObject = new RpcObject<>(rpcInterface, rpcInstance, rpcNamePrefix,
			requiredAuthType);
		rpcObject.computeRpcMethods();
		return rpcObject;
	}

	/**
	 * Creates a new RpcObject without any RPC methods
	 *
	 * @param <RpcInterface>
	 *        the RPC interface type
	 * @param rpcInterface
	 *        the RPC interface
	 * @param rpcInstance
	 *        the RPC interface instance that RPC calls are delegated to
	 * @param rpcNamePrefix
	 *        a prefix for the RPC method names
	 * @param requiredAuthType
	 *        the type of authentication required for all methods that don't specify their own auth
	 *        type
	 * @return the newly created RpcObject instance
	 */
	public static <RpcInterface> RpcObject<RpcInterface> createRpcObjectWithoutMethods(Class<RpcInterface> rpcInterface,
																					   RpcInterface rpcInstance,
																					   String rpcNamePrefix,
																					   Class<? extends RpcAuthToken> requiredAuthType) {
		return new RpcObject<>(rpcInterface, rpcInstance, rpcNamePrefix, requiredAuthType);
	}

	private RpcObject(Class<RpcInterface> rpcInterface, RpcInterface rpcInstance, String rpcNamePrefix,
					  Class<? extends RpcAuthToken> requiredAuthType) {
		Validate.notNull(rpcInterface, "The interface that defines the methods to register is required.");
		Validate.notNull(rpcInstance, "The implementation of the rpc methods is required.");
		Validate.isTrue(rpcInterface.isInterface(), "The rpc interface given is not an interface.");
		this.rpcInstance = rpcInstance;
		this.rpcInterface = rpcInterface;
		this.namePrefix = rpcNamePrefix;
		this.name = RpcObjectRegistry.computeRpcObjectName(rpcNamePrefix, rpcInterface);
		if (requiredAuthType != null) {
			this.requiredAuthType = requiredAuthType;
		} else {
			this.requiredAuthType = ReflectionUtils.determineAuthTokenTypeOfRpcInstanceOrInterface(rpcInstance,
				rpcInterface);
		}
		rpcMethods = new ArrayList<>(rpcInterface.getMethods().length);
	}

	private void computeRpcMethods() {
		rpcMethods.clear();
		for (Method method : rpcInterface.getMethods()) {
			createAndAddRpcMethod(method, null);
		}
	}

	/**
	 * Adds an RpcMethod to this RpcObject. The method provided must be declared by the RPC
	 * interface of this RpcObject
	 *
	 * @param method
	 *        the method to be added
	 * @param rpcMethodNameSuffix
	 *        a suffix that is to be used for the RPC method name
	 * @return the newly added RpcMethod
	 */
	public RpcMethod createAndAddRpcMethod(Method method, String rpcMethodNameSuffix) {
		Validate.isTrue(method.getDeclaringClass() == rpcInterface,
			"Cannot add methods that don't belong to this RPC object's RPC interface " + rpcInterface.getName());
		RpcMethod rpcMethod = new ObjectBoundRpcMethod(this, method, rpcMethodNameSuffix);
		rpcMethods.add(rpcMethod);
		return rpcMethod;
	}

	public Class<RpcInterface> getRpcInterface() {
		return rpcInterface;
	}

	public RpcInterface getRpcInstance() {
		return rpcInstance;
	}

	public String getNamePrefix() {
		return namePrefix;
	}

	/**
	 * @see Object#toString()
	 */
	@Override
	public String toString() {
		return "RpcObject [rpcInterface=" + rpcInterface + ", namePrefix=" + namePrefix + ", requiredAuthType="
			+ requiredAuthType + ", name=" + name + "]";
	}

	public Class<? extends RpcAuthToken> getRequiredAuthType() {
		return requiredAuthType;
	}

	public String getName() {
		return name;
	}

	/**
	 * Returns a list with all registered RpcMethods of this RpcObject
	 *
	 * @return the registered RPC methods
	 */
	public List<RpcMethod> getRpcMethods() {
		return rpcMethods;
	}

	List<RpcService> getRpcServices() {
		List<RpcService> rpcServices = new LinkedList<>();
		List<AutomaticFileTransfer> automaticFileTransfers = getAutomaticFileTransfers();
		for (AutomaticFileTransfer automaticFileTransfer : automaticFileTransfers) {
			rpcServices.addAll(automaticFileTransfer.getRpcServices());
		}
		return rpcServices;
	}

	/**
	 * Returns a list with all {@link AutomaticFileTransfer} instances used by all RpcMethods of
	 * this RpcObject.
	 *
	 * @return the automatic file transfers in use
	 */
	public List<AutomaticFileTransfer> getAutomaticFileTransfers() {
		List<AutomaticFileTransfer> automaticFileTransfers = new LinkedList<>();
		for (RpcMethod rpcMethod : rpcMethods) {
			FileTranslator fileTranslator = rpcMethod.getFileTranslator();
			if (fileTranslator instanceof AutomaticFileTransfer) {
				automaticFileTransfers.add((AutomaticFileTransfer) fileTranslator);
			}
		}
		return automaticFileTransfers;
	}
}
