/*
 * $Id$
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on Nov 23, 2007 by may
 * 
 * $HeadURL:
 * http://192.168.2.2/repos/verticomisc/xmpp-xml-rpc/src/de/vertico/com/xmlrpc/XmlRpcProcessorRegistry
 * .java $
 * 
 * $LastChangedDate$ $LastChangedBy$
 * $LastChangedRevision$
 */
package de.starface.com.rpc.server;

import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.com.rpc.annotation.RpcProcedure;
import de.starface.com.rpc.common.ReflectionUtils;
import de.starface.com.rpc.common.authentication.RpcAuthToken;
import de.starface.com.rpc.services.RpcService;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.Validate;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.io.Serializable;
import java.lang.reflect.Method;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

/**
 * This class implements a central registry for {@link RpcObject}s/{@link RpcMethod}s which can be
 * used to execute remote procedure calls. The registry maps {@link RpcMethod} to unique identifies
 * which may occur in RPC requests as the <i>methodName</i>.
 *
 * <p>
 * Furthermore this registy supports the concept of {@link RpcService}s. It provides mechanics to
 * register whole services as a specific set of functions implemented by an corresponding interface
 * for a desired name.
 * </p>
 *
 * @author may
 *
 */
public class RpcObjectRegistry implements Serializable {
	private static final long serialVersionUID = 1L;

	private static final Log log = LogFactory.getLog(RpcObjectRegistry.class);

	/** Convenient way to implement shared registries */
	public static final RpcObjectRegistry DEFAULT_REGISTRY = new RpcObjectRegistry();

	private ConcurrentMap<String, RpcMethod> rpcMethods = new ConcurrentHashMap<>(32, 0.75f, 1);

	private Map<String, RpcObject<?>> rpcObjects = new HashMap<>();

	/**
	 * Registers an RpcMethod in this registry
	 *
	 * @param rpcMethod
	 *        the RPC method to be registered
	 * @throws IllegalArgumentException
	 *         if the method with the same name is already registerd in this registry
	 */
	public void registerRpcMethod(RpcMethod rpcMethod) throws IllegalArgumentException {
		Validate.notNull(rpcMethod, "rpcMethod=null");

		RpcMethod reigsteredServerMethod = rpcMethods.putIfAbsent(rpcMethod.getName(), rpcMethod);

		if (reigsteredServerMethod != null) {
			throw new IllegalArgumentException("Cannot register " + rpcMethod
				+ ". The following server method with the same name is already registered: "
				+ reigsteredServerMethod);
		} else {
			log.debug("Registered  " + rpcMethod);
		}
	}

	/**
	 * Determines a {@link RpcMethod} for the given identifier. Returns {@code null} if no matching
	 * method was found.
	 *
	 * @param rpcMethodName
	 *        The identifier
	 * @return The corresponding processor or null
	 */
	public RpcMethod lookupRpcMethod(String rpcMethodName) {
		Validate.notEmpty(rpcMethodName, "rpcMethodName=empty");

		log.trace("Looking up server method with name " + rpcMethodName);
		return rpcMethods.get(rpcMethodName);
	}

	/**
	 * Registers all methods of an interface implemented by an object for execution by RPC. This
	 * means that all public declared methods defined by the given interface and its super
	 * interfaces are registered. Calls to the interface-methods are then redirected to the given
	 * implementing instance.
	 *
	 * @param <RpcInterface>
	 *        The type of the RPC interface
	 * @param <RpcInstance>
	 *        The type of the RPC instance
	 * @param <AuthToken>
	 *        The {@link RpcAuthToken} required for authentication
	 * @param rpcInstance
	 *        The instance all method calls should be redirected to
	 * @param rpcInterface
	 *        The interface class defining the methods to register
	 * @return the newly registered {@link RpcObject}
	 */
	@SuppressWarnings("hiding")
	public <RpcInterface, RpcInstance extends RpcInterface, AuthToken extends RpcAuthToken> RpcObject<RpcInterface> registerRpcObject(final RpcInstance rpcInstance,
																																	  final Class<RpcInterface> rpcInterface) {

		RpcObject<RpcInterface> rpcObject = RpcObject.createRpcObjectWithInterfaceMethods(rpcInterface,
			rpcInstance, null, null);

		registerRpcObject(rpcObject);

		return rpcObject;
	}

	/**
	 * Registers all {@link RpcMethod}s in the given {@link RpcObject} for execution by RPC.
	 *
	 * @param <RpcInterface>
	 *        The type of the RPC interface
	 * @param rpcObject
	 *        the RPC object holding the methods
	 */
	@SuppressWarnings("hiding")
	public <RpcInterface> void registerRpcObject(RpcObject<RpcInterface> rpcObject) {

		Validate.notNull(rpcObject, "rpcObject=null");

		synchronized (rpcObjects) {
			if (rpcObjects.containsKey(rpcObject.getName())) {
				throw new IllegalArgumentException("An rpc object with the same name as " + rpcObject
					+ " is already registered.");
			}
			Set<RpcMethod> registeredRpcMethods = new HashSet<>();
			Set<RpcService> registeredRpcServices = new HashSet<>();
			try {
				if (log.isTraceEnabled()) {
					log.trace("Registering " + rpcObject + "...");
				}
				for (RpcMethod rpcMethod : rpcObject.getRpcMethods()) {
					registerRpcMethod(rpcMethod);
					registeredRpcMethods.add(rpcMethod);
				}
				for (RpcService rpcService : rpcObject.getRpcServices()) {
					registerRpcService(rpcService);
					registeredRpcServices.add(rpcService);
				}
				log.debug("Successfully registered " + rpcObject);
			} catch (IllegalArgumentException e) {
				unregisterRpcMethods(registeredRpcMethods);
				unregisterRpcServices(registeredRpcServices);
				log.debug("Failed to register " + rpcObject);
				throw e;
			}
			rpcObjects.put(rpcObject.getName(), rpcObject);
		}
	}

	/**
	 * Removes all registered methods from the registry.
	 */
	public void clear() {
		synchronized (rpcObjects) {
			rpcObjects.clear();
			rpcMethods.clear();
		}
	}

	/**
	 * Checks if this registry does not hold any registered RPC methods or objects
	 *
	 * @return {@code true} if no methods or objects are registered
	 */
	public boolean isEmpty() {
		synchronized (rpcObjects) {
			return rpcMethods.isEmpty() && rpcObjects.isEmpty();
		}
	}

	/**
	 * Unregisters all {@link RpcMethod}s in the given collection from this registry
	 *
	 * @param rpcMethods
	 *        the methods to be unregistered
	 */
	public void unregisterRpcMethods(Collection<RpcMethod> rpcMethods) {
		Validate.notNull(rpcMethods, "rpcMethods=null");

		for (RpcMethod method : rpcMethods) {
			unregisterRpcMethod(method);
		}
	}

	/**
	 * Unregister the given {@link RpcMethod} from this registry
	 *
	 * @param rpcMethod
	 *        the method to be unregistered
	 * @return {@code true} if the method was registered in the registry {@code false} otherwise
	 */
	public boolean unregisterRpcMethod(RpcMethod rpcMethod) {
		Validate.notNull(rpcMethod, "rpcMethod=null");

		return unregisterRpcMethod(rpcMethod.getName());
	}

	/**
	 * Unregister the given {@link RpcMethod} from this registry
	 *
	 * @param rpcMethodName
	 *        the name of the method to be unregistered
	 * @return {@code true} if the method was registered in the registry {@code false} otherwise
	 */
	public boolean unregisterRpcMethod(String rpcMethodName) {
		Validate.notEmpty(rpcMethodName, "rpcMethodName=empty");

		if (rpcMethods.remove(rpcMethodName) != null) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Removes the processors previously registered for the given interface type
	 *
	 * @param <RpcInterface>
	 *        the type of the interface or implementation
	 * @param rpcInterface
	 *        the type to unregister processors for
	 * @return the newly unregistered {@link RpcObject}
	 */
	@SuppressWarnings("hiding")
	public <RpcInterface> RpcObject<RpcInterface> unregisterRpcObject(Class<RpcInterface> rpcInterface) {
		return unregisterRpcObject(null, rpcInterface);
	}

	/**
	 * Unregister all {@link RpcMethod}s of the given {@link RpcObject} from the registry
	 *
	 * @param rpcObject
	 *        the object to unregister methods for
	 * @return {@code true} if the object was registered in this registry
	 */
	public boolean unregisterRpcObject(RpcObject<?> rpcObject) {
		Validate.notNull(rpcObject, "rpcObject=null");
		return unregisterRpcObject(rpcObject.getNamePrefix(), rpcObject.getRpcInterface()) != null;
	}

	@SuppressWarnings({"hiding", "unchecked"})
	private <RpcInterface> RpcObject<RpcInterface> unregisterRpcObject(String rpcNamePrefix,
																	   Class<RpcInterface> rpcInterface) {
		Validate.notNull(rpcInterface, "rpcInterface=null");

		synchronized (rpcObjects) {
			RpcObject<?> rpcObject = rpcObjects.remove(computeRpcObjectName(rpcNamePrefix, rpcInterface));
			if (rpcObject != null) {
				unregisterRpcMethods(rpcObject.getRpcMethods());
				unregisterRpcServices(rpcObject.getRpcServices());
				return (RpcObject<RpcInterface>) rpcObject;
			}
			return null;
		}
	}

	public static String computeRpcMethodName(final String rpcNamePrefix, final Class<?> rpcInterface,
											  final Method method) {
		return computeRpcMethodName(rpcNamePrefix, rpcInterface, method, null);
	}

	public static String computeRpcMethodName(final String rpcNamePrefix, final Class<?> rpcInterface,
											  final Method method, final String rpcMethodNameSuffix) {
		String name = computeRpcObjectName(rpcNamePrefix, rpcInterface);
		if (StringUtils.isNotEmpty(name)) {
			name += ".";
		}
		if (StringUtils.isNotEmpty(rpcMethodNameSuffix)) {
			name += rpcMethodNameSuffix;
		} else {
			RpcProcedure rpcProcedureAnnotation = ReflectionUtils.findRpcAnnotationFromMethod(RpcProcedure.class,
				method);
			if (rpcProcedureAnnotation != null) {
				name += rpcProcedureAnnotation.value();
			} else {
				name += method.getName();
			}
		}

		return name;
	}

	public static String computeRpcObjectName(final String rpcNamePrefix, final Class<?> rpcInterface) {
		String name = "";
		if (StringUtils.isNotBlank(rpcNamePrefix)) {
			name = rpcNamePrefix;
		}
		RpcInterface rpcInterfaceAnnotaion = ReflectionUtils.findRpcAnnotationFromInterface(RpcInterface.class,
			rpcInterface);
		if (rpcInterfaceAnnotaion != null) {
			String interfaceName = rpcInterfaceAnnotaion.value();
			if (StringUtils.isNotBlank(interfaceName)) {
				if (StringUtils.isNotEmpty(name)) {
					name += ".";
				}
				name += interfaceName;
			}
		}
		return name;
	}

	Set<String> getRegisteredRpcMethodNames() {
		return new HashSet<>(rpcMethods.keySet());
	}

	/**
	 * Registers the {@link RpcMethod}s of the given {@link RpcService} in this registry
	 *
	 * @param rpcService
	 *        the service to register methods for
	 */
	public void registerRpcService(RpcService rpcService) {
		Validate.notNull(rpcService, "rpcService=null");
		rpcService.register(this);
	}

	/**
	 * Unregisters the {@link RpcMethod}s of the given {@link RpcService} in this registry
	 *
	 * @param rpcService
	 *        the service to unregister methods for
	 */
	public void unregisterRpcService(RpcService rpcService) {
		Validate.notNull(rpcService, "rpcService=null");
		rpcService.unregister(this);
	}

	/**
	 * Unregisters the {@link RpcMethod}s of the given {@link RpcService}s in this registry
	 *
	 * @param rpcServices
	 *        the services to unregister methods for
	 */
	public void unregisterRpcServices(Collection<RpcService> rpcServices) {
		Validate.notNull(rpcServices, "rpcServices=null");

		for (RpcService service : rpcServices) {
			unregisterRpcService(service);
		}
	}
}
