package de.starface.com.rpc.server;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.annotation.RpcAuthRequired;
import de.starface.com.rpc.client.RpcClient;
import de.starface.com.rpc.common.RpcTransportToken;
import de.starface.com.rpc.common.authentication.RpcAuthToken;
import de.starface.com.rpc.common.authentication.RpcAuthTokenConverter;

/**
 * An RpcServer serves all {@link RpcMethod}s contained in its {@link RpcObjectRegistry} to be
 * accessed by RPC calls through a single transport protocol.
 *
 * @author tilmann
 *
 * @param <TransportToken>
 *        The type of {@link RpcTransportToken} of this RpcServer's transport protocol
 */
public interface RpcServer<TransportToken extends RpcTransportToken> {

	/**
	 * Retrieve the {@link RpcObjectRegistry} that holds the {@link RpcMethod}s this RpcServer is
	 * publishing. If not set the {@link RpcObjectRegistry#DEFAULT_REGISTRY} is used.
	 *
	 * @return The {@link RpcObjectRegistry} in use or <code>null</code> if not set.
	 */
	public RpcObjectRegistry getRegistry();

	/**
	 * Sets the {@link RpcObjectRegistry} that holds the {@link RpcMethod}s this RpcServer is
	 * publishing. If set to <code>null</code> the {@link RpcObjectRegistry#DEFAULT_REGISTRY} is
	 * used.
	 *
	 * @param registry
	 *        The {@link RpcObjectRegistry} to use or <code>null</code>.
	 */
	public void setRegistry(RpcObjectRegistry registry);

	/**
	 * Enables authentication for this RpcServer using the given {@link RpcAuthTokenConverter}.
	 * which converts {@link RpcTransportToken}s to {@link RpcAuthToken}s. Per default
	 * authentication is disabled.
	 *
	 * <p>
	 * Once authentication is enabled the underlying transport channel can be used to authenticate
	 * the caller by converting the caller-unique transport token to a {@link RpcAuthToken} which
	 * can be used for authentication using the {@link RpcAuthRequired} annotation.
	 * </p>
	 *
	 * @param authTokenConverter
	 *        The {@link RpcAuthTokenConverter} which should be used by this server
	 *
	 * @see RpcAuthTokenConverter#convert(RpcAuthToken)
	 */
	public void enableAuthentication(RpcAuthTokenConverter<? super TransportToken, ? extends RpcAuthToken> authTokenConverter);

	/**
	 * Check whether this RpcServer is enabled to verify the authentication of RPC calls prior to
	 * invoking the corresponding {@link RpcMethod}s. Per default authentication is disabled.
	 *
	 * @return <code>true</code> if authentication is enabled, <code>false</code> otherwise.
	 */
	public boolean isAuthenticationEnabled();

	/**
	 * Disable authentication for this RpcServer if it has previously been enabled.
	 */
	public void disableAuthentication();

	/**
	 * Starts this RpcServer so that it processes incoming requests. Only start the server if it is
	 * not currently running.
	 *
	 * @throws RpcException
	 *         if a problem occurs during startup.
	 */
	public void startup() throws RpcException;

	/**
	 * Stops this RpcServer so that it no longer processes incoming requests and frees occupied
	 * resources. Only stops the server if it is currently running.
	 *
	 * @throws RpcException
	 *         if a problem occurs during shutdown.
	 */
	public void shutdown() throws RpcException;

	/**
	 * Check if this RpcServer is currently running (between a call to {@link #startup()} and
	 * {@link #shutdown()}).
	 *
	 * @return <code>true</code> if running, <code>false</code> otherwise
	 */
	public boolean isRunning();

	/**
	 * This is a convenience method that can be used to run the RpcServer. Invoking this method will
	 * start up the server and than wait until the Java VM is going down. In this case the server
	 * will be shut down using a shutdown hook. A call to this method will never return until the
	 * RpcServer has shut down or the waiting thread is interrupted.
	 *
	 * @throws RpcException
	 *         if a problem occurs during startup or shutdown.
	 * @throws InterruptedException
	 *         if the waiting thread has been interrupted
	 */
	public void runAndWait() throws RpcException, InterruptedException;

	/**
	 * This is a convenience method that can be used to run the RpcServer. Invoking this method will
	 * start up the server and set u a shutdown hook that will shut down the server when the Java VM
	 * is going down.
	 *
	 * @throws RpcException
	 *         if a problem occurs during startup.
	 */
	public void runAndReturn() throws RpcException;

	/**
	 * Retrieve the {@link RpcTransportToken} that can be used to address this RpcServer when used
	 * with an {@link RpcClient}.
	 *
	 * @return the {@link RpcTransportToken} representing this RpcServer's transport address.
	 */
	public TransportToken getServerAddress();
}
