package de.starface.com.rpc.server;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.common.RpcTransportToken;
import de.starface.com.rpc.common.authentication.RpcAuthToken;
import de.starface.com.rpc.common.authentication.RpcAuthTokenConverter;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.util.concurrent.CountDownLatch;

/**
 * This is an abstract base class for the implementation of {@link RpcServer} instances handling
 * most of the things these have in common.
 *
 * @author tilmann
 *
 * @param <TransportToken>
 *        The type of {@link RpcTransportToken} of this RpcServer's transport protocol
 */
public abstract class RpcServerBase<TransportToken extends RpcTransportToken> implements RpcServer<TransportToken> {
	private static final Log log = LogFactory.getLog(RpcServerBase.class);

	/**
	 * The {@link RpcObjectRegistry} made available to the subclasses
	 */
	protected RpcObjectRegistry processorRegistry;

	/**
	 * The {@link RpcAuthTokenConverter} made available to the subclasses
	 */
	protected RpcAuthTokenConverter<? super TransportToken, ? extends RpcAuthToken> authTokenConverter;

	/**
	 * A flag indicating if this server has successfully been started and not stopped
	 */
	protected boolean running;

	/**
	 * @see de.starface.com.rpc.server.RpcServer#getRegistry()
	 */
	public RpcObjectRegistry getRegistry() {
		return processorRegistry;
	}

	/**
	 * @see de.starface.com.rpc.server.RpcServer#setRegistry(de.starface.com.rpc.server.RpcObjectRegistry)
	 */
	public void setRegistry(RpcObjectRegistry registry) {
		this.processorRegistry = registry;
	}

	public RpcAuthTokenConverter<? super TransportToken, ? extends RpcAuthToken> getAuthTokenConverter() {
		return authTokenConverter;
	}

	/**
	 * @see de.starface.com.rpc.server.RpcServer#enableAuthentication(de.starface.com.rpc.common.authentication.RpcAuthTokenConverter)
	 */
	public void enableAuthentication(RpcAuthTokenConverter<? super TransportToken, ? extends RpcAuthToken> authTokenConverter) {
		this.authTokenConverter = authTokenConverter;
	}

	/**
	 * @see de.starface.com.rpc.server.RpcServer#isAuthenticationEnabled()
	 */
	public boolean isAuthenticationEnabled() {
		return authTokenConverter != null;
	}

	/**
	 * {@inheritDoc}
	 *
	 * @see de.starface.com.rpc.server.RpcServer#disableAuthentication()
	 */
	public void disableAuthentication() {
		authTokenConverter = null;
	}

	/**
	 * @see de.starface.com.rpc.server.RpcServer#startup()
	 */
	public synchronized void startup() throws RpcException {
		if (!running) {
			startupImpl();
			running = true;
		}
	}

	protected abstract void startupImpl() throws RpcException;

	/**
	 * @see de.starface.com.rpc.server.RpcServer#shutdown()
	 */
	public synchronized void shutdown() throws RpcException {
		if (running) {
			log.debug("shutdown: Shutting down RPC server...");
			shutdownImpl();
			running = false;
			log.debug("shutdown: RPC server shut down.");
		} else {
			log.info("shutdown: RPC server is currently not running.");
		}
	}

	/**
	 * Stops this RpcServer so that it no longer processes incoming requests and frees occupied
	 * resources.
	 *
	 * @throws RpcException
	 *         if a problem occurs during shutdown.
	 */
	protected abstract void shutdownImpl() throws RpcException;

	/**
	 * @see de.starface.com.rpc.server.RpcServer#isRunning()
	 */
	public synchronized boolean isRunning() {
		return running;
	}

	/**
	 * @see de.starface.com.rpc.server.RpcServer#runAndWait()
	 */
	public void runAndWait() throws RpcException, InterruptedException {
		startup();
		final CountDownLatch countDownLatch = new CountDownLatch(1);
		Runtime.getRuntime().addShutdownHook(new Thread() {
			@Override
			public void run() {
				countDownLatch.countDown();
			}
		});
		countDownLatch.await();
		shutdown();
	}

	/**
	 * @see de.starface.com.rpc.server.RpcServer#runAndReturn()
	 */
	public void runAndReturn() throws RpcException {
		startup();
		Runtime.getRuntime().addShutdownHook(new Thread() {
			@Override
			public void run() {
				try {
					shutdown();
				} catch (RpcException e) {
					log.warn("runAndReturn: Could not shutdown() this RpcServer.", e);
				}
			}
		});
	}
}
