package de.starface.com.rpc.server.interceptor;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.annotation.RpcLogging;
import de.starface.com.rpc.annotation.RpcLoggingLevel;
import de.starface.com.rpc.common.LoggingUtils;
import de.starface.com.rpc.common.ReflectionUtils;
import de.starface.com.rpc.common.RpcTransportToken;
import de.starface.com.rpc.common.interceptor.RequestInterceptorChainLink;
import de.starface.com.rpc.server.IncomingRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.lang.reflect.Method;

/**
 * This request interceptor implements the aspect of logging RPC requests.
 *
 * @author tilmann
 */
public class LoggingRequestInterceptor implements IncomingRequestInterceptor<RpcTransportToken> {
	private static final Log log = LogFactory.getLog(LoggingRequestInterceptor.class);

	/**
	 * @see de.starface.com.rpc.common.interceptor.RpcRequestInterceptor#interceptRequest(de.starface.com.rpc.common.RpcRequest,
	 *      de.starface.com.rpc.common.interceptor.RequestInterceptorChainLink)
	 */
	public void interceptRequest(IncomingRequest<RpcTransportToken> request,
								 RequestInterceptorChainLink interceptorChain) throws RpcException {
		RpcLoggingLevel logLevel = getLogLevelForCall(request.getRpcInterface(), request.getRpcMethod());

		logIncomingRequest(logLevel, request);

		interceptorChain.processRequest();

		logOutgoingReturnValue(logLevel, request);
	}

	private static void logIncomingRequest(RpcLoggingLevel level, IncomingRequest<RpcTransportToken> request) {
		if (level != null) {
			switch (level) {
				case TRACE:
					if (log.isTraceEnabled()) {
						log.trace(LoggingUtils.incomingRequestToString(request));
					}
					break;
				case DEBUG:
					if (log.isDebugEnabled()) {
						log.debug(LoggingUtils.incomingRequestToString(request));
					}
					break;
				case INFO:
					if (log.isInfoEnabled()) {
						log.info(LoggingUtils.incomingRequestToString(request));
					}
					break;
				case WARN:
					if (log.isWarnEnabled()) {
						log.warn(LoggingUtils.incomingRequestToString(request));
					}
					break;
				case ERROR:
					if (log.isErrorEnabled()) {
						log.error(LoggingUtils.incomingRequestToString(request));
					}
					break;
				case FATAL:
					if (log.isFatalEnabled()) {
						log.fatal(LoggingUtils.incomingRequestToString(request));
					}
					break;
				default:
					break;
			}
		}
	}

	private static void logOutgoingReturnValue(RpcLoggingLevel level, IncomingRequest<RpcTransportToken> request) {
		if (level != null) {
			switch (level) {
				case TRACE:
					if (log.isTraceEnabled()) {
						log.trace(LoggingUtils.outgoingReturnValueToString(request));
					}
					break;
				case DEBUG:
					if (log.isDebugEnabled()) {
						log.debug(LoggingUtils.outgoingReturnValueToString(request));
					}
					break;
				case INFO:
					if (log.isInfoEnabled()) {
						log.info(LoggingUtils.outgoingReturnValueToString(request));
					}
					break;
				case WARN:
					if (log.isWarnEnabled()) {
						log.warn(LoggingUtils.outgoingReturnValueToString(request));
					}
					break;
				case ERROR:
					if (log.isErrorEnabled()) {
						log.error(LoggingUtils.outgoingReturnValueToString(request));
					}
					break;
				case FATAL:
					if (log.isFatalEnabled()) {
						log.fatal(LoggingUtils.outgoingReturnValueToString(request));
					}
					break;
				default:
					break;
			}
		}
	}

	public static RpcLoggingLevel getLogLevelForCall(Class<?> rpcInterface, Method rpcMethod) {
		if (rpcInterface != null && rpcMethod != null) {
			RpcLogging annotation = ReflectionUtils.findRpcAnnotation(RpcLogging.class, rpcInterface, rpcMethod);
			if (annotation != null) {
				return annotation.value();
			}
		}
		return RpcLogging.DefaultValues.DEFAULT_LOG_LEVEL;
	}
}
