package de.starface.com.rpc.server.interceptor;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.common.RpcRequest;
import de.starface.com.rpc.common.RpcTransportToken;
import de.starface.com.rpc.common.interceptor.RequestInterceptorChainLink;
import de.starface.com.rpc.common.valuetranslation.FileTranslator;
import de.starface.com.rpc.common.valuetranslation.ValueTranslator;
import de.starface.com.rpc.server.IncomingRequest;

import java.lang.reflect.Type;

import static de.starface.com.rpc.common.valuetranslation.ValueTranslator.createValueTranslatorForRequest;

/**
 * This request interceptor implements the aspect of value translation during RPC requests.
 *
 * @author tilmann
 */
public class ValueTranslationRequestInterceptor implements IncomingRequestInterceptor<RpcTransportToken> {
	private final ValueTranslator valueTranslator;

	public static IncomingRequestInterceptor<RpcTransportToken> createInterceptorForRequest(IncomingRequest<? extends RpcTransportToken> request,
																							FileTranslator fileTranslator)
		throws RpcException {
		ValueTranslator valueTranslator = createValueTranslatorForRequest(request, fileTranslator);

		if (valueTranslator == null) {
			return NoOperationRequestInterceptor.INSTANCE;
		}

		return new ValueTranslationRequestInterceptor(valueTranslator);
	}

	private ValueTranslationRequestInterceptor(ValueTranslator valueTranslator) {
		this.valueTranslator = valueTranslator;
	}

	/**
	 * @see de.starface.com.rpc.common.interceptor.RpcRequestInterceptor#interceptRequest(de.starface.com.rpc.common.RpcRequest,
	 *      de.starface.com.rpc.common.interceptor.RequestInterceptorChainLink)
	 */
	public void interceptRequest(IncomingRequest<RpcTransportToken> request,
								 RequestInterceptorChainLink interceptorChain) throws RpcException {
		translateParameters(request, valueTranslator);

		interceptorChain.processRequest();

		translateReturnValue(request, valueTranslator);
	}

	private void translateReturnValue(RpcRequest<?> request, ValueTranslator valueTranslator) throws RpcException {
		try {
			Object returnValue = request.getReturnValue();
			Type returnType = request.getRpcMethod().getGenericReturnType();

			returnValue = valueTranslator.javaValueToRpcValue(returnValue, returnType);

			request.setReturnValue(returnValue);
		} catch (Exception e) {
			throw new RpcException(RpcException.Codes.Server.VALUE_TRANSLATION_FAILED,
				"Could not translate return value from Java objects to XML-RPC format.", e);
		}
	}

	private void translateParameters(RpcRequest<?> request, ValueTranslator valueTranslator) throws RpcException {
		try {
			Object[] parameters = request.getParameters();
			Type[] parameterTypes = request.getRpcMethod().getGenericParameterTypes();

			if (parameters != null) {
				for (int i = 0; i < parameters.length; i++) {
					parameters[i] = valueTranslator.rpcValueToJavaValue(parameters[i], parameterTypes[i]);
				}
			}
		} catch (Exception e) {
			throw new RpcException(RpcException.Codes.Server.VALUE_TRANSLATION_FAILED,
				"Could not translate given parameter values from XML-RPC format to Java objects.", e);
		}
	}
}
