package de.starface.com.rpc.services.filetransfer;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueObject;
import de.starface.com.rpc.services.filetransfer.common.OutgoingFile;

import org.apache.commons.lang3.StringUtils;

import java.io.File;
import java.net.URLConnection;

/**
 * A value object holding basic information about a file being transferred
 *
 * @author tilmann
 */
@RpcValueObject
public class FileInfo {
	@RpcValue
	private String description = "";

	@RpcValue
	private String fileName = "";

	@RpcValue
	private String mimeType = "";

	@RpcValue
	private long fileSize;

	@Deprecated
	public FileInfo(String description, String fileName, String mimeType, long fileSize) {
		init(description, fileName, mimeType, fileSize);
	}

	public FileInfo(File file) {
		this(file, null, null, null);
	}

	public FileInfo(File file, String description, String fileName, String mimeType) {
		OutgoingFile.validateFile(file);

		if (fileName == null) {
			fileName = file.getName();
		}
		if (mimeType == null) {
			mimeType = URLConnection.guessContentTypeFromName(fileName);
		}
		init(description, fileName, mimeType, file.length());
	}

	private void init(String description, String fileName, String mimeType, long fileSize) {
		this.description = StringUtils.trimToEmpty(description);
		this.fileName = StringUtils.trimToEmpty(fileName);
		this.mimeType = StringUtils.trimToEmpty(mimeType);
		this.fileSize = fileSize;
	}

	public FileInfo() {
		// Provide empty constructor
	}

	public FileInfo(FileInfo fileStart) {
		this.description = fileStart.description;
		this.fileName = fileStart.fileName;
		this.mimeType = fileStart.mimeType;
		this.fileSize = fileStart.fileSize;
	}

	public String getDescription() {
		return description;
	}

	public String getFileName() {
		return fileName;
	}

	public String getMimeType() {
		return mimeType;
	}

	public long getFileSize() {
		return fileSize;
	}

	@Override
	public String toString() {
		return "FileInfo [description=" + description + ", fileName=" + fileName + ", mimeType=" + mimeType
			+ ", fileSize=" + fileSize + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((description == null) ? 0 : description.hashCode());
		result = prime * result + ((fileName == null) ? 0 : fileName.hashCode());
		result = prime * result + (int) (fileSize ^ (fileSize >>> 32));
		result = prime * result + ((mimeType == null) ? 0 : mimeType.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		FileInfo other = (FileInfo) obj;
		if (description == null) {
			if (other.description != null) {
				return false;
			}
		} else if (!description.equals(other.description)) {
			return false;
		}
		if (fileName == null) {
			if (other.fileName != null) {
				return false;
			}
		} else if (!fileName.equals(other.fileName)) {
			return false;
		}
		if (fileSize != other.fileSize) {
			return false;
		}
		if (mimeType == null) {
			if (other.mimeType != null) {
				return false;
			}
		} else if (!mimeType.equals(other.mimeType)) {
			return false;
		}
		return true;
	}
}
