/*
 * $Id$
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on Sep 26, 2008 by may
 * 
 * $HeadURL$
 * 
 * $LastChangedDate$ $LastChangedBy$ $LastChangedRevision$
 */
package de.starface.com.rpc.services.filetransfer;

import de.starface.com.rpc.common.authentication.RpcAuthToken;
import de.starface.com.rpc.server.RpcObject;
import de.starface.com.rpc.server.RpcObjectRegistry;
import de.starface.com.rpc.services.RpcService;
import de.starface.com.rpc.services.filetransfer.common.RpcFileDownload;
import de.starface.com.rpc.services.filetransfer.service.RpcFileDownloadImpl;
import de.starface.com.rpc.services.filetransfer.service.RpcFileRegistry;

/**
 * An RPC service which provides logic to provide file downloads and enables corresponding
 * notifications via event listeners.
 *
 * @author tilmann
 */
public class RpcFileDownloadService implements RpcService {
	private static final Long DEFAULT_RPC_CALL_TIMEOUT = 1000L * 30;

	/** The actual implementation */
	private final RpcFileDownloadImpl fileDownload;

	private final RpcObject<?> fileDownloadRpcObject;

	/**
	 * Creates a new RpcFileDownloadService
	 *
	 * @param namespace
	 *        To use at method name prefix for this service in the {@link RpcObjectRegistry}
	 * @param requiredAuthType
	 *        the type of authentication required for the callers trying to download files
	 * @param timeout
	 *        Time in milliseconds which might pass between two calls on an ongoing file transfer
	 *        before the server assumes the transfer to be failed
	 */
	public RpcFileDownloadService(String namespace, Class<? extends RpcAuthToken> requiredAuthType, long timeout) {
		fileDownload = new RpcFileDownloadImpl(timeout);
		fileDownloadRpcObject = RpcObject.createRpcObjectWithInterfaceMethods(RpcFileDownload.class, fileDownload,
			namespace, requiredAuthType);
	}

	/**
	 * Creates a new RpcFileDownloadService with a default timeout of 30 seconds
	 *
	 * @param namespace
	 *        To use at method name prefix for this service in the {@link RpcObjectRegistry}
	 * @param requiredAuthType
	 *        the type of authentication required for the callers trying to download files
	 */
	public RpcFileDownloadService(String namespace, Class<? extends RpcAuthToken> requiredAuthType) {
		this(namespace, requiredAuthType, DEFAULT_RPC_CALL_TIMEOUT);
	}

	/**
	 * Adds a {@link FileTransferEventListener} to be informed about downloads using this service
	 *
	 * @param listener
	 *        the listener to be added
	 */
	public void addEventListener(FileTransferEventListener listener) {
		fileDownload.addEventListener(listener);
	}

	/**
	 * Removes a {@link FileTransferEventListener} from this service's listeners
	 *
	 * @param listener
	 *        the listener to be removed
	 */
	public void removeEventListener(FileTransferEventListener listener) {
		fileDownload.removeEventListener(listener);
	}

	/**
	 * @see de.starface.com.rpc.services.RpcService#register(de.starface.com.rpc.server.RpcObjectRegistry)
	 */
	public void register(RpcObjectRegistry registry) {
		registry.registerRpcObject(fileDownloadRpcObject);
	}

	/**
	 * @see de.starface.com.rpc.services.RpcService#unregister(de.starface.com.rpc.server.RpcObjectRegistry)
	 */
	public void unregister(RpcObjectRegistry registry) {
		registry.unregisterRpcObject(fileDownloadRpcObject);
	}

	/**
	 * Sets the file registry that this service uses to look up files to be served as downloads
	 *
	 * @param fileRegistry
	 *        the file registry to be used
	 */
	public void setFileRegisty(RpcFileRegistry fileRegistry) {
		fileDownload.setFileRegistry(fileRegistry);
	}

	/**
	 * Retrieves the file registry that this service uses to look up files to be served as downloads
	 *
	 * @return the file registry in use
	 */
	public RpcFileRegistry getFileRegistry() {
		return fileDownload.getFileRegistry();
	}
}
