/*
 * $Id$
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on Sep 26, 2008 by may
 * 
 * $HeadURL$
 * 
 * $LastChangedDate$ $LastChangedBy$ $LastChangedRevision$
 */
package de.starface.com.rpc.services.filetransfer;

import de.starface.com.rpc.RpcRuntimeException;
import de.starface.com.rpc.client.RpcClientBase;
import de.starface.com.rpc.common.events.EventsProviderSupport;
import de.starface.com.rpc.services.RpcAgent;
import de.starface.com.rpc.services.filetransfer.FileTransfer.HashMethod;
import de.starface.com.rpc.services.filetransfer.FileTransfer.State;
import de.starface.com.rpc.services.filetransfer.common.CompleteUploadResult;
import de.starface.com.rpc.services.filetransfer.common.ContinueUploadResult;
import de.starface.com.rpc.services.filetransfer.common.FileBlock;
import de.starface.com.rpc.services.filetransfer.common.FileEnd;
import de.starface.com.rpc.services.filetransfer.common.InitializeUploadResult;
import de.starface.com.rpc.services.filetransfer.common.OutgoingFileTransfer;
import de.starface.com.rpc.services.filetransfer.common.RpcFileUpload;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.io.File;

/**
 * Implementation of the agent-counterpart for the file upload service. Encapsulates the whole file
 * upload process in a simple method call and thus allows RPC clients to easily upload files to a
 * corresponding server.
 *
 * @author tilmann
 */
public class RpcFileUploadAgent implements RpcAgent<RpcFileUpload> {
	private static final Log log = LogFactory.getLog(RpcFileUploadAgent.class);

	/** The used proxy to call the server */
	private RpcFileUpload fileUploadProxy;

	private EventsProviderSupport<FileTransferEventListener> eventsSupport = new EventsProviderSupport<>(
		FileTransferEventListener.class);

	/**
	 * Default constructor used by {@link RpcClientBase} for example.
	 */
	public RpcFileUploadAgent() {
		// Default constructor used by XmlRpcProxyFactory
	}

	/**
	 * Constructor that lets the caller set the proxy during construction.
	 *
	 * @param proxy
	 *        the {@link RpcFileUpload} proxy to be used during the transfer.
	 */
	public RpcFileUploadAgent(RpcFileUpload proxy) {
		setProxy(proxy);
	}

	/**
	 * Upload the File with default filename, mime type and hash algorithm using the given
	 * description.
	 *
	 * @param description
	 *        Description of the file
	 * @param file
	 *        The file to transfer
	 * @return information about the transferred file
	 * @throws FileTransferFailedException
	 *         if an error occurs sending the file
	 *
	 * @see RpcFileUploadAgent#uploadFile(String, String, String, FileTransfer.HashMethod, File)
	 */
	public RpcFile uploadFile(final String description, final File file) throws FileTransferFailedException {
		return this.uploadFile(description, null, file);
	}

	/**
	 * Upload the File with default mime type and hash algorithm using the given description and
	 * filename.
	 *
	 * @param description
	 *        Description of the transfer
	 * @param filename
	 *        Name of the file to transfer
	 * @param file
	 *        The file to transfer
	 * @return information about the transferred file
	 * @throws FileTransferFailedException
	 *         if an error occurs sending the file
	 *
	 * @see RpcFileUploadAgent#uploadFile(String, String, String, FileTransfer.HashMethod, File)
	 */
	public RpcFile uploadFile(final String description, final String filename, File file)
		throws FileTransferFailedException {
		return this.uploadFile(description, filename, null, file);
	}

	/**
	 * Upload the File with default hash algorithm using the given description, filename and mime
	 * type
	 *
	 * @param description
	 *        Description of the transfer
	 * @param filename
	 *        Name of the file to transfer
	 * @param mimeType
	 *        Mime type of the file to transfer
	 * @param file
	 *        The file to transfer
	 * @return information about the transferred file
	 * @throws FileTransferFailedException
	 *         if an error occurs sending the file
	 *
	 * @see RpcFileUploadAgent#uploadFile(String, String, String, FileTransfer.HashMethod, File)
	 */
	public RpcFile uploadFile(final String description, final String filename, final String mimeType, File file)
		throws FileTransferFailedException {
		return this.uploadFile(description, filename, mimeType, null, file);
	}

	/**
	 * Upload the file via RPC. This method returns when transfer terminated (successful <b>or</b>
	 * unsuccessful). Status information is provided via corresponding events and can be obtained by
	 * registering a corresponding {@link FileTransferEventListener}.
	 *
	 * @param description
	 *        A description for the whole file transfer - empty string is used when {@code null}
	 * @param filename
	 *        A name for the file to transfer - its current name is used when {@code null}
	 * @param mimeType
	 *        A mime type of the file to transfer - its current mime type is tried to determine when
	 *        {@code null}
	 * @param hashMethod
	 *        A hash algorithm to use for verification of this file transfer -
	 *        {@link FileTransfer.HashMethod#SHA} is used when {@code null}
	 * @param file
	 *        The file to transfer - must not be {@code null} and must be a valid file on the file
	 *        system
	 * @return information about the transferred file
	 * @throws FileTransferFailedException
	 *         if an error occurs sending the file
	 */
	public RpcFile uploadFile(String description, String filename, String mimeType, HashMethod hashMethod,
							  File file) throws FileTransferFailedException {
		FileInfo fileInfo = new FileInfo(file, description, filename, mimeType);

		String transferId = null;

		try {
			InitializeUploadResult initializeResult = fileUploadProxy.initializeUpload(fileInfo, hashMethod);
			transferId = initializeResult.getFileTransferId();

			RpcFile rpcFile = new RpcFile(transferId, file, fileInfo);
			OutgoingFileTransfer outgoingTransfer = new OutgoingFileTransfer(transferId, rpcFile, hashMethod,
				eventsSupport.getEventsDistributor(), false);

			if (State.CANCELLED == initializeResult.getFileTransferState() || StringUtils.isEmpty(transferId)) {
				outgoingTransfer.failAndThrow(FileTransferFailedEvent.Codes.DENIED_BY_SERVER,
					"The file transfer request was not accepted by the server.");
			}

			if (State.IN_PROGRESS != initializeResult.getFileTransferState()) {
				outgoingTransfer.setIncomingServerState(initializeResult.getFileTransferState());
			}

			try {
				outgoingTransfer.initializeTransfer(file);

				while (!outgoingTransfer.allBytesTransferred() && outgoingTransfer.getState() == State.IN_PROGRESS) {
					FileBlock fileBlock = outgoingTransfer.getBlockToSend();

					ContinueUploadResult continueResult = fileUploadProxy.continueUpload(transferId, fileBlock);

					outgoingTransfer.setIncomingServerState(continueResult.getFileTransferState());
					outgoingTransfer.signalProgress();
				}

				if (outgoingTransfer.allBytesTransferred() && outgoingTransfer.getState() == State.IN_PROGRESS) {
					FileEnd fileEnd = outgoingTransfer.getEndToSend();

					CompleteUploadResult completeResult = fileUploadProxy.completeUpload(transferId, fileEnd);

					outgoingTransfer.setIncomingServerState(completeResult.getFileTransferState());
					outgoingTransfer.completeTransfer();
				}
				return rpcFile;
			} catch (FileTransferFailedException e) {
				if (e.getCode() == FileTransferFailedEvent.Codes.ABORTED_BY_CLIENT) {
					cancelUploadSilently(outgoingTransfer);
				}
				throw e;
			} catch (RpcRuntimeException e) {
				cancelUploadSilently(outgoingTransfer);
				outgoingTransfer.failAndThrow(FileTransferFailedEvent.Codes.RPC_EXCEPTION,
					"An exception occurred during an XML-RPC call.", e);

				// Just to satisfy the compiler
				return null;
			}
		} catch (RpcRuntimeException e) {
			FileTransferFailedException exception = new FileTransferFailedException(
				FileTransferFailedEvent.Codes.RPC_EXCEPTION, null,
				"An exception occurred during an XML-RPC call.", e);

			eventsSupport.getEventsDistributor().fileTransferFailed(exception);

			throw exception;
		}
	}

	private void cancelUploadSilently(OutgoingFileTransfer outgoingFileTransfer) {
		try {
			fileUploadProxy.cancelUpload(outgoingFileTransfer.getId());
		} catch (Exception e) {
			log.info("Exception while cancelling file upload.", e);
		}
	}

	/**
	 * @see de.starface.com.rpc.services.RpcAgent#getProxyType()
	 */
	public Class<RpcFileUpload> getProxyType() {
		return RpcFileUpload.class;
	}

	/**
	 * @see de.starface.com.rpc.services.RpcAgent#setProxy(Object)
	 */
	public void setProxy(RpcFileUpload proxy) {
		if (fileUploadProxy != null) {
			throw new IllegalStateException("The proxy for this FiletransferAgent has allready been set.");
		}
		fileUploadProxy = proxy;
	}

	/**
	 * Adds a {@link FileTransferEventListener} to be informed about uploads using this agent
	 *
	 * @param listener
	 *        the listener to be added
	 */
	public void addEventListener(FileTransferEventListener listener) {
		eventsSupport.subscribeEvents(listener);
	}

	/**
	 * Removes a {@link FileTransferEventListener} from this agent's listeners
	 *
	 * @param listener
	 *        the listener to be removed
	 */
	public void removeEventListener(FileTransferEventListener listener) {
		eventsSupport.unsubscribeEvents(listener);
	}
}
