/*
 * $Id$
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on Sep 29, 2008 by may
 * 
 * $HeadURL$
 * 
 * $LastChangedDate$ $LastChangedBy$ $LastChangedRevision$
 */
package de.starface.com.rpc.services.filetransfer.common;

import de.starface.com.rpc.services.filetransfer.FileTransfer;
import de.starface.com.rpc.services.filetransfer.FileTransferEventListener;
import de.starface.com.rpc.services.filetransfer.FileTransferFailedEvent;
import de.starface.com.rpc.services.filetransfer.FileTransferFailedException;
import de.starface.com.rpc.services.filetransfer.RpcFile;

import org.apache.commons.lang3.Validate;

/**
 * This class is a base class for the incoming and outgoing file transfer implementation containing
 * common functionality
 *
 * @author tilmann
 */
public abstract class FileTransferBase implements FileTransfer {
	/** Default Hash Algorithm */
	private static final HashMethod DEFAULT_HASH_METHOD = HashMethod.SHA;

	private String id;

	private HashMethod hashMethod;

	private FileTransferEventListener eventListener;

	private long transferredBytes = 0L;

	private State state = State.NEW;

	private boolean isServerFile;

	private FileTransferFailedException failedException;

	private RpcFile rpcFile;

	public FileTransferBase(String id, RpcFile rpcFile, HashMethod hashMethod,
							FileTransferEventListener eventListener, boolean isServerFile) {
		Validate.notNull(rpcFile, "rpcFile=null");
		Validate.notNull(eventListener, "eventListener=null");
		Validate.notEmpty(id, "id=empty");

		this.id = id;
		this.rpcFile = rpcFile;
		this.eventListener = eventListener;
		this.isServerFile = isServerFile;
		if (hashMethod == null) {
			this.hashMethod = DEFAULT_HASH_METHOD;
		} else {
			this.hashMethod = hashMethod;
		}
	}

	/**
	 * @return Returns the sendBytes.
	 */
	public long getTransferredBytes() {
		return transferredBytes;
	}

	protected synchronized void incrementTransferredBytes(long newlyTransferredBytes) {
		this.transferredBytes += newlyTransferredBytes;
	}

	public boolean allBytesTransferred() {
		return transferredBytes >= rpcFile.getFileInfo().getFileSize();
	}

	/**
	 * @return Returns the progress.
	 */
	public double getProgress() {
		return transferredBytes / (double) rpcFile.getFileInfo().getFileSize();
	}

	public State getState() {
		return state;
	}

	protected synchronized void initialize() throws FileTransferFailedException {
		eventListener.fileTransferInitialized(this);
		signalExceptionIfSet();
		throwExceptionIfSet();
		state = State.IN_PROGRESS;
	}

	public void cancel() {
		if (isServerFile) {
			cancelFromServer();
		} else {
			cancelFromClient();
		}
	}

	private void cancelFromOtherSide() {
		if (isServerFile) {
			cancelFromClient();
		} else {
			cancelFromServer();
		}
	}

	private synchronized void cancelFromServer() {
		int failedEventCode = getCancelFromServerFailedEventCode();
		state = State.CANCELLED;
		cleanUpOnAbort();
		failedException = new FileTransferFailedException(failedEventCode, this,
			"The file transfer has been candelled by the server.");
	}

	public synchronized void cancelFromClientAndSendEvent() {
		cancelFromClient();
		signalExceptionIfSet();
	}

	private synchronized void cancelFromClient() {
		state = State.CANCELLED;
		cleanUpOnAbort();
		failedException = new FileTransferFailedException(FileTransferFailedEvent.Codes.ABORTED_BY_CLIENT, this,
			"The file transfer has been candelled by the client.");
	}

	public synchronized void timeout() {
		state = State.TIMEOUT;
		cleanUpOnAbort();
		failedException = new FileTransferFailedException(FileTransferFailedEvent.Codes.TIMEOUT, this,
			"The client has timed out during the file transfer.");
		eventListener.fileTransferFailed(failedException);
	}

	private int getCancelFromServerFailedEventCode() {
		if (state == State.NEW) {
			// This is the case when the file transfer has not yet been initialized
			return FileTransferFailedEvent.Codes.DENIED_BY_SERVER;
		} else {
			return FileTransferFailedEvent.Codes.ABORTED_BY_SERVER;
		}
	}

	public void failAndThrow(int code, String message, Throwable cause) throws FileTransferFailedException {
		failAndThrowImpl(new FileTransferFailedException(code, this, message, cause));
	}

	public void failAndThrow(int code, String message) throws FileTransferFailedException {
		failAndThrowImpl(new FileTransferFailedException(code, this, message));
	}

	private synchronized void failAndThrowImpl(FileTransferFailedException exception)
		throws FileTransferFailedException {
		state = State.FAILED;
		failedException = exception;
		cleanUpOnAbort();
		eventListener.fileTransferFailed(exception);
		throw exception;
	}

	public synchronized void signalProgress() throws FileTransferFailedException {
		if (state == State.IN_PROGRESS) {
			eventListener.fileTransferProgress(this);
			signalExceptionIfSet();
			throwExceptionIfSet();
		}
	}

	protected synchronized void complete() {
		state = State.COMPLETED;
		eventListener.fileTransferCompleted(this);
	}

	private synchronized void signalExceptionIfSet() {
		if (failedException != null) {
			eventListener.fileTransferFailed(failedException);
		}
	}

	private synchronized void throwExceptionIfSet() throws FileTransferFailedException {
		if (failedException != null) {
			throw failedException;
		}
	}

	public synchronized void setIncomingServerState(State state) throws FileTransferFailedException {
		switch (state) {
			case CANCELLED:
				cancelFromOtherSide();
				signalExceptionIfSet();
				throwExceptionIfSet();
				break;
			case TIMEOUT:
				timeout();
				throwExceptionIfSet();
				break;
			case FAILED:
				failAndThrow(FileTransferFailedEvent.Codes.ABORTED_BY_SERVER,
					"The file transfer failed on the server.");
				break;
			default:
				this.state = state;
		}
	}

	protected abstract void cleanUpOnAbort();

	@Override
	public String toString() {
		return "FileTransferBase [id=" + id + ", hashMethod=" + hashMethod + ", eventListener=" + eventListener
			+ ", transferredBytes=" + transferredBytes + ", state=" + state + ", isServerFile=" + isServerFile
			+ ", rpcFile=" + rpcFile + "]";
	}

	public HashMethod getHashMethod() {
		return hashMethod;
	}

	public String getId() {
		return id;
	}

	public RpcFile getRpcFile() {
		return rpcFile;
	}
}
