package de.starface.com.rpc.services.filetransfer.common;

import de.starface.com.rpc.services.filetransfer.FileTransferEventListener;
import de.starface.com.rpc.services.filetransfer.FileTransferFailedEvent;
import de.starface.com.rpc.services.filetransfer.FileTransferFailedException;
import de.starface.com.rpc.services.filetransfer.RpcFile;

import org.apache.commons.lang3.Validate;

import java.io.File;
import java.io.IOException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

/**
 * This class encapsulates the common logic for an outgoing file transfer on either server or client
 * side.
 *
 * @author tilmann
 */
public class OutgoingFileTransfer extends FileTransferBase {

	/** Default Buffer Size */
	public int BUFFER_SIZE = 4000;

	private OutgoingFile outgoingFile = new OutgoingFile();

	private MessageDigest transferDigest;

	private byte[] dataBuffer = new byte[BUFFER_SIZE];

	public OutgoingFileTransfer(String id, RpcFile rpcFile, HashMethod hashMethod,
								FileTransferEventListener eventListener, boolean isServerFile) {
		super(id, rpcFile, hashMethod, eventListener, isServerFile);
	}

	public synchronized void initializeTransfer(File file) throws FileTransferFailedException {
		initialize();
		initializeTransferDigest();
		initializeFileIO(file);
	}

	private void initializeFileIO(File file) throws FileTransferFailedException {
		try {
			outgoingFile.openWithDigestName(getHashMethod().name(), file);
		} catch (Exception e) {
			failAndThrow(FileTransferFailedEvent.Codes.FILE_IO_FAILURE,
				"Could not open file for reading " + file.getAbsolutePath(), e);
		}
		Validate.isTrue(outgoingFile.getSize() == getRpcFile().getFileInfo().getFileSize(),
			"Illegal file size. Expected " + getRpcFile().getFileInfo().getFileSize() + " but actual "
				+ outgoingFile.getSize());
	}

	private void initializeTransferDigest() throws FileTransferFailedException {
		try {
			transferDigest = MessageDigest.getInstance(getHashMethod().name());
		} catch (NoSuchAlgorithmException e) {
			failAndThrow(FileTransferFailedEvent.Codes.FILE_IO_FAILURE,
				"Could not create message digest for data transfer.", e);
		}
	}

	public synchronized FileBlock getBlockToSend() throws FileTransferFailedException {
		if (getTransferredBytes() >= getRpcFile().getFileInfo().getFileSize()) {
			failAndThrow(FileTransferFailedEvent.Codes.FILE_CORRUPT,
				"Cannot read more data from file, everything has been sent.");
		}
		int dataLength = 0;
		try {
			dataLength = outgoingFile.read(dataBuffer);
		} catch (IOException e) {
			failAndThrow(FileTransferFailedEvent.Codes.FILE_IO_FAILURE, "Could not read from outgoing file "
				+ outgoingFile.getName(), e);
		}
		if ((dataLength == 0)
			|| (dataLength < dataBuffer.length && getTransferredBytes() + dataLength < getRpcFile().getFileInfo().getFileSize())) {
			failAndThrow(FileTransferFailedEvent.Codes.FILE_CORRUPT,
				"Cannot read remaining data to send from file " + outgoingFile.getName());
		}
		incrementTransferredBytes(dataLength);
		byte[] blockDigest = transferDigest.digest(dataBuffer);
		return new FileBlock(dataBuffer, blockDigest);
	}

	public synchronized FileEnd getEndToSend() throws FileTransferFailedException {
		if (getTransferredBytes() < getRpcFile().getFileInfo().getFileSize()) {
			failAndThrow(FileTransferFailedEvent.Codes.FILE_CORRUPT,
				"Trying to end transfer before everything has been sent.");
		}
		byte[] fileDigest = null;
		try {
			fileDigest = outgoingFile.closeAndComputeDigest();
		} catch (IOException e) {
			failAndThrow(FileTransferFailedEvent.Codes.FILE_IO_FAILURE, "Could not close outgoing file "
				+ outgoingFile.getName(), e);
		}
		return new FileEnd(fileDigest);
	}

	public synchronized void completeTransfer() {
		cleanUp();
		complete();
	}

	public File getFile() {
		return outgoingFile.getFile();
	}

	@Override
	protected synchronized void cleanUpOnAbort() {
		cleanUp();
	}

	private void cleanUp() {
		outgoingFile.closeSilently();
		dataBuffer = null;
	}
}
