/*
 * $Id$
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on Sep 29, 2008 by may
 * 
 * $HeadURL$
 * 
 * $LastChangedDate$ $LastChangedBy$ $LastChangedRevision$
 */
package de.starface.com.rpc.services.filetransfer.service;

import de.starface.com.rpc.common.events.EventsProviderSupport;
import de.starface.com.rpc.services.filetransfer.FileTransfer.HashMethod;
import de.starface.com.rpc.services.filetransfer.FileTransfer.State;
import de.starface.com.rpc.services.filetransfer.FileTransferEventListener;
import de.starface.com.rpc.services.filetransfer.FileTransferFailedException;
import de.starface.com.rpc.services.filetransfer.RpcFile;
import de.starface.com.rpc.services.filetransfer.common.CompleteDownloadResult;
import de.starface.com.rpc.services.filetransfer.common.ContinueDownloadResult;
import de.starface.com.rpc.services.filetransfer.common.FileBlock;
import de.starface.com.rpc.services.filetransfer.common.FileEnd;
import de.starface.com.rpc.services.filetransfer.common.InitializeDownloadResult;
import de.starface.com.rpc.services.filetransfer.common.OutgoingFileTransfer;
import de.starface.com.rpc.services.filetransfer.common.RpcFileDownload;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.util.UUID;

/**
 * Implementation of the server specific part of the RPC file download.
 *
 * @author tilmann
 */
public class RpcFileDownloadImpl implements RpcFileDownload {
	private static final Log log = LogFactory.getLog(RpcFileDownloadImpl.class);

	private final EventsProviderSupport<FileTransferEventListener> eventsSupport = new EventsProviderSupport<>(
		FileTransferEventListener.class);

	private final FileTransferRegistry<OutgoingFileTransfer> fileTransferRegistry = new FileTransferRegistry<>();

	private RpcFileRegistry fileRegistry;

	private final long timeout;

	public RpcFileDownloadImpl(long timeout) {
		this.timeout = timeout;
	}

	public void addEventListener(FileTransferEventListener listener) {
		eventsSupport.subscribeEvents(listener);
	}

	public void removeEventListener(FileTransferEventListener listener) {
		eventsSupport.unsubscribeEvents(listener);
	}

	/**
	 * @see de.starface.com.rpc.services.filetransfer.common.RpcFileDownload#initializeDownload(String,
	 *      de.starface.com.rpc.services.filetransfer.FileTransfer.HashMethod)
	 */
	public InitializeDownloadResult initializeDownload(String fileId, HashMethod hashMethod) {
		RpcFile downloadFile = getFileRegistry().getFile(fileId);

		if (downloadFile == null) {
			return new InitializeDownloadResult(null, State.FAILED, null);
		}

		OutgoingFileTransfer fileTransfer = new OutgoingFileTransfer(UUID.randomUUID().toString(), downloadFile,
			hashMethod, eventsSupport.getEventsDistributor(), true);

		try {
			fileTransfer.initializeTransfer(downloadFile.getFile());
			fileTransferRegistry.registerFileTransfer(fileTransfer, timeout);
		} catch (FileTransferFailedException e) {
			log.info("File download failed.", e);
		}
		return new InitializeDownloadResult(fileTransfer.getId(), fileTransfer.getState(),
			downloadFile.getFileInfo());
	}

	/**
	 * @see de.starface.com.rpc.services.filetransfer.common.RpcFileDownload#continueDownload(String)
	 */
	public ContinueDownloadResult continueDownload(String fileTransferId) {
		OutgoingFileTransfer outgoingTransfer = fileTransferRegistry.getFileTransfer(fileTransferId);

		if (outgoingTransfer == null) {
			return new ContinueDownloadResult(State.TIMEOUT, null);
		}

		FileBlock fileBlock = null;

		if (State.IN_PROGRESS == outgoingTransfer.getState()) {
			try {
				fileBlock = outgoingTransfer.getBlockToSend();
				outgoingTransfer.signalProgress();
			} catch (FileTransferFailedException e) {
				log.info("File download failed.", e);
			}
		}
		updateRegistry(outgoingTransfer);
		return new ContinueDownloadResult(outgoingTransfer.getState(), fileBlock);
	}

	/**
	 * @see de.starface.com.rpc.services.filetransfer.common.RpcFileDownload#completeDownload(String)
	 */
	public CompleteDownloadResult completeDownload(String fileTransferId) {
		OutgoingFileTransfer outgoingTransfer = fileTransferRegistry.getFileTransfer(fileTransferId);

		if (outgoingTransfer == null) {
			return new CompleteDownloadResult(State.TIMEOUT, null);
		}

		FileEnd fileEnd = null;

		if (State.IN_PROGRESS == outgoingTransfer.getState()) {
			try {
				fileEnd = outgoingTransfer.getEndToSend();
				outgoingTransfer.completeTransfer();
			} catch (FileTransferFailedException e) {
				log.info("File download failed.", e);
			}
		}
		updateRegistry(outgoingTransfer);
		return new CompleteDownloadResult(outgoingTransfer.getState(), fileEnd);
	}

	/**
	 * @see de.starface.com.rpc.services.filetransfer.common.RpcFileDownload#cancelDownload(String)
	 */
	public void cancelDownload(String fileTransferId) {
		OutgoingFileTransfer outgoingTransfer = fileTransferRegistry.getFileTransfer(fileTransferId);

		if (outgoingTransfer != null) {
			outgoingTransfer.cancelFromClientAndSendEvent();
			updateRegistry(outgoingTransfer);
		}
	}

	private void updateRegistry(OutgoingFileTransfer outgoingTransfer) {
		if (outgoingTransfer.getState() != State.IN_PROGRESS) {
			fileTransferRegistry.unregisterFileTransfer(outgoingTransfer);
		}
	}

	public RpcFileRegistry getFileRegistry() {
		if (fileRegistry == null) {
			return RpcFileRegistry.DEFAULT_REGISTY;
		} else {
			return fileRegistry;
		}
	}

	public void setFileRegistry(RpcFileRegistry fileRegistry) {
		this.fileRegistry = fileRegistry;
	}
}
