package de.starface.com.rpc.services.filetransfer.service;

import de.starface.com.rpc.common.events.EventsProviderSupport;
import de.starface.com.rpc.services.filetransfer.FileInfo;
import de.starface.com.rpc.services.filetransfer.RpcFile;
import de.starface.com.rpc.services.filetransfer.RpcFileDownloadService;
import de.starface.com.rpc.services.filetransfer.common.OutgoingFile;

import org.apache.commons.lang3.Validate;

import java.io.File;
import java.util.HashMap;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

/**
 * This class is a registry of files that are available to download by RPC file download services.
 * It provides means to limit download of files with a timeout.
 *
 * @author tilmann
 */
public class RpcFileRegistry {
	/**
	 * The default registry that is used when no other has been set in the corresponding
	 * {@link RpcFileDownloadService}.
	 */
	public static final RpcFileRegistry DEFAULT_REGISTY = new RpcFileRegistry();

	private static Timer timeoutTimer;
	private static final Object syncTimeoutTimer = new Object();

	private void createTimeoutTimer() {
		synchronized (syncTimeoutTimer) {
			if (timeoutTimer == null) {
				timeoutTimer = new Timer("XML-RPC File transfer timeout", true);
				// initialize with cleanup task
				timeoutTimer.schedule(new TimerTask() {
					@Override
					public void run() {
						synchronized (syncTimeoutTimer) {
							if (timeoutTimer != null) {
								timeoutTimer.purge();
							}
						}
					}
				}, 3600000L, 3600000L);
			}
		}
	}

	private void destroyTimeoutTimer() {
		synchronized (syncTimeoutTimer) {
			if (timeoutTimer != null) {
				timeoutTimer.cancel();
				timeoutTimer.purge();
				timeoutTimer = null;
			}
		}
	}

	private void createOrDestroyTimeoutTimer() {
		if (registeredFiles.isEmpty()) {
			destroyTimeoutTimer();
		} else {
			createTimeoutTimer();
		}
	}

	private final Map<String, RegistryEntry> registeredFiles = new HashMap<>();

	private final EventsProviderSupport<FileTimeoutListener> eventsSupport = new EventsProviderSupport<>(
		FileTimeoutListener.class);

	public RpcFile registerFileWithTimeout(String fileId, File file, Long timeout) {
		return registerFile(fileId, file, null, null, null, timeout, null);
	}

	public RpcFile registerFileWithCount(String fileId, File file, Integer count) {
		return registerFile(fileId, file, null, null, null, null, count);
	}

	public RpcFile registerFile(String fileId, File file, Long timeout, Integer count) {
		return registerFile(fileId, file, null, null, null, timeout, count);
	}

	public RpcFile registerFile(String fileId, File file, String fileName, String description,
								String mimeType, Long timeout, Integer count) {
		FileInfo fileInfo = new FileInfo(file, description, fileName, mimeType);
		RpcFile rpcFile = new RpcFile(fileId, file, fileInfo);
		registerRpcFile(rpcFile, timeout, count);
		return rpcFile;
	}

	public void registerRpcFile(RpcFile rpcFile, Long timeout, Integer count) {
		Validate.notNull(rpcFile, "rpcFile=null");
		OutgoingFile.validateFile(rpcFile.getFile());
		synchronized (registeredFiles) {
			registeredFiles.put(rpcFile.getId(), new RegistryEntry(rpcFile, count));
			createOrDestroyTimeoutTimer();
			createNewTimoutTask(rpcFile.getId(), timeout);
		}
	}

	public RpcFile getFile(String fileId) {
		synchronized (registeredFiles) {
			RegistryEntry entry = registeredFiles.get(fileId);
			if (entry == null) {
				return null;
			}
			if (entry.isToBeRemoved()) {
				registeredFiles.remove(fileId);
			}
			return entry.getRpcFile();
		}
	}

	public RpcFile unregisterFile(String fileId) {
		synchronized (registeredFiles) {
			RegistryEntry entry = removeRegisteredFileAndCreateOrDestroyTimer(fileId);
			if (entry == null) {
				return null;
			}
			return entry.getRpcFile();
		}
	}

	private void createNewTimoutTask(String fileId, Long timeout) {
		if (timeout != null) {
			TimeoutTask timeoutTask = new TimeoutTask(fileId);
			if (timeoutTimer != null) {
				timeoutTimer.schedule(timeoutTask, timeout);
			}
		}
	}

	private RegistryEntry removeRegisteredFileAndCreateOrDestroyTimer(String fileId) {
		RegistryEntry ret = null;
		ret = registeredFiles.remove(fileId);
		createOrDestroyTimeoutTimer();
		return ret;
	}

	class TimeoutTask extends TimerTask {
		private final String fileId;

		public TimeoutTask(String fileId) {
			this.fileId = fileId;
		}

		@Override
		public void run() {
			synchronized (registeredFiles) {
				RegistryEntry entry = removeRegisteredFileAndCreateOrDestroyTimer(fileId);
				if (entry != null) {
					eventsSupport.getEventsDistributor().fileRemvedByTimeout(entry.getRpcFile(),
						entry.getCount());
				}
			}
			this.cancel();
		}
	}

	private static class RegistryEntry {
		private final RpcFile rpcFile;
		private Integer count;

		public RegistryEntry(RpcFile rpcFile, Integer count) {
			this.rpcFile = rpcFile;
			this.count = count;
		}

		public RpcFile getRpcFile() {
			return rpcFile;
		}

		public Integer getCount() {
			return count;
		}

		public boolean isToBeRemoved() {
			if (count == null) {
				return false;
			} else {
				return --count <= 0;
			}
		}
	}

	public void addFileTimeoutListener(FileTimeoutListener listener) {
		eventsSupport.subscribeEvents(listener);
	}

	public void removeFileTimeoutListener(FileTimeoutListener listener) {
		eventsSupport.unsubscribeEvents(listener);
	}

	public static interface FileTimeoutListener {
		void fileRemvedByTimeout(RpcFile rpcFile, Integer remainingCount);
	}
}
