/*
 * $Id$
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on Sep 29, 2008 by may
 * 
 * $HeadURL$
 * 
 * $LastChangedDate$ $LastChangedBy$ $LastChangedRevision$
 */
package de.starface.com.rpc.services.filetransfer.service;

import de.starface.com.rpc.common.events.EventsProviderSupport;
import de.starface.com.rpc.services.filetransfer.FileInfo;
import de.starface.com.rpc.services.filetransfer.FileTransfer.HashMethod;
import de.starface.com.rpc.services.filetransfer.FileTransfer.State;
import de.starface.com.rpc.services.filetransfer.FileTransferEventListener;
import de.starface.com.rpc.services.filetransfer.FileTransferFailedException;
import de.starface.com.rpc.services.filetransfer.RpcFile;
import de.starface.com.rpc.services.filetransfer.common.CompleteUploadResult;
import de.starface.com.rpc.services.filetransfer.common.ContinueUploadResult;
import de.starface.com.rpc.services.filetransfer.common.FileBlock;
import de.starface.com.rpc.services.filetransfer.common.FileEnd;
import de.starface.com.rpc.services.filetransfer.common.IncomingFileTransfer;
import de.starface.com.rpc.services.filetransfer.common.InitializeUploadResult;
import de.starface.com.rpc.services.filetransfer.common.RpcFileUpload;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Implementation of the server specific part of RPC file upload.
 *
 * @author tilmann
 */
public class RpcFileUploadImpl implements RpcFileUpload {
	private static final Log log = LogFactory.getLog(RpcFileUploadImpl.class);

	private final EventsProviderSupport<FileTransferEventListener> eventsSupport = new EventsProviderSupport<>(
		FileTransferEventListener.class);

	private final FileTransferRegistry<IncomingFileTransfer> fileTransferRegistry = new FileTransferRegistry<>();

	private final long timeout;

	public RpcFileUploadImpl(long timeout) {
		this.timeout = timeout;
	}

	public void addEventListener(FileTransferEventListener listener) {
		eventsSupport.subscribeEvents(listener);
	}

	public void removeEventListener(FileTransferEventListener listener) {
		eventsSupport.unsubscribeEvents(listener);
	}

	/**
	 * @see de.starface.com.rpc.services.filetransfer.common.RpcFileUpload#initializeUpload(de.starface.com.rpc.services.filetransfer.FileInfo,
	 *      de.starface.com.rpc.services.filetransfer.FileTransfer.HashMethod)
	 */
	public InitializeUploadResult initializeUpload(FileInfo fileInfo, HashMethod hashMethod) {
		// assemble incoming file information
		RpcFile rpcFile = new RpcFile(fileInfo);

		IncomingFileTransfer incomingFileTransfer = new IncomingFileTransfer(rpcFile.getId(), rpcFile, hashMethod,
			eventsSupport.getEventsDistributor(), true);

		try {
			incomingFileTransfer.initializeTransfer();
			fileTransferRegistry.registerFileTransfer(incomingFileTransfer, timeout);
		} catch (FileTransferFailedException e) {
			log.info("File upload failed.", e);
		}
		return new InitializeUploadResult(incomingFileTransfer.getId(), incomingFileTransfer.getState());
	}

	/**
	 * @see de.starface.com.rpc.services.filetransfer.common.RpcFileUpload#continueUpload(String,
	 *      de.starface.com.rpc.services.filetransfer.common.FileBlock)
	 */
	public ContinueUploadResult continueUpload(String fileTransferId, FileBlock fileBlock) {
		IncomingFileTransfer incomingFileTransfer = fileTransferRegistry.getFileTransfer(fileTransferId);

		if (incomingFileTransfer == null) {
			return new ContinueUploadResult(State.TIMEOUT);
		}

		try {
			incomingFileTransfer.receiveBlock(fileBlock);
		} catch (FileTransferFailedException e) {
			log.info("File upload failed.", e);
		}

		updateRegistry(incomingFileTransfer);
		return new ContinueUploadResult(incomingFileTransfer.getState());

	}

	/**
	 * @see de.starface.com.rpc.services.filetransfer.common.RpcFileUpload#completeUpload(String,
	 *      de.starface.com.rpc.services.filetransfer.common.FileEnd)
	 */
	public CompleteUploadResult completeUpload(String fileTransferId, FileEnd fileEnd) {
		IncomingFileTransfer incomingFileTransfer = fileTransferRegistry.getFileTransfer(fileTransferId);

		if (incomingFileTransfer == null) {
			return new CompleteUploadResult(State.TIMEOUT);
		}

		try {
			incomingFileTransfer.completeTransfer(fileEnd);
		} catch (FileTransferFailedException e) {
			log.info("File upload failed.", e);
		}

		updateRegistry(incomingFileTransfer);
		return new CompleteUploadResult(incomingFileTransfer.getState());
	}

	/**
	 * @see de.starface.com.rpc.services.filetransfer.common.RpcFileUpload#cancelUpload(String)
	 */
	public void cancelUpload(String fileTransferId) {
		IncomingFileTransfer incomingFileTransfer = fileTransferRegistry.getFileTransfer(fileTransferId);

		if (incomingFileTransfer != null) {
			incomingFileTransfer.cancelFromClientAndSendEvent();
			updateRegistry(incomingFileTransfer);
		}
	}

	private void updateRegistry(IncomingFileTransfer incomingFileTransfer) {
		if (incomingFileTransfer.getState() != State.IN_PROGRESS) {
			fileTransferRegistry.unregisterFileTransfer(incomingFileTransfer);
		}
	}
}
