/*
 * $Id$
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on Mar 26, 2008 by may
 * 
 * $HeadURL$
 * 
 * $LastChangedDate$ $LastChangedBy$ $LastChangedRevision$
 */
package de.starface.com.rpc.xmlrpc;

import org.jivesoftware.smack.util.Base64;
import org.xmlpull.v1.XmlPullParser;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 * Basic parser implementation to retrieve object representation of the XML-RPC types following the
 * XML-RPC specification. The different build results are created using an
 * {@link XmlRpcObjectBuilder} and the Builder Pattern.
 *
 * @author may
 */
public class XmlRpcObjectParser {

	private final XmlRpcObjectBuilder objectBuilder;

	private final String lastElementName;

	public XmlRpcObjectParser(XmlRpcObjectBuilder objectBuilder, String lastElementName) {
		this.objectBuilder = objectBuilder;
		this.lastElementName = lastElementName;
	}

	public void parse(XmlPullParser parser) throws Exception {
		boolean done = false;

		while (!done) {
			int eventType = parser.next();
			if (eventType == XmlPullParser.START_TAG) {
				if (parser.getName().equals("methodCall")) {
					parseMethodCall(parser);
				} else if (parser.getName().equals("methodResponse")) {
					parseMethodResponse(parser);
				} else {
					throw new IllegalArgumentException("RPC was neither call nor response");
				}
			} else if (eventType == XmlPullParser.END_DOCUMENT) {
				done = true;
			} else if (lastElementName != null && eventType == XmlPullParser.END_TAG
				&& lastElementName.equals(parser.getName())) {
				done = true;
			}
		}
	}

	private void parseMethodCall(XmlPullParser parser) throws Exception {
		boolean done = false;
		String method = null;
		Object[] o = null;

		while (!done) {
			int eventType = parser.next();
			if (eventType == XmlPullParser.START_TAG) {
				if (parser.getName().equals("methodName")) {
					method = parser.nextText();
				}
				if (parser.getName().equals("params")) {
					o = parseParams(parser);
				}

			} else if (eventType == XmlPullParser.END_TAG) {
				if (parser.getName().equals("methodCall")) {
					objectBuilder.createIncomingRequest(method, o);
					done = true;
				}
			}
		}
	}

	private void parseMethodResponse(XmlPullParser parser) throws Exception {
		boolean done = false;
		Object[] o = null;

		while (!done) {
			int eventType = parser.next();
			if (eventType == XmlPullParser.START_TAG) {
				if (parser.getName().equals("fault")) {
					parseFault(parser);
				} else if (parser.getName().equals("params")) {
					o = XmlRpcObjectParser.parseParams(parser);
					if (o == null || o.length != 1) {
						// responses are supposed to have exactly one parameter 
						throw new IllegalArgumentException("malorganized response");
					}
					objectBuilder.createIncomingResponse(o[0]);
				}
			} else if (eventType == XmlPullParser.END_TAG) {
				if (parser.getName().equals("methodResponse")) {
					done = true;
				}
			}
		}
	}

	private void parseFault(XmlPullParser parser) throws Exception {
		boolean done = false;

		while (!done) {
			int eventType = parser.next();
			if (eventType == XmlPullParser.START_TAG) {
				if (parser.getName().equals("value")) {
					Object m = XmlRpcObjectParser.parseValue(parser);
					if (m instanceof Map<?, ?>) {
						int code = (Integer) ((Map<?, ?>) m).get("faultCode");
						String string = (String) ((Map<?, ?>) m).get("faultString");
						objectBuilder.createIncomingFaultResponse(code, string);
					}
				}
			} else if (eventType == XmlPullParser.END_TAG) {
				if (parser.getName().equals("fault")) {
					done = true;
				}
			}
		}
	}

	private static Object[] parseParams(XmlPullParser parser) throws Exception {
		ArrayList<Object> objects = new ArrayList<>();

		boolean done = false;
		boolean param = false;

		while (!done) {
			int eventType = parser.next();
			if (eventType == XmlPullParser.START_TAG) {
				if (parser.getName().equals("param")) {
					if (param) {
						throw new IllegalArgumentException("Malformattet params");
					} else {
						param = true;
					}
				} else if (parser.getName().equals("value")) {
					objects.add(parseValue(parser));
				}
			} else if (eventType == XmlPullParser.END_TAG) {
				if (parser.getName().equals("param")) {
					param = false;
				}
				if (parser.getName().equals("params")) {
					done = true;
				}
			}
		}
		return objects.toArray();
	}

	public static Object parseValue(XmlPullParser parser) throws Exception {
		Object o = null;

		boolean done = false;

		String lasttext = null;

		while (!done) {
			int eventType = parser.next();
			if (eventType == XmlPullParser.TEXT) {
				lasttext = parser.getText();
			}
			if (eventType == XmlPullParser.START_TAG) {
				if (parser.getName().equals("int")) {
					o = Integer.parseInt(parser.nextText());
				} else if (parser.getName().equals("i4")) {
					o = Integer.parseInt(parser.nextText());
				} else if (parser.getName().equals("double")) {
					o = Double.parseDouble(parser.nextText());
				} else if (parser.getName().equals("boolean")) {
					o = parseBoolean(parser.nextText());
				} else if (parser.getName().equals("dateTime.iso8601")) {
					o = parseDate(parser.nextText());
				} else if (parser.getName().equals("base64")) {
					o = Base64.decode(parser.nextText());
				} else if (parser.getName().equals("struct")) {
					o = parseStruct(parser);
				} else if (parser.getName().equals("array")) {
					o = parseArray(parser);
				} else if (parser.getName().equals("string")) {
					o = parser.nextText();
				}
			} else if (eventType == XmlPullParser.END_TAG) {
				// without type information, the type is assumed to be string
				if (parser.getName().equals("value")) {
					done = true;
					if (o == null) {
						o = lasttext;
					}
					// if still null, something is wrong!
					if (o == null) {
						o = "";
						//
						// throw new
						// IllegalArgumentException("A value without a value is hardly a value at all!");
					}
				}
			}
		}

		return o;
	}

	private static Object parseStruct(XmlPullParser parser) throws Exception {
		Map<String, Object> o = new HashMap<>();

		boolean done = false;
		String key = null;
		Object value = null;

		while (!done) {
			int eventType = parser.next();
			if (eventType == XmlPullParser.START_TAG) {
				if (parser.getName().equals("name")) {
					if (key != null) {
						// we got two keys in a row
						throw new IllegalArgumentException("malorganized struct");
					} else {
						key = parser.nextText();
					}
				} else if (parser.getName().equals("value")) {
					if (key != null && value == null) {
						value = parseValue(parser);
						o.put(key, value);
						// we got a key-value-pair, now set next key to null
						key = null;
						value = null;
					} else {
						// we got a value without key
						throw new IllegalArgumentException("malorganized struct");
					}
				}
			} else if (eventType == XmlPullParser.END_TAG) {
				if (parser.getName().equals("struct")) {
					done = true;
				}
			}
		}

		return o;
	}

	private static Object parseArray(XmlPullParser parser) throws Exception {
		List<Object> o = new LinkedList<>();

		boolean done = false;
		boolean data = false;

		while (!done) {
			int eventType = parser.next();
			if (eventType == XmlPullParser.START_TAG) {
				if (parser.getName().equals("data")) {
					if (data) {
						// nested data not allowed in an array
						throw new IllegalArgumentException("Malformattet array");
					} else {
						data = true;
					}
				} else if (parser.getName().equals("value")) {
					o.add(parseValue(parser));
				}
			} else if (eventType == XmlPullParser.END_TAG) {
				if (parser.getName().equals("array")) {
					done = true;
				}
			}
		}

		return o;
	}

	private static boolean parseBoolean(final String aString) throws ParseException {
		if (aString == null) {
			return false;
		}
		String value = aString.trim();

		boolean ret;
		if (value.equalsIgnoreCase("true") || value.equalsIgnoreCase("1")) {
			ret = true;
		} else if (value.equalsIgnoreCase("false") || value.equalsIgnoreCase("0")) {
			ret = false;
		} else {
			throw new ParseException("Invalid XML-RPC boolean", 0);
		}
		return ret;
	}

	public static Date parseDate(final String aString) throws ParseException {
		String value = aString.trim();

		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
		try {
			return sdf.parse(value);
		} catch (ParseException pe) {
			sdf = new SimpleDateFormat("yyyyMMdd'T'HH:mm:ss");
			return sdf.parse(value);
		}
	}
}
