/*
 * $Id$
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on Jan 31, 2008 by may
 * 
 * $HeadURL$
 * 
 * $LastChangedDate$ $LastChangedBy$ $LastChangedRevision$
 */
package de.starface.com.rpc.xmlrpc.http;

import de.starface.com.rpc.common.RpcTransportToken;

import org.apache.commons.httpclient.NameValuePair;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.Validate;

import java.io.Serializable;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;

/**
 * Implementation of a {@link RpcTransportToken} for XML-RPC via HTTP
 *
 * @author may
 *
 * @see RpcTransportToken
 */
public class URL implements RpcTransportToken, Cloneable, Serializable {

	private static final long serialVersionUID = 1L;
	public static final String KEY_CALLBACK_TYPE = "de.vertico.starface.callback.type";
	public static final String KEY_CALLBACK_HOST = "de.vertico.starface.callback.host";
	public static final String KEY_CALLBACK_PORT = "de.vertico.starface.callback.port";
	public static final String KEY_CALLBACK_PATH = "de.vertico.starface.callback.path";
	public static final String KEY_USER = "de.vertico.starface.user";
	public static final String KEY_AUTH = "de.vertico.starface.auth";

	private ArrayList<NameValuePair> _args = new ArrayList<>();

	private int _port = 80;
	private String _host = "";
	private String _path = "/";
	private Protocol _type = Protocol.http;

	/**
	 * Identification for the used Protocol. allows to choose between
	 *
	 * @author may
	 *
	 */
	public static enum Protocol {
		/** HTTP as the default protocol without SSL */
		http,
		/** HTTPS with strict certificates checks */
		https,
		/** HTTPS with allowance for self-singed certificates */
		ehttps;
	}

	/**
	 * Builds an URL with the given parameters.
	 *
	 * @param type
	 *        "http" when parameter {@code null}
	 * @param host
	 *        "127.0.0.1" when parameter {@code null}
	 * @param port
	 *        "80" when parameter {@code null}
	 * @param path
	 *        "/" when parameter {@code null}
	 */
	public URL(Protocol type, String host, Integer port, String path) {
		// default values if no params are provided
		if (type != null) {
			_type = type;
		}
		if (host != null) {
			_host = host;
		}
		if (port != null) {
			_port = port;
		}
		if (path != null) {
			_path = path;
		}
	}

	/**
	 * Parses an URL from the given String
	 *
	 * @param url
	 *        the URL to be parsed
	 * @throws IllegalArgumentException
	 *         if the syntax of the given String is not a valid URL
	 */
	public URL(String url) {
		java.net.URL javaUrl;
		try {
			javaUrl = new java.net.URL(url);
		} catch (MalformedURLException e) {
			throw new IllegalArgumentException(url + " is not a valid URL");
		}
		String protocol = javaUrl.getProtocol();
		Validate.notEmpty(protocol, "protocol=empty");
		_type = Protocol.valueOf(protocol);
		Validate.notNull(_type, "protocol=invalid");
		_host = javaUrl.getHost();
		Validate.notEmpty(_host, "host=empty");
		_path = javaUrl.getPath();
		_port = javaUrl.getPort();
		if (_port == -1) {
			_port = javaUrl.getDefaultPort();
		}
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((_type == null) ? 0 : _type.hashCode());
		result = prime * result + ((_path == null) ? 0 : _path.hashCode());
		result = prime * result + ((_host == null) ? 0 : _host.hashCode());
		result = prime * result + _port;
		String auth = getFirstParamWithName(URL.KEY_AUTH);
		result = prime * result + ((auth == null) ? 0 : auth.hashCode());
		String user = getFirstParamWithName(URL.KEY_USER);
		result = prime * result + ((user == null) ? 0 : user.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final URL other = (URL) obj;
		if (_type == null) {
			if (other._type != null) {
				return false;
			}
		} else if (!_type.equals(other._type)) {
			return false;
		}
		if (_path == null) {
			if (other._path != null) {
				return false;
			}
		} else if (!_path.equals(other._path)) {
			return false;
		}
		if (_host == null) {
			if (other._host != null) {
				return false;
			}
		} else if (!_host.equals(other._host)) {
			return false;
		}
		if (_port != other._port) {
			return false;
		}
		if (!StringUtils.equals(getFirstParamWithName(KEY_AUTH), other.getFirstParamWithName(KEY_AUTH))) {
			return false;
		}
		if (!StringUtils.equals(getFirstParamWithName(KEY_USER), other.getFirstParamWithName(KEY_USER))) {
			return false;
		}
		return true;
	}

	/**
	 * Factory method used in the {@link XmlRpcServlet} to parse the basic properties host, port and
	 * endpointURI from the callback address set in the incoming request. Basically tries to resolve
	 * the static keys from the parameter map.
	 *
	 * @param params
	 * @return A new URL instance pointing to the callback address
	 */
	public static URL createCallbackUrlFromRequestParameters(Map<?, ?> params) {
		return new URL(params);
	}

	/**
	 * Constructor used to parse the basic properties host, port and endpointURI from the incoming
	 * request. Basically tries to resolve the static keys from the parameter map.
	 *
	 * @param params
	 */
	private URL(Map<?, ?> params) {
		if (params == null) {
			// default values if no params are provided
			return;
		}

		for (Map.Entry<?, ?> entry : params.entrySet()) {
			if (entry.getValue() instanceof String[]) {
				for (String value : (String[]) entry.getValue()) {
					if (entry.getKey().equals(KEY_CALLBACK_TYPE)) {
						try {
							_type = Protocol.valueOf(value);
						} catch (Exception e) {
							_type = Protocol.http;
						}
					} else if (entry.getKey().equals(KEY_CALLBACK_HOST)) {
						_host = value;
					} else if (entry.getKey().equals(KEY_CALLBACK_PORT)) {
						try {
							_port = new Integer(value);
						} catch (Exception e) {
							_port = 80;
						}
					} else if (entry.getKey().equals(KEY_CALLBACK_PATH)) {
						_path = value;
					} else if (entry.getKey().equals(KEY_USER)) {
						addParam(entry.getKey().toString(), value);
					} else if (entry.getKey().equals(KEY_AUTH)) {
						addParam(entry.getKey().toString(), value);
					}
				}
			}
		}
	}

	/**
	 * Resolves a parameter from the paremeter map;
	 *
	 * @param paramName
	 *        name of the parameter
	 * @return the value if found
	 * @throws IllegalArgumentException
	 *         if paramName is {@code null}
	 */
	public String getFirstParamWithName(String paramName) throws IllegalArgumentException {
		Validate.notEmpty(paramName, "paramName=empty");
		String value = null;
		for (NameValuePair pair : _args) {
			if (paramName.equals(pair.getName())) {
				value = pair.getValue();
			}
		}
		return value;
	}

	public void removeAllParamsWithName(String paramName) throws IllegalArgumentException {
		Validate.notEmpty(paramName, "paramName=empty");
		Iterator<NameValuePair> iter = _args.iterator();
		while (iter.hasNext()) {
			NameValuePair pair = iter.next();
			if (paramName.equals(pair.getName())) {
				iter.remove();
			}
		}
	}

	/**
	 * @return Returns the header.
	 */
	protected NameValuePair[] getParams() {
		return _args.toArray(new NameValuePair[_args.size()]);
	}

	/**
	 * @param key
	 * @param value
	 */
	public void addParam(String key, String value) {
		_args.add(new NameValuePair(key, value));
	}

	public void clearParams() {
		_args.clear();
	}

	/**
	 * @return Returns the protocol.
	 */
	public Protocol getType() {
		return _type;
	}

	/**
	 * @param protocol
	 *        The protocol to set.
	 */
	public void setType(Protocol protocol) {
		_type = protocol;
	}

	/**
	 * @return Returns the port.
	 */
	public int getPort() {
		return _port;
	}

	/**
	 * @param port
	 *        The port to set.
	 */
	public void setPort(int port) {
		_port = port;
	}

	/**
	 * @return Returns the host.
	 */
	public String getHost() {
		return _host;
	}

	/**
	 * @param host
	 *        The host to set.
	 */
	public void setHost(String host) {
		_host = host;
	}

	/**
	 * @return Returns the path.
	 */
	public String getPath() {
		return _path;
	}

	/**
	 * @param path
	 *        The path to set.
	 */
	public void setPath(String path) {
		_path = path;
	}

	@Override
	public String toString() {
		StringBuilder url = new StringBuilder(_type.name() + "://" + _host + ":" + _port + _path);
		String separator = "?";
		for (NameValuePair arg : _args) {
			url.append(separator).append(arg.getName()).append("=").append(arg.getValue());
			separator = "&";
		}
		return url.toString();
	}

	@SuppressWarnings("unchecked")
	@Override
	public URL clone() {
		URL url = new URL(_type, _host, _port, _path);
		url._args = (ArrayList<NameValuePair>) _args.clone();
		return url;
	}

	/**
	 * Checks whether this URL has a callback URL set
	 *
	 * @return <code>true</code> if set, <code>false</otherwise>
	 */
	public boolean hasCallbackUrl() {
		return getFirstParamWithName(KEY_CALLBACK_HOST) != null
			&& getFirstParamWithName(KEY_CALLBACK_PATH) != null
			&& getFirstParamWithName(KEY_CALLBACK_PORT) != null
			&& getFirstParamWithName(KEY_CALLBACK_TYPE) != null;
	}

	/**
	 * Removes the callback URL set in this URL
	 */
	public void removeCallbackUrl() {
		removeAllParamsWithName(KEY_CALLBACK_TYPE);
		removeAllParamsWithName(KEY_CALLBACK_HOST);
		removeAllParamsWithName(KEY_CALLBACK_PORT);
		removeAllParamsWithName(KEY_CALLBACK_PATH);
	}

	/**
	 * Sets the given URL as a callback URL in this URL. Also sets Auth and User from the given URL
	 * if present.
	 *
	 * @param callbackServerToken
	 *        the callback URL to be set
	 */
	public void setCallbackUrl(URL callbackServerToken) {
		if (hasCallbackUrl()) {
			removeCallbackUrl();
		}
		addParam(KEY_CALLBACK_TYPE, callbackServerToken._type.name());
		addParam(KEY_CALLBACK_HOST, callbackServerToken._host);
		addParam(KEY_CALLBACK_PORT, Integer.toString(callbackServerToken._port));
		addParam(KEY_CALLBACK_PATH, callbackServerToken._path);
		String user = callbackServerToken.getUser();
		if (user != null) {
			setUser(user);
		}
		String auth = callbackServerToken.getAuth();
		if (auth != null) {
			setAuth(auth);
		}
	}

	/**
	 * Gets the STARFACE user parameter from this URL
	 *
	 * @return the user parameter or <code>null</code> if not set
	 */
	public String getUser() {
		return getFirstParamWithName(KEY_USER);
	}

	/**
	 * Sets the STARFACE user parameter on this URL
	 *
	 * @param user
	 *        the new user parameter
	 */
	public void setUser(String user) {
		removeAllParamsWithName(KEY_USER);
		if (user != null) {
			addParam(KEY_USER, user);
		}
	}

	/**
	 * Checks whether this URL as a auth parameter set
	 *
	 * @return <code>true</code> if set, <code>false</otherwise>
	 */
	public boolean hasAuth() {
		return getFirstParamWithName(KEY_AUTH) != null;
	}

	/**
	 * Checks whether this URL as a user parameter set
	 *
	 * @return <code>true</code> if set, <code>false</otherwise>
	 */
	public boolean hasUser() {
		return getFirstParamWithName(KEY_USER) != null;
	}

	/**
	 * Sets the STARFACE auth parameter on this URL
	 *
	 * @param auth
	 *        the new auth parameter
	 */
	public void setAuth(String auth) {
		removeAllParamsWithName(KEY_AUTH);
		if (auth != null) {
			addParam(KEY_AUTH, auth);
		}
	}

	/**
	 * Gets the STARFACE auth parameter from this URL
	 *
	 * @return the auth parameter or <code>null</code> if not set
	 */
	public String getAuth() {
		return getFirstParamWithName(KEY_AUTH);
	}

	public String getIdentifierString() {
		StringBuilder url = new StringBuilder(_type.name() + "://" + _host + ":" + _port + _path);
		String separator = "?";
		for (NameValuePair arg : _args) {
			url.append(separator).append(arg.getName()).append("=").append(arg.getValue());
			separator = "&";
		}
		return url.toString();
	}
}
