/*
 * $Id: XmlRpcResponsePacket.java 422 2011-10-28 18:29:03Z tilmann $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on Nov 6, 2007 by may
 * 
 * $HeadURL:
 * http://192.168.2.2/repos/verticomisc/xmpp-xml-rpc/src/de/vertico/com/xmpp/smack/XmlRpcResponsePacket
 * .java $
 * 
 * $LastChangedDate: 2011-10-28 20:29:03 +0200 (Fri, 28 Oct 2011) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 422 $
 */
/**
 *
 */
package de.starface.com.rpc.xmlrpc.smack;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.xmlrpc.XmlRpcObjectSerializer;
import de.starface.com.rpc.xmlrpc.XmlRpcResponse;

import org.jivesoftware.smack.packet.IQ;

import java.util.HashMap;

/**
 * An XMPP-XML-RPC-Packet as specified in XEP-0009 <a
 * href="http://www.xmpp.org/extensions/xep-0009.html"
 * >http://www.xmpp.org/extensions/xep-0009.html</a>
 *
 * This packet represents the method response.
 *
 * @author may
 *
 */
public class XmlRpcResponsePacket extends IQ implements XmlRpcResponse {

	private final Object _result;
	private String _serializedResult;
	private final boolean _fault;

	private String _faultMessage;
	private int _faultCode;

	public static XmlRpcResponsePacket createOutgoingResponse(Object response) throws RpcException {
		XmlRpcResponsePacket packet = new XmlRpcResponsePacket(response);
		try {
			packet.serializeResult();
		} catch (Exception e) {
			throw new RpcException(RpcException.Codes.Server.XML_SERIALIZATION_ERROR,
				"Could not serialize return value of RPC call to XML-RPC.", e);
		}
		return packet;
	}

	public static XmlRpcResponsePacket createIncomingResponse(Object response) {
		return new XmlRpcResponsePacket(response);
	}

	public static XmlRpcResponsePacket createOutgoingFaultResponse(int faultCode, String faultString) {
		XmlRpcResponsePacket packet = new XmlRpcResponsePacket(faultCode, faultString);
		packet.serializeResult();
		return packet;
	}

	public static XmlRpcResponsePacket createIncomingFaultResponse(int faultCode, String faultString) {
		return new XmlRpcResponsePacket(faultCode, faultString);
	}

	/**
	 * Constructor for a valid method response as specified in <a
	 * href="http://www.xmlrpc.com/spec">http://www.xmlrpc.com/spec</a>
	 *
	 * @param response
	 *        The result. See {@link XmlRpcObjectSerializer} for supported Object types
	 */
	private XmlRpcResponsePacket(Object response) {
		if (response == null) {
			throw new IllegalArgumentException("XmlRpc does not allow the response to be null!");
		}

		_fault = false;
		_result = response;
		setType(IQ.Type.RESULT);
	}

	private void serializeResult() {
		if (_result != null) {
			_serializedResult = XmlRpcObjectSerializer.serialize(_result);
		}
	}

	/**
	 * Constructor for a faulty method response as specified in <a
	 * href="http://www.xmlrpc.com/spec">http://www.xmlrpc.com/spec</a>
	 *
	 * @param faultCode
	 *        The fault code
	 * @param faultString
	 *        The fault string
	 */
	private XmlRpcResponsePacket(int faultCode, String faultString) {
		if (faultString == null) {
			faultString = "";
		}
		_fault = true;
		HashMap<String, Object> faultMap = new HashMap<>();
		faultMap.put("faultCode", Integer.valueOf(faultCode));
		faultMap.put("faultString", faultString);
		_result = faultMap;
		_faultCode = faultCode;
		_faultMessage = faultString;
		setType(IQ.Type.RESULT);
	}

	/**
	 * @return The query
	 */
	@Override
	public String getChildElementXML() {
		StringBuilder sb = new StringBuilder();
		if (_fault) {
			if (_serializedResult == null) {
				serializeResult();
			}
			sb.append("<query xmlns='jabber:iq:rpc'>");
			sb.append("<methodResponse>");
			sb.append("<fault>");
			sb.append(_serializedResult);
			sb.append("</fault>");
			sb.append("</methodResponse>");
			sb.append("</query>");
		} else {
			sb.append("<query xmlns='jabber:iq:rpc'>");
			sb.append("<methodResponse>");
			if (_result != null) {
				if (_serializedResult == null) {
					serializeResult();
				}
				sb.append("<params>");
				sb.append("<param>");
				sb.append(_serializedResult);
				sb.append("</param>");
				sb.append("</params>");
			}
			sb.append("</methodResponse>");
			sb.append("</query>");
		}
		return sb.toString();
	}

	/**
	 * @return Returns the result.
	 */
	public Object getResult() {
		return _result;
	}

	/**
	 * @return Returns the fault.
	 */
	public boolean isFault() {
		return _fault;
	}

	/**
	 * @return Returns the faultMessage.
	 */
	public String getFaultMessage() {
		return _faultMessage;
	}

	/**
	 * @return Returns the faultCode.
	 */
	public int getFaultCode() {
		return _faultCode;
	}
}
