/*
 * $Id: XmppXmlRpcServer.java 440 2012-05-04 16:49:10Z tilmann $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on Nov 12, 2007 by may
 * 
 * $HeadURL:
 * http://192.168.2.2/repos/verticomisc/xmpp-xml-rpc/src/de/vertico/com/xmlrpc/smack/XmppXmlRpcServer
 * .java $
 * 
 * $LastChangedDate: 2012-05-04 18:49:10 +0200 (Fri, 04 May 2012) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 440 $
 */
package de.starface.com.rpc.xmlrpc.smack;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.common.RpcRequest;
import de.starface.com.rpc.common.RpcThreadFactory;
import de.starface.com.rpc.common.RpcTransportToken;
import de.starface.com.rpc.common.authentication.RpcAuthTokenConverter;
import de.starface.com.rpc.server.RequestExecutor;
import de.starface.com.rpc.server.RpcServer;
import de.starface.com.rpc.server.RpcServerBase;
import de.starface.com.rpc.xmlrpc.XmlRpcRequestExecutor;
import de.starface.com.xmpp.smack.XmlRpcPacketProvider;
import de.starface.com.xmpp.smack.XmppConnectionException;
import de.starface.com.xmpp.smack.XmppConnector;

import org.apache.commons.lang3.Validate;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jivesoftware.smack.Connection;
import org.jivesoftware.smack.PacketListener;
import org.jivesoftware.smack.filter.AndFilter;
import org.jivesoftware.smack.filter.IQTypeFilter;
import org.jivesoftware.smack.filter.PacketTypeFilter;
import org.jivesoftware.smack.packet.IQ;
import org.jivesoftware.smack.packet.Packet;
import org.jivesoftware.smack.provider.ProviderManager;

import java.util.Arrays;
import java.util.List;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.RejectedExecutionException;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import com.google.common.annotations.VisibleForTesting;
import com.google.common.cache.CacheBuilder;
import com.google.common.cache.CacheLoader;
import com.google.common.cache.LoadingCache;
import com.google.common.cache.RemovalListener;

/**
 * Implementation of {@link RpcServer} which uses the SMACK XMPP library and XML-RPC for
 * transport.
 *
 * <p>
 * The aim of this server is to process incoming remote method calls. This implementation uses the
 * Smack XMPP API to process incoming request via XMPP. A {@link RpcRequest} uses the {@link JID} of
 * the incoming XMPP-packet as {@link RpcTransportToken} to identify the caller. This can be used
 * for authentication by registering a corresponding {@link RpcAuthTokenConverter} via
 * {@link #enableAuthentication(RpcAuthTokenConverter)}.
 * </p>
 *
 * <p>
 * This implementation currently does not allow to set an explicit processor registry for call
 * processors. The request executor will use the default registry
 * {@link de.starface.com.rpc.server.RpcObjectRegistry#DEFAULT_REGISTRY} instead.
 * </p>
 *
 * <p>
 * Features:
 * <ul>
 * <li>auto-reconnecting of the XMPP connection</li>
 * </ul>
 * </p>
 *
 * @author tilmann, may
 *
 * @see XmlRpcRequestExecutor
 * @see RpcAuthTokenConverter
 * @see RpcServer
 */
public class XmppXmlRpcServer extends RpcServerBase<JID> implements PacketListener {

	private static final Log log = LogFactory.getLog(XmppXmlRpcServer.class);
	public static final String XMPP_XML_RPC_SERVER_MAX_QUEUE_LENGTH = "XmppXmlRpcServer.maxQueueLength";

	static {
		ProviderManager.getInstance().addIQProvider("query", "jabber:iq:rpc", new XmlRpcPacketProvider());
	}

	class XmlRpcRequestHandler {
		private final RequestExecutor<JID> requestExecutor = new XmlRpcRequestExecutor<>();
		private final ThreadPoolExecutor executor;
		private final RpcThreadFactory threadFactory;
		private final String name;

		public XmlRpcRequestHandler(String name) {
			this.name = name;
			threadFactory = new RpcThreadFactory("AsynchronousExecutor[" + name + "]");
			int maxQueueLength = Integer.getInteger(XMPP_XML_RPC_SERVER_MAX_QUEUE_LENGTH, 30);
			executor = new ThreadPoolExecutor(0, 1, 30, TimeUnit.SECONDS,
				new LinkedBlockingQueue<>(maxQueueLength), threadFactory);
		}

		public void shutdown() {
			final List<Runnable> runnables = executor.shutdownNow();
			if (!runnables.isEmpty()) {
				log.warn("Shutdown executor for " + name + " with " + runnables.size() + " tasks in queue.");
			}
		}

		public void handle(XmlRpcRequestPacket packet) {
			try {
				if (executor.isTerminated()) {
					log.info(name + ": ignore packet due terminated executor ");
					return;
				}
				executor.execute(() -> {
					handleImpl(packet);
				});
			} catch (RejectedExecutionException e) {
				log.error(name + ": Request rejected: " + name + " " + executor + " " + e.getLocalizedMessage());
				final CharSequence stackTrace = threadFactory.printStackTracesOfLiveThreads();
				log.error("===============Thread dump==================");
				log.error(stackTrace);
				log.error("=============Thread dump end================");
			}
		}

		void handleImpl(XmlRpcRequestPacket packet) {
			String toJid = packet.getTo();
			String fromJid = packet.getFrom();
			String myJid = _connection.getUser();

			if (toJid != null && myJid != null && toJid.equals(myJid)) {
				Packet resp;

				requestExecutor.setServerObjectRegistry(getRegistry());
				requestExecutor.setAuthTokenConverter(getAuthTokenConverter());

				try {
					Object returnValue = requestExecutor.execute(packet.getMethodName(), packet.getParameters(), new JID(
						fromJid));

					resp = XmlRpcResponsePacket.createOutgoingResponse(returnValue);
					resp.setPacketID(packet.getPacketID());
					resp.setFrom(toJid);
					resp.setTo(fromJid);
				} catch (Exception e) {
					if (e.getCause() != null && "UcpNotConnectedException".equals(e.getCause().getClass().getSimpleName())) {
						if (log.isInfoEnabled()) {
							log.info(e.getMessage() + ": " + e.getCause().getMessage() + " Request was " + createRequestInfo(packet, fromJid));
						}
					} else {
						log.warn("The RPC procedure call " + createRequestInfo(packet, fromJid) + " could not be completed.", e);
					}

					RpcException rpcException = requestExecutor.convertExceptionToRpcException(e);
					resp = XmlRpcResponsePacket.createOutgoingFaultResponse(rpcException.getCode(),
						rpcException.getMessage());
					resp.setPacketID(packet.getPacketID());
					resp.setFrom(toJid);
					resp.setTo(fromJid);
				}
				if (log.isTraceEnabled()) {
					log.trace("XmlRpcRequestHandler.run: Sending xml-rpc result: " + resp.toXML());
				}
				_connection.sendPacket(resp);
			} else {
				log.warn("XmlRpcRequestHandler.run: Dropping xml-rpc call for wrong XMPP user. Expected for: "
					+ myJid + " Received for: " + toJid);
			}
		}

		private String createRequestInfo(XmlRpcRequestPacket packet, String fromJid) {
			return packet.getMethodName() + "(" + Arrays.toString(packet.getParameters()) + ") for " + fromJid;
		}
	}

	private XmppConnector _connector;

	private Connection _connection;

	private boolean myConnection = false;
	private final LoadingCache<String, XmlRpcRequestHandler> cache;

	/**
	 * Creates a new XmppXmlRpcServer using the given XMPP connection.
	 */
	@VisibleForTesting
	XmppXmlRpcServer(Connection connection) {
		Validate.notNull(connection, "connection=null");
		myConnection = false;
		_connection = connection;
		log.trace("constructor: Xml-rpc server created.");
		cache = buildCache();
	}

	@VisibleForTesting
	XmlRpcRequestHandler createXmlRpcRequestHandler(String key) {
		return new XmlRpcRequestHandler(key);
	}

	private XmppXmlRpcServer(String xmppUserName, String xmppResource, String xmppPassword,
							 String xmppServerHostName, int xmppServerPort, boolean xmppUseSSL,
							 boolean xmppAutoReconnect, boolean xmppSendPresence) {
		myConnection = true;
		_connector = new XmppConnector(xmppUserName, xmppResource, xmppPassword, xmppServerHostName,
			xmppServerPort, xmppUseSSL, xmppAutoReconnect, xmppSendPresence);
		_connection = _connector.getConnection();
		log.trace("constructor: Xml-rpc server created.");
		cache = buildCache();
	}

	public static XmppXmlRpcServer createWithExistingConnection(Connection connection) {
		return new XmppXmlRpcServer(connection);
	}

	public static XmppXmlRpcServer createWithNewConnection(String xmppUserName, String xmppResource,
														   String xmppPassword, String xmppServerHostName,
														   int xmppServerPort, boolean xmppUseSSL,
														   boolean xmppAutoReconnect, boolean xmppSendPresence) {
		return new XmppXmlRpcServer(xmppUserName, xmppResource, xmppPassword, xmppServerHostName, xmppServerPort,
			xmppUseSSL, xmppAutoReconnect, xmppSendPresence);
	}

	private LoadingCache<String, XmlRpcRequestHandler> buildCache() {
		return CacheBuilder.newBuilder()
			.expireAfterAccess(10, TimeUnit.MINUTES)
			.removalListener((RemovalListener<String, XmlRpcRequestHandler>) notification -> notification.getValue().shutdown())
			.build(new CacheLoader<String, XmlRpcRequestHandler>() {
				@Override
				public XmlRpcRequestHandler load(String key) throws Exception {
					if (log.isDebugEnabled()) {
						log.debug("Create new XmlRpcHandler for " + key);
					}
					return createXmlRpcRequestHandler(key);
				}
			});
	}

	public void processPacket(Packet packet) {
		if (packet instanceof XmlRpcRequestPacket) {
			if (log.isTraceEnabled()) {
				log.trace("XmppXmlRpcPacketListener.processPacket: Recieved xml-rpc call: " + packet.toXML());
			}
			cache.getUnchecked(packet.getFrom()).handle((XmlRpcRequestPacket) packet);
		}
	}
	/**
	 * {@inheritDoc}
	 *
	 * @see de.starface.com.rpc.server.RpcServerBase#shutdownImpl()
	 */
	@Override
	protected synchronized void shutdownImpl() {
		running = false;
		stopListening();
		cache.asMap().values().forEach(XmlRpcRequestHandler::shutdown);
		if (myConnection) {
			_connector.disconnect();
		}
	}

	/**
	 *
	 * Starts the server and tries to start listening to client calls. If the underlying XMPP
	 * connection is not established use asynchronous connection attempts.
	 *
	 * @throws RpcException
	 *         if the connection cannot be made on the first try
	 *
	 * @see de.starface.com.rpc.server.RpcServerBase#startupImpl()
	 */
	@Override
	protected void startupImpl() throws RpcException {
		try {
			startup(false);
		} catch (XmppConnectionException e) {
			throw new RpcException(RpcException.Codes.Server.TRANSPORT_FAILURE,
				"Could not connect to XMPP server on first try", e);
		}
	}

	/**
	 * Starts the server and tries to start listening to client calls. If the underlying XMPP
	 * connection is not established use asynchronous connection attempts.
	 */
	public void startupAsync() {
		try {
			startup(true);
		} catch (XmppConnectionException e) {
			// This cannot happen.
			log.warn("startupAsync: Asynchronous startup caught exception.", e);
		}
	}

	/**
	 * Starts the server and tries to start listening to client calls. If the underlying XMPP
	 * connection is not established use either synchronous or asynchronous connection attempts.
	 *
	 * @param async
	 *        use asynchronous connection attempts if set to <code>true</code>
	 */
	private synchronized void startup(boolean async) throws XmppConnectionException {
		if (!running) {
			XmppConnectionException ex = null;
			if (async) {
				log.debug("startup: Xml-rpc server starting in asynchronous mode...");
			} else {
				log.debug("startup: Xml-rpc server starting in regular mode...");
			}
			if (myConnection) {
				if (async) {
					_connector.startAsyncConnects();
				} else {
					try {
						_connector.startConnects();
					} catch (XmppConnectionException e) {
						ex = e;
					}
				}
			}
			startListening();
			running = true;
			if (ex != null) {
				throw ex;
			}
		} else {
			log.debug("startup: Xml-rpc server is allready running.");
		}
	}

	/**
	 * Starts listening to client requests, not starting or connecting the server
	 */
	private void startListening() {
		// only listen for valid requests (type set)
		_connection.addPacketListener(this, new AndFilter(new PacketTypeFilter(
			XmlRpcRequestPacket.class), new IQTypeFilter(IQ.Type.SET)));
	}

	/**
	 * Stops listening to client request, not stopping or disconnecting the server
	 */
	private void stopListening() {
		_connection.removePacketListener(this);
	}

	/**
	 * @return Returns the XMPP connection used by the server
	 */
	public Connection getConnection() {
		return _connection;
	}

	/**
	 * Returns the JID belonging to the underlying XMPP connection. This method may return null if
	 * the underlying connection has not been established yet.
	 *
	 * @return the JID of the XMPP connection
	 *
	 * @see de.starface.com.rpc.server.RpcServer#getServerAddress()
	 */
	public JID getServerAddress() {
		return JID.fetchFromConnection(_connection);
	}
}
