package de.starface.com.rpc.common.valuetranslation;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueObject;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static org.junit.jupiter.api.Assertions.assertArrayEquals;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertNull;
import static org.junit.jupiter.api.Assertions.assertTrue;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.spy;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;

public class ValueTranslatorTest {
	ValueTranslator sut;

	@BeforeEach
	public void setup() {
		sut = spy(ValueTranslator.class);
	}

	@Test
	public void testGetDefaultValue() {
		assertNull(sut.getDefaultValue(Object.class));

		assertEquals(new Date(0), sut.getDefaultValue(Date.class));
		assertEquals("", sut.getDefaultValue(Enum.class));
		assertEquals("", sut.getDefaultValue(String.class));
		assertEquals((Integer) 0, sut.getDefaultValue(Integer.class));
		assertEquals((Double) 0D, sut.getDefaultValue(Double.class));
		assertEquals(false, sut.getDefaultValue(Boolean.class));
		assertEquals(String.valueOf(0L), sut.getDefaultValue(Long.class));
		assertArrayEquals(new byte[0], (byte[]) sut.getDefaultValue(byte[].class));
		assertEquals(Collections.emptyMap(), sut.getDefaultValue(Map.class));
		assertEquals(Collections.emptyList(), sut.getDefaultValue(List.class));
	}

	@Test
	public void testJavaObjectToRpcMapWithNulls() {
		TestClass testObject = new TestClass();

		Map<String, Object> stringObjectMap = sut.javaObjectToRpcMap(testObject, testObject.getClass());
		verify(sut, times(36)).getDefaultValue(any(Class.class));

		assertEquals("", stringObjectMap.get("nullString"));
		assertEquals("", stringObjectMap.get("nullEnum"));
		assertEquals(0, stringObjectMap.get("nullInteger"));
		assertEquals(String.valueOf(0L), stringObjectMap.get("nullLong"));
		assertEquals(0D, stringObjectMap.get("nullDouble"));
		assertEquals(false, stringObjectMap.get("nullBoolean"));
		assertArrayEquals(new byte[0], (byte[]) stringObjectMap.get("nullByteArr"));
		assertEquals(new Date(0), stringObjectMap.get("nullDate"));
		assertEquals(Collections.emptyList(), stringObjectMap.get("nullList"));
		assertEquals(Collections.emptyMap(), stringObjectMap.get("nullMap"));

		assertEquals("nonNull", stringObjectMap.get("nonNullString"));
		assertEquals(1, stringObjectMap.get("nonNullInteger"));
		assertEquals("1", stringObjectMap.get("nonNullLong"));
		assertEquals(1D, stringObjectMap.get("nonNullDouble"));
		assertEquals(true, stringObjectMap.get("nonNullBoolean"));
		assertArrayEquals(new byte[]{127}, (byte[]) stringObjectMap.get("nonNullByteArr"));
		assertNotNull(stringObjectMap.get("nonNullDate"));
		assertEquals(1, ((List) stringObjectMap.get("nonNullList")).size());
		assertEquals(1, ((Map) stringObjectMap.get("nonNullMap")).size());

		//verify innerClass
		Object innerClass = stringObjectMap.get("innerClass");
		assertNotNull(innerClass);
		assertTrue(Map.class.isInstance(innerClass),
			"object for key 'innerClass' should be of type Map<String, Object>");
		Map<String, Object> innerClassMap = (Map<String, Object>) innerClass;

		assertEquals("", innerClassMap.get("nullString"));
		assertEquals("", innerClassMap.get("nullEnum"));
		assertEquals(0, innerClassMap.get("nullInteger"));
		assertEquals(String.valueOf(0L), innerClassMap.get("nullLong"));
		assertEquals(0D, innerClassMap.get("nullDouble"));
		assertEquals(false, innerClassMap.get("nullBoolean"));
		assertArrayEquals(new byte[0], (byte[]) innerClassMap.get("nullByteArr"));
		assertEquals(new Date(0), innerClassMap.get("nullDate"));
		assertEquals(Collections.emptyList(), innerClassMap.get("nullList"));
		assertEquals(Collections.emptyMap(), innerClassMap.get("nullMap"));

		assertEquals("nonNull", innerClassMap.get("nonNullString"));
		assertEquals(1, innerClassMap.get("nonNullInteger"));
		assertEquals("1", innerClassMap.get("nonNullLong"));
		assertEquals(1D, innerClassMap.get("nonNullDouble"));
		assertEquals(true, innerClassMap.get("nonNullBoolean"));
		assertArrayEquals(new byte[]{127}, (byte[]) innerClassMap.get("nonNullByteArr"));
		assertNotNull(innerClassMap.get("nonNullDate"));
		assertEquals(1, ((List) innerClassMap.get("nonNullList")).size());
		assertEquals(1, ((Map) innerClassMap.get("nonNullMap")).size());

		//verify innerClassNull
		Object innerClassNull = stringObjectMap.get("innerClass");
		assertNotNull(innerClassNull);
		assertTrue(Map.class.isInstance(innerClass),
			"object for key 'innerClass' should be of type Map<String, Object>");
		Map<String, Object> innerClassNullMap = (Map<String, Object>) innerClassNull;

		assertEquals("", innerClassNullMap.get("nullString"));
		assertEquals("", innerClassNullMap.get("nullEnum"));
		assertEquals(0, innerClassNullMap.get("nullInteger"));
		assertEquals(String.valueOf(0L), innerClassNullMap.get("nullLong"));
		assertEquals(0D, innerClassNullMap.get("nullDouble"));
		assertEquals(false, innerClassNullMap.get("nullBoolean"));
		assertArrayEquals(new byte[0], (byte[]) innerClassNullMap.get("nullByteArr"));
		assertEquals(new Date(0), innerClassNullMap.get("nullDate"));
		assertEquals(Collections.emptyList(), innerClassNullMap.get("nullList"));
		assertEquals(Collections.emptyMap(), innerClassNullMap.get("nullMap"));

		assertEquals("nonNull", innerClassNullMap.get("nonNullString"));
		assertEquals(1, innerClassNullMap.get("nonNullInteger"));
		assertEquals("1", innerClassNullMap.get("nonNullLong"));
		assertEquals(1D, innerClassNullMap.get("nonNullDouble"));
		assertEquals(true, innerClassNullMap.get("nonNullBoolean"));
		assertArrayEquals(new byte[]{127}, (byte[]) innerClassNullMap.get("nonNullByteArr"));
		assertNotNull(innerClassNullMap.get("nonNullDate"));
		assertEquals(1, ((List) innerClassNullMap.get("nonNullList")).size());
		assertEquals(1, ((Map) innerClassNullMap.get("nonNullMap")).size());

	}

	@RpcValueObject
	private static class TestClass {
		@RpcValue
		private InnerClass innerClass;
		@RpcValue
		private InnerClass nullInnerClass = null;
		@RpcValue
		private String nullString;
		@RpcValue
		private Integer nullInteger;
		@RpcValue
		private Long nullLong;
		@RpcValue
		private Double nullDouble;
		@RpcValue
		private Boolean nullBoolean;
		@RpcValue
		private byte[] nullByteArr;
		@RpcValue
		private Map<Object, Object> nullMap;
		@RpcValue
		private List<Object> nullList;
		@RpcValue
		private Date nullDate;
		@RpcValue
		private MyEnum nullEnum;

		@RpcValue
		private List<String> innerNullStringList;
		@RpcValue
		private Map<String, String> innerNullStringValueMap;

		@RpcValue
		private String nonNullString = "nonNull";
		@RpcValue
		private Integer nonNullInteger = 1;
		@RpcValue
		private Long nonNullLong = 1L;
		@RpcValue
		private Double nonNullDouble = 1D;
		@RpcValue
		private Boolean nonNullBoolean = true;
		@RpcValue
		private byte[] nonNullByteArr = new byte[]{127};
		@RpcValue
		private Map<Object, Object> nonNullMap = new HashMap<Object, Object>();
		@RpcValue
		private List<Object> nonNullList = new ArrayList<Object>();
		@RpcValue
		private Date nonNullDate = new Date();

		public TestClass() {
			this.innerNullStringList = new ArrayList<String>();
			this.innerNullStringList.add(null);

			this.innerNullStringValueMap = new HashMap<String, String>();
			this.innerNullStringValueMap.put("innerNull", null);

			this.nonNullMap.put("", "");
			this.nonNullList.add("");

			innerClass = new InnerClass();
		}
	}

	@RpcValueObject
	public static class InnerClass {
		@RpcValue
		private String nullString;
		@RpcValue
		private Integer nullInteger;
		@RpcValue
		private Long nullLong;
		@RpcValue
		private Double nullDouble;
		@RpcValue
		private Boolean nullBoolean;
		@RpcValue
		private byte[] nullByteArr;
		@RpcValue
		private Map<Object, Object> nullMap;
		@RpcValue
		private List<Object> nullList;
		@RpcValue
		private Date nullDate;
		@RpcValue
		private MyEnum nullEnum;

		@RpcValue
		private List<String> innerNullStringList;
		@RpcValue
		private Map<String, String> innerNullStringValueMap;

		@RpcValue
		private String nonNullString = "nonNull";
		@RpcValue
		private Integer nonNullInteger = 1;
		@RpcValue
		private Long nonNullLong = 1L;
		@RpcValue
		private Double nonNullDouble = 1D;
		@RpcValue
		private Boolean nonNullBoolean = true;
		@RpcValue
		private byte[] nonNullByteArr = new byte[]{127};
		@RpcValue
		private Map<Object, Object> nonNullMap = new HashMap<Object, Object>();
		@RpcValue
		private List<Object> nonNullList = new ArrayList<Object>();
		@RpcValue
		private Date nonNullDate = new Date();

		public InnerClass() {
			this.innerNullStringList = new ArrayList<String>();
			this.innerNullStringList.add(null);

			this.innerNullStringValueMap = new HashMap<String, String>();
			this.innerNullStringValueMap.put("innerNull", null);

			this.nonNullMap.put("", "");
			this.nonNullList.add("");
		}
	}

	private enum MyEnum {
		TEST,
		HUHU,
		BLUBB;
	}
}
