/*
 * Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.v22.client.impl;

import de.starface.integration.uci.v22.client.UciProxyConfigurationFailedException;
import de.starface.integration.uci.v22.client.transport.UcpTransport;
import de.starface.integration.uci.v22.client.transport.UcpTransportFactory;
import de.starface.integration.uci.v22.client.transport.UcpTransportFactory.Configurator;

import org.apache.commons.lang3.Validate;

import java.io.File;
import java.util.Map;

/**
 * A base class used to implement a factory for UCI proxies, that initializes UcpTransport creation
 * and connection management.
 *
 * @author tilmann
 *
 */
public class UciProxyFactoryBase {
	private static final String DEFAULT_PROPERTIES_FILENAME = "uci-proxy.properties";

	private UcpTransportFactory ucpTransportFactory;

	/**
	 * Creates a new UciProxyFactoryBase and tries to read its configuration from the default
	 * properties file with the name "uci-proxy.properties".
	 *
	 * There must exist one mandatory property in the file with the key
	 * "ucpTransportFactory.configuratorClass" that contains the fully qualified Java class name of
	 * a {@link Configurator} implementation. Additional properties that are required by the
	 * specified {@link Configurator} implementation must also be present in the file.
	 *
	 * @throws UciProxyConfigurationFailedException
	 *         if a problem occurs while accessing the file or creating or executing the specified
	 *         {@link Configurator} implementation.
	 */
	protected UciProxyFactoryBase() throws UciProxyConfigurationFailedException {
		ucpTransportFactory = UcpTransportFactory.createFromPropertiesFile(DEFAULT_PROPERTIES_FILENAME);
	}

	/**
	 * Creates a new UciProxyFactoryBase and tries to read its configuration from the given file.
	 *
	 * There must exist one mandatory property in the file with the key
	 * "ucpTransportFactory.configuratorClass" that contains the fully qualified Java class name of
	 * a {@link Configurator} implementation. Additional properties that are required by the
	 * specified {@link Configurator} implementation must also be present in the file.
	 *
	 * @param filename
	 *        the Java properties file that contains the configuration properties
	 * @throws UciProxyConfigurationFailedException
	 *         if a problem occurs while accessing the file or creating or executing the specified
	 *         {@link Configurator} implementation.
	 * @throws IllegalArgumentException
	 *         if null or empty filename given
	 */
	protected UciProxyFactoryBase(String filename) throws UciProxyConfigurationFailedException {
		ucpTransportFactory = UcpTransportFactory.createFromPropertiesFile(filename);
	}

	/**
	 * Creates a new UciProxyFactoryBase and tries to read its configuration from the given file.
	 *
	 * There must exist one mandatory property in the file with the key
	 * "ucpTransportFactory.configuratorClass" that contains the fully qualified Java class name of
	 * a {@link Configurator} implementation. Additional properties that are required by the
	 * specified {@link Configurator} implementation must also be present in the file.
	 *
	 * @param file
	 *        the Java properties file that contains the configuration properties
	 * @throws UciProxyConfigurationFailedException
	 *         if a problem occurs while accessing the file or creating or executing the specified
	 *         {@link Configurator} implementation.
	 * @throws IllegalArgumentException
	 *         if null or empty file given
	 */
	protected UciProxyFactoryBase(File file) throws UciProxyConfigurationFailedException {
		ucpTransportFactory = UcpTransportFactory.createFromPropertiesFile(file);
	}

	/**
	 * Creates a new UciProxyFactoryBase and tries to read its configuration from the given
	 * {@link Map}.
	 *
	 * There must exist one mandatory property in the map with the key
	 * "ucpTransportFactory.configuratorClass" that contains the fully qualified Java class name of
	 * a {@link Configurator} implementation. Additional properties that are required by the
	 * specified {@link Configurator} implementation must also be present in the map.
	 *
	 * @param properties
	 *        the configuration properties
	 * @throws UciProxyConfigurationFailedException
	 *         if a problem occurs while creating or executing the specified {@link Configurator}
	 *         implementation.
	 * @throws IllegalArgumentException
	 *         if null properties given
	 */
	protected UciProxyFactoryBase(Map<String, String> properties) throws UciProxyConfigurationFailedException {
		ucpTransportFactory = UcpTransportFactory.createFromPropertiesMap(properties);
	}

	/**
	 * Creates a new {@link UciProxyFactoryBase} obtaining its connections from the given
	 * {@link UcpTransportFactory}. Thus select the proper factory to use depending on the transport
	 * mechanism to use for the connections.
	 *
	 * @param ucpTransportFactory
	 *        the {@link UcpTransportFactory} to be used
	 * @throws IllegalArgumentException
	 *         if null ucpTransportFactory given
	 * @throws IllegalArgumentException
	 *         if null ucpTransportFactory given
	 */
	protected UciProxyFactoryBase(UcpTransportFactory ucpTransportFactory) {
		Validate.notNull(ucpTransportFactory, "ucpTransportFactory=null");
		this.ucpTransportFactory = ucpTransportFactory;
	}

	/**
	 * Creates a {@link UcpTransport} for the STARFACE user identified by the given login id and
	 * password.
	 *
	 * @param loginId
	 *        the login id of the STARFACE user
	 * @param password
	 *        the password of the STARFACE user
	 * @return The {@link UcpTransport} tied to the STARFACE user.
	 * @throws IllegalArgumentException
	 *         if null or empty loginId or null password given
	 */
	protected UcpTransport createUcpTransport(String loginId, String password) {
		Validate.notEmpty(loginId, "loginId=empty");
		Validate.notNull(password, "password=null");

		return ucpTransportFactory.createUcpTransport(loginId, password);
	}
}
