/*
 * Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.v22.client.transport;

import de.starface.integration.uci.v22.client.UciProxyConfigurationFailedException;
import de.starface.integration.uci.v22.client.utils.ConfigurationUtils;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.Validate;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Map;
import java.util.Properties;

/**
 * An abstract factory used to create {@link UcpTransport} instances for different STARFACE users.
 *
 * @author tilmann
 *
 */
public abstract class UcpTransportFactory {
	private static final String DEFAULT_PROPERTIES_FILENAME = "ucp-transport.properties";

	/**
	 * Creates a new {@link UcpTransport} for the STARFACE user identified by the given loginId and
	 * password.
	 *
	 * @param loginId
	 *        the login ID of the STARFACE user
	 * @param password
	 *        the password of the STARFACE user
	 * @return an instance of {@link UcpTransport} that can be used to connect to the UCP server
	 */
	public abstract UcpTransport createUcpTransport(String loginId, String password);

	/**
	 * An abstract factory used to create and configure {@link UcpTransportFactory} instances from
	 * configuration properties. This interface should be implemented for every UcpTransportFactory
	 * sub class to make it automatically configurable.
	 *
	 * @author tilmann
	 *
	 */
	public interface Configurator {
		/**
		 * Creates a new {@link UcpTransportFactory} instance and configures it from the values in
		 * the given {@link Map}
		 *
		 * @param configuration
		 *        the configuration properties to be used
		 * @return a newly created {@link UcpTransportFactory} instance
		 * @throws UciProxyConfigurationFailedException
		 *         if a problem occurs while creating or configuring the new instance
		 */
		public UcpTransportFactory configureFromMap(Map<String, String> configuration)
			throws UciProxyConfigurationFailedException;
	}

	/**
	 * Creates a new UciProxyFactory and tries to read its configuration from the default properties
	 * file with the name "uci-proxy.properties".
	 *
	 * There must exist one mandatory property in the file with the key
	 * "ucpTransportFactory.configuratorClass" that contains the fully qualified Java class name of
	 * a {@link Configurator} implementation. Additional properties that are required by the
	 * specified {@link Configurator} implementation must also be present in the file.
	 *
	 * @return the newly created and configured {@link UcpTransportFactory}
	 * @throws UciProxyConfigurationFailedException
	 *         if a problem occurs while accessing the file or creating or executing the specified
	 *         {@link Configurator} implementation.
	 */
	public static UcpTransportFactory createFromDefaultPropertiesFile()
		throws UciProxyConfigurationFailedException {
		InputStream inputStream = ClassLoader.getSystemResourceAsStream(DEFAULT_PROPERTIES_FILENAME);
		if (inputStream == null) {
			throw new UciProxyConfigurationFailedException("The default configuration file "
				+ DEFAULT_PROPERTIES_FILENAME + " could not be found in the classpath.");
		}
		return createFromPropertiesFileStream(inputStream);
	}

	/**
	 * Creates a new UciProxyFactory and tries to read its configuration from the given file.
	 *
	 * There must exist one mandatory property in the file with the key
	 * "ucpTransportFactory.configuratorClass" that contains the fully qualified Java class name of
	 * a {@link Configurator} implementation. Additional properties that are required by the
	 * specified {@link Configurator} implementation must also be present in the file.
	 *
	 * @param filename
	 *        the Java properties file that contains the configuration properties
	 * @return the newly created and configured {@link UcpTransportFactory}
	 * @throws UciProxyConfigurationFailedException
	 *         if a problem occurs while accessing the file or creating or executing the specified
	 *         {@link Configurator} implementation.
	 * @throws IllegalArgumentException
	 *         if null or empty filename given
	 */
	public static UcpTransportFactory createFromPropertiesFile(String filename)
		throws UciProxyConfigurationFailedException {
		Validate.notEmpty(filename, "filename=empty");
		File file = new File(filename);
		return createFromPropertiesFile(file);
	}

	/**
	 * Creates a new UciProxyFactory and tries to read its configuration from the given file.
	 *
	 * There must exist one mandatory property in the file with the key
	 * "ucpTransportFactory.configuratorClass" that contains the fully qualified Java class name of
	 * a {@link Configurator} implementation. Additional properties that are required by the
	 * specified {@link Configurator} implementation must also be present in the file.
	 *
	 * @param file
	 *        the Java properties file that contains the configuration properties
	 * @return the newly created and configured {@link UcpTransportFactory}
	 * @throws UciProxyConfigurationFailedException
	 *         if a problem occurs while accessing the file or creating or executing the specified
	 *         {@link Configurator} implementation.
	 * @throws IllegalArgumentException
	 *         if null file given
	 */
	public static UcpTransportFactory createFromPropertiesFile(File file)
		throws UciProxyConfigurationFailedException {
		Validate.notNull(file, "file=null");
		InputStream inputStream;
		try {
			inputStream = new FileInputStream(file);
		} catch (FileNotFoundException e) {
			throw new UciProxyConfigurationFailedException("Could not find properties file "
				+ file.getAbsolutePath());
		}
		return createFromPropertiesFileStream(inputStream);
	}

	@SuppressWarnings({"unchecked", "rawtypes"})
	private static UcpTransportFactory createFromPropertiesFileStream(InputStream inputStream)
		throws UciProxyConfigurationFailedException {
		Validate.notNull(inputStream, "inputStream=null");
		Properties properties = new Properties();
		try {
			properties.load(inputStream);
		} catch (IOException e) {
			throw new UciProxyConfigurationFailedException("Could not load configuration properties from file.", e);
		}
		return createFromPropertiesMap((Map) properties);
	}

	/**
	 * Creates a new UciProxyFactory and tries to read its configuration from the given {@link Map}.
	 *
	 * There must exist one mandatory property in the map with the key
	 * "ucpTransportFactory.configuratorClass" that contains the fully qualified Java class name of
	 * a {@link Configurator} implementation. Additional properties that are required by the
	 * specified {@link Configurator} implementation must also be present in the map.
	 *
	 * @param properties
	 *        the configuration properties
	 * @return the newly created and configured {@link UcpTransportFactory}
	 * @throws UciProxyConfigurationFailedException
	 *         if a problem occurs while creating or executing the specified {@link Configurator}
	 *         implementation.
	 * @throws IllegalArgumentException
	 *         if null properties given
	 */
	public static UcpTransportFactory createFromPropertiesMap(Map<String, String> properties)
		throws UciProxyConfigurationFailedException {
		Validate.notNull(properties, "properties=null");
		String configuratorClassName = properties.get("ucpTransportFactory.configuratorClass");
		if (StringUtils.isEmpty(configuratorClassName)) {
			throw new UciProxyConfigurationFailedException(
				"The configuration property ucpTransportFactory.configuratorClass is not set.");
		}

		Configurator configurator = ConfigurationUtils.getInterfaceImplementationForClassName(
			configuratorClassName, Configurator.class);

		UcpTransportFactory transportFactory = configurator.configureFromMap(properties);

		return transportFactory;
	}
}
