/*
 * Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.v22.client.transport.direct;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.client.RpcClient;
import de.starface.com.rpc.direct.DirectRpcClient;
import de.starface.com.rpc.direct.DirectRpcServer;
import de.starface.com.rpc.direct.DirectTransportToken;
import de.starface.com.rpc.server.RpcObjectRegistry;
import de.starface.com.rpc.server.RpcServer;
import de.starface.integration.uci.v22.client.UcpConnectionFailedException;
import de.starface.integration.uci.v22.client.transport.ExceptionConverter;
import de.starface.integration.uci.v22.client.transport.UcpTransport;
import de.starface.integration.uci.v22.client.transport.UcpTransportEvents;
import de.starface.integration.uci.v22.client.utils.MD5;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * This class implements the {@link UcpTransport} providing UCP connectivity over direct method
 * calls using direct RPC servers inside the same Java VM provided by STARFACE RPC.
 *
 * @author tilmann
 */
public class DirectUcpTransport implements UcpTransport {
	private static final Log log = LogFactory.getLog(DirectUcpTransport.class);

	private static final String UCI_DIRECT_SERVER_NAME = "starface.direct.uci";

	private String applicationName;
	private String loginId;
	private String password;

	private RpcServer<DirectTransportToken> directRpcServer;

	private RpcClient<DirectTransportToken> directRpcClient;

	private final ExceptionConverter exceptionConverter = new ExceptionConverter();

	DirectUcpTransport(String applicationName, String loginId, String password) {
		this.applicationName = applicationName;
		this.loginId = loginId;
		this.password = password;
	}

	/**
	 * @see de.starface.integration.uci.v22.client.transport.UcpTransport#close()
	 */
	public synchronized void close() {
		closeServer();
		closeClient();
	}

	private void closeServer() {
		if (directRpcServer != null) {
			try {
				directRpcServer.shutdown();
			} catch (RpcException e) {
				log.info("close: Could not shut down directRpcServer.", e);
			} finally {
				directRpcServer = null;
			}
		}
	}

	private void closeClient() {
		if (directRpcClient != null) {
			directRpcClient = null;
		}
	}

	private void createClient() {
		if (directRpcClient == null) {
			String auth = MD5.toMd5Digest(loginId + "*" + password);
			DirectTransportToken callbackServerToken = new DirectTransportToken(getCallbackServerName(), auth);
			DirectTransportToken uciServerToken = new DirectTransportToken(UCI_DIRECT_SERVER_NAME);
			directRpcClient = DirectRpcClient.createWithDefaultAndCallbackServer(uciServerToken,
				callbackServerToken);
			directRpcClient.setExceptionConverter(exceptionConverter);
		}
	}

	private void createServer() throws UcpConnectionFailedException {
		try {
			if (directRpcServer == null) {
				directRpcServer = DirectRpcServer.createWithServerName(getCallbackServerName());
				directRpcServer.setRegistry(new RpcObjectRegistry());
			}
			if (!directRpcServer.isRunning()) {
				directRpcServer.startup();
			}
		} catch (RpcException e) {
			throw new UcpConnectionFailedException(UcpConnectionFailedException.Codes.TRANSPORT_FAILURE,
				"Could not create RPC server", e);
		}
	}

	private String getCallbackServerName() {
		return applicationName + "/" + loginId;
	}

	/**
	 * @see de.starface.integration.uci.v22.client.transport.UcpTransport#open()
	 */
	public void open() throws UcpConnectionFailedException {
		createClient();
	}

	/**
	 * @see de.starface.integration.uci.v22.client.transport.UcpTransport#subscribeUcpTransportEvents(de.starface.integration.uci.v22.client.transport.UcpTransportEvents)
	 */
	public void subscribeUcpTransportEvents(UcpTransportEvents listener) {
		exceptionConverter.subscribeEvents(listener);
	}

	/**
	 * @see de.starface.integration.uci.v22.client.transport.UcpTransport#unsubscribeUcpTransportEvents(de.starface.integration.uci.v22.client.transport.UcpTransportEvents)
	 */
	public void unsubscribeUcpTransportEvents(UcpTransportEvents listener) {
		exceptionConverter.unsubscribeEvents(listener);
	}

	/**
	 * @see de.starface.integration.uci.v22.client.transport.UcpTransport#registerUcpEventListener(java.lang.Object,
	 *      java.lang.Class)
	 */
	public <Events> void registerUcpEventListener(Events listener, Class<Events> eventsInterface)
		throws UcpConnectionFailedException {
		createServer();

		directRpcServer.getRegistry().registerRpcObject(listener, eventsInterface);
	}

	/**
	 * @see de.starface.integration.uci.v22.client.transport.UcpTransport#unregisterUcpEventListener(java.lang.Class)
	 */
	public <Events> void unregisterUcpEventListener(Class<Events> eventsInterface) {
		if (directRpcServer != null) {
			directRpcServer.getRegistry().unregisterRpcObject(eventsInterface);

			if (directRpcServer.getRegistry().isEmpty()) {
				closeServer();
			}
		}
	}

	/**
	 * @see de.starface.integration.uci.v22.client.transport.UcpTransport#getUcpRequests(java.lang.Class)
	 */
	public <Requests> Requests getUcpRequests(Class<Requests> requestsInterface)
		throws UcpConnectionFailedException {
		try {
			return directRpcClient.createRpcProxy(requestsInterface);
		} catch (RpcException e) {
			close();
			throw new UcpConnectionFailedException(UcpConnectionFailedException.Codes.TRANSPORT_FAILURE,
				"Could not create RPC proxy", e);
		}
	}
}
