/*
 * Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.v22.client.transport.direct;

import de.starface.integration.uci.v22.client.UciProxyConfigurationFailedException;
import de.starface.integration.uci.v22.client.transport.UcpTransportFactory;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.Validate;

import java.util.Map;

/**
 * This implementation of {@link UcpTransportFactory} creates instances of
 * {@link DirectUcpTransport} that can be used to connect to the UCP server over direct method
 * calls.
 *
 * @author tilmann
 */
public class DirectUcpTransportFactory extends UcpTransportFactory {
	private String applicationName;

	/**
	 * Creates a new DirectUcpTransportFactory that uses the given connection information to create
	 * {@link DirectUcpTransport} instances that connect to the UCP server over direct java calls.
	 *
	 * @param applicationName
	 *        a unique name that identifies the application using this factory. Direct connections
	 *        use this as a resource to identify the communication endpoint so for a given user and
	 *        application combination only one client can be logged in at the server side. May
	 *        contain only word characters, hyphen and dot.
	 * @throws IllegalArgumentException
	 *         if null empty or invalid applicationName given
	 */
	public DirectUcpTransportFactory(String applicationName) {
		Validate.notEmpty(applicationName, "applicationName=empty");
		if (!applicationName.matches("\\A[\\-\\w\\.]+\\z")) {
			throw new IllegalArgumentException(
				"applicationName must not contain any characters other than word charactrs, the hyphen and the dot");
		}

		this.applicationName = applicationName;
	}

	/**
	 * Creates a new {@link DirectUcpTransport} for the STARFACE user identified by the given
	 * loginId and password.
	 *
	 * @param loginId
	 *        the login ID of the STARFACE user
	 * @param password
	 *        the password of the STARFACE user
	 * @return an instance of {@link DirectUcpTransport} that can be used to connect to the UCP
	 *         server over direct java calls
	 * @throws IllegalArgumentException
	 *         if null or empty loginId or null password given
	 */
	public DirectUcpTransport createUcpTransport(String loginId, String password) {
		Validate.notEmpty(loginId, "loginId=empty");
		Validate.notNull(password, "password=null");

		return new DirectUcpTransport(applicationName, loginId, password);
	}

	/**
	 * This {@link Configurator} implementation is used to create and configure
	 * {@link DirectUcpTransportFactory} instances from configuration properties.
	 *
	 * There is only one property to set for this Configurator:
	 * <ul>
	 * <li>directUcpTransportFactory.applicationName</li>
	 * </ul>
	 * The value to be set is the same as the corresponding constructor parameter of
	 * {@link DirectUcpTransportFactory#DirectUcpTransportFactory(String)}
	 *
	 * @author tilmann
	 *
	 */
	public static class Configurator implements UcpTransportFactory.Configurator {
		private static final String APPLICATION_NAME_PROPERTY = "directUcpTransportFactory.applicationName";

		/**
		 * Creates a new {@link DirectUcpTransportFactory} instance and configures it from the
		 * values in the given {@link Map}
		 *
		 * @param configuration
		 *        the configuration properties to be used
		 * @return a newly created {@link DirectUcpTransportFactory} instance
		 * @throws UciProxyConfigurationFailedException
		 *         if a problem occurs while creating or configuring the new instance
		 */
		public DirectUcpTransportFactory configureFromMap(Map<String, String> configuration)
			throws UciProxyConfigurationFailedException {
			String applicationName = configuration.get(APPLICATION_NAME_PROPERTY);

			if (StringUtils.isEmpty(applicationName)) {
				throw new UciProxyConfigurationFailedException("configuration property "
					+ APPLICATION_NAME_PROPERTY + " is not set");
			}

			return new DirectUcpTransportFactory(applicationName);
		}
	}
}
