/*
 * $Id: LoginUrlAuthToken.java 432 2010-11-05 17:13:01Z tilmann $
 * 
 * Copyright (C) by vertico software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * $LastChangedDate: 2010-11-05 18:13:01 +0100 (Fr, 05 Nov 2010) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 432 $
 */
package de.starface.integration.uci.v22.client.transport.http;

import de.starface.com.rpc.common.authentication.RpcAuthToken;
import de.starface.com.rpc.xmlrpc.http.URL;
import de.starface.com.rpc.xmlrpc.http.URL.Protocol;
import de.starface.integration.uci.v22.client.utils.MD5;

import org.apache.commons.lang3.Validate;

/**
 * An {@link XmlRpcAuthToken} that is used to map between a {@link URL} and a STARFACE login ID. And
 * is used to distinguish between the corresponding UCP clients for incoming server events.
 *
 * @author tilmann
 */
class LoginUrlAuthToken implements RpcAuthToken {
	private static final String ENDPOINT = "/xml-rpc";

	private String loginId;

	private String authHash;

	private URL url;

	String getLoginId() {
		return loginId;
	}

	/**
	 * Creates a new {@link LoginUrlAuthToken} for the user with the given login credentials for
	 * connection to the given server URL.
	 *
	 * @param loginId
	 *        the login ID of the STARFACE user
	 * @param password
	 *        the password of the STARFACE user
	 * @param serverUrl
	 *        the URL of the STARFACE server. This URL will be enriched with an authentication token
	 * @throws IllegalArgumentException
	 *         if null or empty loginId or null password or null serverUrl given
	 */
	LoginUrlAuthToken(String serverHost, int serverPort, boolean useSsl, String loginId, String password) {
		Validate.notEmpty(loginId, "loginId=empty");
		Validate.notNull(password, "password=null");
		Validate.notEmpty(serverHost, "serverHost=empty");
		if (serverPort <= 0) {
			throw new IllegalArgumentException("serverPort must not be less or equal to zero");
		}

		this.loginId = loginId;
		authHash = MD5.toMd5Digest(loginId + "*" + password);
		url = createServerUrl(serverHost, serverPort, useSsl);
		url.setAuth(authHash);
		url.setUser(loginId);
	}

	URL getUrl() {
		return url;
	}

	/**
	 * Computes the hash code depending on the loginId given at construction time
	 *
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((loginId == null) ? 0 : loginId.hashCode());
		return result;
	}

	/**
	 * Computes the equality depending on the loginId given at construction time
	 *
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		LoginUrlAuthToken other = (LoginUrlAuthToken) obj;
		if (loginId == null) {
			if (other.loginId != null) {
				return false;
			}
		} else if (!loginId.equals(other.loginId)) {
			return false;
		}
		return true;
	}

	private URL createServerUrl(String serverHost, int serverPort, boolean useSsl) {
		if (useSsl) {
			return new URL(Protocol.https, serverHost, serverPort, ENDPOINT);
		} else {
			return new URL(Protocol.http, serverHost, serverPort, ENDPOINT);
		}
	}
}
