/*
 * Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.v30.client;

/**
 * This interfaces contains the connection requests that can be sent to the {@link UciProxy} to
 * change its connection state.
 *
 * @author tilmann
 */
public interface UciConnectionRequests {
	/**
	 * Connects this {@link UciProxy}. A first connection attempt is made synchronously and if it
	 * fails a {@link UcpConnectionFailedException} is thrown. Afterwards the connection will be
	 * retried and reestablished asynchronously every 30 seconds until disconnect() is called on
	 * this proxy. If this proxy was built with a client implementing {@link UciConnectionEvents}
	 * then connection states and exceptions during connection establishment will be signaled to the
	 * client.
	 *
	 * @throws UcpConnectionFailedException
	 *         if the first connection attempt to the server fails
	 */
	public void connect() throws UcpConnectionFailedException;

	/**
	 * Connects this {@link UciProxy}. This method immediately returns. The UCP connection will be
	 * established and reestablished asynchronously every 30 seconds until disconnect() is called on
	 * this proxy. If this proxy was built with a client implementing {@link UciConnectionEvents}
	 * then connection states and exceptions during connection establishment will be signaled to the
	 * client.
	 */
	public void connectAsync();

	/**
	 * A call to disconnect tells the {@link UciProxy} that it is no longer used. It will disconnect
	 * from the UCP server and release its resources.
	 */
	public void disconnect();

	/**
	 * getConnectionState can be called by the {@link UciProxy} client to get to know if the
	 * connection to the UCP server is currently available.
	 *
	 * @return <code>true</code> if the connection to the UCP server is available,
	 *         <code>false</code> otherwise
	 */
	public boolean getConnectionState();

	/**
	 * Adds a listener to the {@link UciProxy} that is called upon connection changes and problems.
	 *
	 * @param listener
	 *        the listener to be added
	 */
	public void addConnectionEventListener(UciConnectionEvents listener);

	/**
	 * Removes a connection event listener from the {@link UciProxy}.
	 *
	 * @param listener
	 *        the listener to be removed
	 */
	public void removeConnectionEventListener(UciConnectionEvents listener);
}
