/*
 * Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.v30.client.impl;

import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.v30.client.UciConnectionEvents;
import de.starface.integration.uci.v30.client.UciConnectionRequests;
import de.starface.integration.uci.v30.client.UciProxy;
import de.starface.integration.uci.v30.client.UciServiceRequests;
import de.starface.integration.uci.v30.client.UcpConnectionFailedException;

import org.apache.commons.lang3.Validate;

import java.util.List;

/**
 * A simple facade implementation of {@link UciProxy} that just delegates calls to the correct
 * implementations.
 *
 * @author tilmann
 */
public class UciProxyFacade implements UciProxy {
	private UciConnectionRequests connectionRequests;

	private UciServiceRequests serviceRequests;

	/**
	 * Creates a new UciProxyFacade wrapping the given implementation instances.
	 *
	 * @param connectionRequests
	 *        the UciConnectionRequests to delegate calls to
	 * @param serviceRequests
	 *        the UciServiceRequests to delegate calls to
	 */
	public UciProxyFacade(UciConnectionRequests connectionRequests, UciServiceRequests serviceRequests) {
		Validate.notNull(connectionRequests, "connectionRequests=null");
		Validate.notNull(serviceRequests, "serviceRequests=null");

		this.connectionRequests = connectionRequests;
		this.serviceRequests = serviceRequests;
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciServiceRequests#getProvidedEvents()
	 */
	public List<String> getProvidedEvents() throws UciException {
		return serviceRequests.getProvidedEvents();
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciServiceRequests#providesEvents(java.lang.String)
	 */
	public boolean providesEvents(String serviceName) throws UciException {
		return serviceRequests.providesEvents(serviceName);
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciServiceRequests#providesEvents(java.lang.Class)
	 */
	public <Events> boolean providesEvents(Class<Events> serviceInterface) throws UciException {
		return serviceRequests.providesEvents(serviceInterface);
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciServiceRequests#subscribeEvents(java.lang.Object,
	 *      java.lang.Class)
	 */
	public <Events> boolean subscribeEvents(Events listener, Class<Events> serviceInterface) throws UciException {
		return serviceRequests.subscribeEvents(listener, serviceInterface);
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciServiceRequests#subscribeEventsAsync(java.lang.Object,
	 *      java.lang.Class)
	 */
	public <Events> void subscribeEventsAsync(Events listener, Class<Events> serviceInterface) throws UciException {
		serviceRequests.subscribeEventsAsync(listener, serviceInterface);
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciServiceRequests#unsubscibeEvents(java.lang.Object,
	 *      java.lang.Class)
	 */
	public <Events> void unsubscibeEvents(Events listener, Class<Events> serviceInterface) {
		serviceRequests.unsubscibeEvents(listener, serviceInterface);
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciServiceRequests#getProvidedRequests()
	 */
	public List<String> getProvidedRequests() throws UciException {
		return serviceRequests.getProvidedRequests();
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciServiceRequests#providesRequests(java.lang.String)
	 */
	public boolean providesRequests(String serviceName) throws UciException {
		return serviceRequests.providesRequests(serviceName);
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciServiceRequests#providesRequests(java.lang.Class)
	 */
	public <Requests> boolean providesRequests(Class<Requests> serviceInterface) throws UciException {
		return serviceRequests.providesRequests(serviceInterface);
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciServiceRequests#getRequests(java.lang.Class)
	 */
	public <Requests> Requests getRequests(Class<Requests> serviceInterface) throws UciException {
		return serviceRequests.getRequests(serviceInterface);
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciConnectionRequests#connect()
	 */
	public void connect() throws UcpConnectionFailedException {
		connectionRequests.connect();
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciConnectionRequests#connectAsync()
	 */
	public void connectAsync() {
		connectionRequests.connectAsync();
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciConnectionRequests#disconnect()
	 */
	public void disconnect() {
		connectionRequests.disconnect();
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciConnectionRequests#getConnectionState()
	 */
	public boolean getConnectionState() {
		return connectionRequests.getConnectionState();
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciConnectionRequests#addConnectionEventListener(de.starface.integration.uci.v30.client.UciConnectionEvents)
	 */
	public void addConnectionEventListener(UciConnectionEvents listener) {
		connectionRequests.addConnectionEventListener(listener);
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciConnectionRequests#removeConnectionEventListener(de.starface.integration.uci.v30.client.UciConnectionEvents)
	 */
	public void removeConnectionEventListener(UciConnectionEvents listener) {
		connectionRequests.removeConnectionEventListener(listener);
	}
}
