/*
 * Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.v30.client.transport;

import de.starface.integration.uci.v30.client.UcpConnectionFailedException;

/**
 * Implementations of this interface encapsulate the connection transport to the UCP server
 *
 * @author tilmann
 */
public interface UcpTransport {
	/**
	 * Opens this UcpTransport so that requests can be sent to STARFACE
	 *
	 * @throws UcpConnectionFailedException
	 *         if there is an error while opening the transport
	 */
	void open() throws UcpConnectionFailedException;

	/**
	 * Closes the current {@link UcpTransport} if it is open
	 */
	void close();

	/**
	 * Subscribe a listener to this transport's events
	 *
	 * @param listener
	 *        the listener to be subscribed
	 */
	public void subscribeUcpTransportEvents(UcpTransportEvents listener);

	/**
	 * Unsubscribe a listener to this transport's events
	 *
	 * @param listener
	 *        the listener to be unsubscribed
	 */
	public void unsubscribeUcpTransportEvents(UcpTransportEvents listener);

	/**
	 * Registers the given event listener in the underlying transport for incoming calls to the
	 * given events interface. Only one listener can be registered for an interface at the same time
	 *
	 * @param <Events>
	 *        the events type
	 * @param listener
	 *        the listener to be registered for incoming calls
	 * @param eventsInterface
	 *        the events interface
	 * @throws UcpConnectionFailedException
	 *         if there is an error during listener registration, for example if a second
	 *         registration attempt is made for an event interface that already has a registered
	 *         listener.
	 */
	public <Events> void registerUcpEventListener(Events listener, Class<Events> eventsInterface)
		throws UcpConnectionFailedException;

	/**
	 * Unregisters the event listener for call to the given events interface.
	 *
	 * @param <Events>
	 *        the events type
	 * @param eventsInterface
	 *        the events interface
	 */
	public <Events> void unregisterUcpEventListener(Class<Events> eventsInterface);

	/**
	 * Creates a communication proxy for the given requests interface using the underlying transport
	 * for outgoing calls.
	 *
	 * @param <Requests>
	 *        the requests type
	 * @param requestsInterface
	 *        the requests interface
	 * @return a communication proxy implementation of the requests interface
	 * @throws UcpConnectionFailedException
	 *         if there is an error during proxy creation
	 */
	public <Requests> Requests getUcpRequests(Class<Requests> requestsInterface)
		throws UcpConnectionFailedException;
}
