/*
 * Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.v30.client.transport.http;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.xmlrpc.http.HttpXmlRpcClient;
import de.starface.integration.uci.v30.client.UcpConnectionFailedException;
import de.starface.integration.uci.v30.client.transport.ExceptionConverter;
import de.starface.integration.uci.v30.client.transport.UcpTransport;
import de.starface.integration.uci.v30.client.transport.UcpTransportEvents;

/**
 * This class implements the {@link UcpTransport} providing UCP connectivity over HTTP
 *
 * @author tilmann
 */
public class HttpUcpTransport implements UcpTransport {

	private LoginUrlAuthToken authToken;

	private HttpUcpTransportServer transportServer;

	private HttpXmlRpcClient httpXmlRpcClient;

	private final ExceptionConverter exceptionConverter = new ExceptionConverter();

	private final LoginUrlAuthTokenConverter authTokenConverter;

	HttpUcpTransport(HttpUcpTransportServer transportServer, LoginUrlAuthTokenConverter authTokenConverter,
					 String serverHost, int serverPort, boolean useSsl, String loginId, String password) {
		this.transportServer = transportServer;
		this.authTokenConverter = authTokenConverter;
		this.authToken = new LoginUrlAuthToken(serverHost, serverPort, useSsl, loginId, password);
		if (transportServer != null) {
			transportServer.addCallbackUrl(authToken);
		}
	}

	/**
	 * @see de.starface.integration.uci.v30.client.transport.UcpTransport#close()
	 */
	public void close() {
		if (transportServer != null) {
			transportServer.removeEventsListeners(authToken);
		}
		httpXmlRpcClient = null;
		authTokenConverter.clearFromCache(authToken);
	}

	/**
	 * @see de.starface.integration.uci.v30.client.transport.UcpTransport#open()
	 */
	public void open() throws UcpConnectionFailedException {
		httpXmlRpcClient = new HttpXmlRpcClient();
		httpXmlRpcClient.registerAuthTokenConverter(authTokenConverter);
		httpXmlRpcClient.setExceptionConverter(exceptionConverter);
	}

	/**
	 * @see de.starface.integration.uci.v30.client.transport.UcpTransport#subscribeUcpTransportEvents(de.starface.integration.uci.v30.client.transport.UcpTransportEvents)
	 */
	public void subscribeUcpTransportEvents(UcpTransportEvents listener) {
		exceptionConverter.subscribeEvents(listener);
	}

	/**
	 * @see de.starface.integration.uci.v30.client.transport.UcpTransport#unsubscribeUcpTransportEvents(de.starface.integration.uci.v30.client.transport.UcpTransportEvents)
	 */
	public void unsubscribeUcpTransportEvents(UcpTransportEvents listener) {
		exceptionConverter.unsubscribeEvents(listener);
	}

	/**
	 * @see de.starface.integration.uci.v30.client.transport.UcpTransport#registerUcpEventListener(java.lang.Object,
	 *      java.lang.Class)
	 */
	public <Events> void registerUcpEventListener(Events listener, Class<Events> eventsInterface)
		throws UcpConnectionFailedException {
		if (transportServer != null) {
			transportServer.addEventsListener(authToken, listener, eventsInterface);
		} else {
			throw new UcpConnectionFailedException(UcpConnectionFailedException.Codes.TRANSPORT_FAILURE,
				"The HTTP transport has not been set up to allow reception of server events.");
		}
	}

	/**
	 * @see de.starface.integration.uci.v30.client.transport.UcpTransport#unregisterUcpEventListener(java.lang.Class)
	 */
	public <Events> void unregisterUcpEventListener(Class<Events> eventsInterface) {
		if (transportServer != null) {
			transportServer.removeEventsListener(authToken, eventsInterface);
		}
	}

	/**
	 * @see de.starface.integration.uci.v30.client.transport.UcpTransport#getUcpRequests(java.lang.Class)
	 */
	public <Requests> Requests getUcpRequests(Class<Requests> requestsInterface)
		throws UcpConnectionFailedException {
		try {
			return httpXmlRpcClient.createRpcProxy(requestsInterface, authToken);
		} catch (RpcException e) {
			throw new UcpConnectionFailedException(UcpConnectionFailedException.Codes.TRANSPORT_FAILURE,
				"Could not create XML-RPC proxy for interface " + requestsInterface.getName(), e);
		}
	}
}
