/*
 * Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.v30.client.transport.http;

import de.starface.integration.uci.v30.client.UciProxyConfigurationFailedException;
import de.starface.integration.uci.v30.client.transport.UcpTransport;
import de.starface.integration.uci.v30.client.transport.UcpTransportFactory;

import org.apache.commons.lang3.StringUtils;

import java.util.Map;

/**
 * This implementation of {@link UcpTransportFactory} creates instances of {@link HttpUcpTransport}
 * that can be used to connect to the UCP server over HTTP
 *
 * @author tilmann
 */
public class HttpUcpTransportFactory extends UcpTransportFactory {
	private final HttpUcpTransportServer transportCommons;

	private final String serverHost;

	private final int serverPort;

	private final boolean useSsl;

	private final LoginUrlAuthTokenConverter authTokenConverter = new LoginUrlAuthTokenConverter();

	/**
	 * Creates a new XmppUcpTransportFactory that uses the given connection information to create
	 * {@link HttpUcpTransport} instances that connect to the UCP server over HTTP.
	 *
	 * The factory created using this constructor can only be used to create one-directional UCP
	 * connections. The listening for UCP events is not possible. To also receive UCP events use the
	 * constructor
	 * {@link #HttpUcpTransportFactory(String, int, String, int, boolean, String, String)} instead.
	 *
	 * @param serverHost
	 *        the host name or IP-address of the UCP server
	 * @param serverPort
	 *        the HTTP port of the UCP server
	 * @param useSsl
	 *        Set to true to use encrypted connection
	 */
	public HttpUcpTransportFactory(String serverHost, int serverPort, boolean useSsl) {
		this.serverHost = serverHost;
		this.serverPort = serverPort;
		this.useSsl = useSsl;
		transportCommons = null;
	}

	/**
	 * Creates a new XmppUcpTransportFactory that uses the given connection information to create
	 * {@link HttpUcpTransport} instances that connect to the UCP server over HTTP.
	 *
	 * The factory created using this constructor can be used to create bidirectional UCP
	 * connections. If listening for UCP events is not required the constructor
	 * {@link #HttpUcpTransportFactory(String, int, boolean)} can be used instead.
	 *
	 * @param applicationHost
	 *        the host name or IP-address of the machine running the current application. It is
	 *        important that applicationHost and applicationPort is accessible from the specified
	 *        server using the HTTP protocol. So the firewall has to be set up correctly to allow
	 *        that.
	 * @param applicationPort
	 *        the port number on the machine running the current application that will be used to
	 *        listen for HTTP calls from the server. It is important that applicationHost and
	 *        applicationPort is accessible from the specified server using the HTTP protocol. So
	 *        the firewall has to be set up correctly to allow that. It is important to note that no
	 *        two applications can use the same port on the same machine.
	 * @param serverHost
	 *        the host name or IP-address of the UCP server
	 * @param serverPort
	 *        the HTTP port of the UCP server
	 * @param useSsl
	 *        Set to true to use encrypted connection
	 * @param keyStorePath
	 *        the path of the SSL key store file to be used. May be omitted if SSL is not used
	 * @param keyStorePassword
	 *        the password of the SSL key store file to be used. May be omitted if SSL is not used
	 */
	public HttpUcpTransportFactory(String applicationHost, int applicationPort, String serverHost, int serverPort,
								   boolean useSsl, String keyStorePath, String keyStorePassword) {
		transportCommons = new HttpUcpTransportServer(authTokenConverter, applicationHost, applicationPort,
			useSsl, keyStorePath, keyStorePassword);
		this.useSsl = useSsl;
		this.serverHost = serverHost;
		this.serverPort = serverPort;
	}

	/**
	 * Creates a new {@link HttpUcpTransport} for the STARFACE user identified by the given loginId
	 * and password.
	 *
	 * @param loginId
	 *        the login ID of the STARFACE user
	 * @param password
	 *        the password of the STARFACE user
	 * @return an instance of {@link HttpUcpTransport} that can be used to connect to the UCP server
	 *         over HTTP
	 */
	public UcpTransport createUcpTransport(String loginId, String password) {
		return new HttpUcpTransport(transportCommons, authTokenConverter, serverHost, serverPort, useSsl, loginId,
			password);
	}

	/**
	 * This {@link Configurator} implementation is used to create and configure
	 * {@link HttpUcpTransportFactory} instances from configuration properties.
	 *
	 * There are several properties to set for this Configurator:
	 * <ul>
	 * <li>httpUcpTransportFactory.serverHost</li>
	 * <li>httpUcpTransportFactory.serverPort</li>
	 * <li>httpUcpTransportFactory.applicationHost</li>
	 * <li>httpUcpTransportFactory.applicationPort</li>
	 * <li>httpUcpTransportFactory.useSsl</li>
	 * <li>httpUcpTransportFactory.keyStorePath</li>
	 * <li>httpUcpTransportFactory.keyStorePassword</li>
	 * </ul>
	 * The values to be set are the same as the corresponding constructor parameters of
	 * {@link HttpUcpTransportFactory#HttpUcpTransportFactory(String, int, String, int, boolean, String, String)}
	 * or {@link HttpUcpTransportFactory#HttpUcpTransportFactory(String, int, boolean)}
	 *
	 * @author tilmann
	 *
	 */
	public static class Configurator implements UcpTransportFactory.Configurator {
		private static final String SERVER_HOST_PROPERTY = "httpUcpTransportFactory.serverHost";
		private static final String SERVER_PORT_PROPERTY = "httpUcpTransportFactory.serverPort";
		private static final String APPLICATION_HOST_PROPERTY = "httpUcpTransportFactory.applicationHost";
		private static final String APPLICATION_PORT_PROPERTY = "httpUcpTransportFactory.applicationPort";
		private static final String USE_SSL_PROPERTY = "httpUcpTransportFactory.useSsl";
		private static final String KEYSTORE_PATH_PROPERTY = "httpUcpTransportFactory.keyStorePath";
		private static final String KEYSTORE_PASSWORD_PROPERTY = "httpUcpTransportFactory.keyStorePassword";

		/**
		 * Creates a new {@link HttpUcpTransportFactory} instance and configures it from the values
		 * in the given {@link Map}
		 *
		 * @param configuration
		 *        the configuration properties to be used
		 * @return a newly created {@link HttpUcpTransportFactory} instance
		 * @throws UciProxyConfigurationFailedException
		 *         if a problem occurs while creating or configuring the new instance
		 */
		public HttpUcpTransportFactory configureFromMap(Map<String, String> configuration)
			throws UciProxyConfigurationFailedException {

			String serverHost = configuration.get(SERVER_HOST_PROPERTY);
			String serverPortString = configuration.get(SERVER_PORT_PROPERTY);
			String applicationHost = configuration.get(APPLICATION_HOST_PROPERTY);
			String applicationPortString = configuration.get(APPLICATION_PORT_PROPERTY);
			String useSslString = configuration.get(USE_SSL_PROPERTY);
			String keyStorePath = configuration.get(KEYSTORE_PATH_PROPERTY);
			String keyStorePassword = configuration.get(KEYSTORE_PASSWORD_PROPERTY);

			int applicationPort;
			int serverPort;
			boolean useSsl;

			if (StringUtils.isEmpty(serverHost)) {
				throw new UciProxyConfigurationFailedException("configuration property " + SERVER_HOST_PROPERTY
					+ " is not set");
			}
			if (StringUtils.isEmpty(serverPortString)) {
				throw new UciProxyConfigurationFailedException("configuration property " + SERVER_PORT_PROPERTY
					+ " is not set");
			}
			if (StringUtils.isEmpty(useSslString)) {
				throw new UciProxyConfigurationFailedException("configuration property " + USE_SSL_PROPERTY
					+ " is not set");
			}
			try {
				serverPort = Integer.parseInt(serverPortString);
			} catch (NumberFormatException e) {
				throw new UciProxyConfigurationFailedException("configuration property " + SERVER_PORT_PROPERTY
					+ " is not set to a port number");
			}
			useSsl = Boolean.parseBoolean(useSslString);
			if (useSsl) {
				if (StringUtils.isEmpty(keyStorePath)) {
					throw new UciProxyConfigurationFailedException("configuration property "
						+ KEYSTORE_PATH_PROPERTY + " is not set");
				}
				if (keyStorePassword == null) {
					throw new UciProxyConfigurationFailedException("configuration property "
						+ KEYSTORE_PASSWORD_PROPERTY + " is not set");
				}
			}

			if (!StringUtils.isEmpty(applicationHost)) {
				if (StringUtils.isEmpty(applicationPortString)) {
					throw new UciProxyConfigurationFailedException("configuration property "
						+ APPLICATION_PORT_PROPERTY + " is not set");
				}
				try {
					applicationPort = Integer.parseInt(applicationPortString);
				} catch (NumberFormatException e) {
					throw new UciProxyConfigurationFailedException("configuration property "
						+ APPLICATION_PORT_PROPERTY + " is not set to a port number");
				}
				return new HttpUcpTransportFactory(applicationHost, applicationPort, serverHost, serverPort,
					useSsl, keyStorePath, keyStorePassword);
			} else {
				return new HttpUcpTransportFactory(serverHost, serverPort, useSsl);
			}
		}
	}
}
