/*
 * Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.v30.client.transport.http;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.server.RpcObject;
import de.starface.com.rpc.server.RpcObjectRegistry;
import de.starface.com.rpc.xmlrpc.jetty.HttpXmlRpcServer;
import de.starface.integration.uci.v30.client.UcpConnectionFailedException;

import org.apache.commons.lang3.Validate;

import java.util.Collection;

/**
 * This class encapsulates the server logic of the {@link HttpUcpTransport}. Since TCP ports can be
 * opended only once the server logic must be shared between all UCP proxies that are to use the
 * same port address to receive server events.
 *
 * @author tilmann
 */
class HttpUcpTransportServer {
	private HttpXmlRpcServer httpXmlRpcServer;

	private final HttpUcpTransportEventsReceiverRegistry eventReceiverRegistry = new HttpUcpTransportEventsReceiverRegistry();

	HttpUcpTransportServer(LoginUrlAuthTokenConverter authTokenConverter, String applicationHost,
						   int applicationPort, boolean useSsl, String keyStorePath, String keyStorePassword) {
		Validate.notEmpty(applicationHost, "applicationHost=empty");
		if (applicationPort <= 0) {
			throw new IllegalArgumentException("applicationPort must not be less or equal to zero");
		}
		if (useSsl) {
			Validate.notEmpty(keyStorePath, "keyStorePath=empty");
			Validate.notNull(keyStorePassword, "keyStorePassword=null");
		}

		createServer(authTokenConverter, applicationHost, applicationPort, useSsl, keyStorePath, keyStorePassword);
	}

	synchronized <Events> void addEventsListener(LoginUrlAuthToken authToken, Events listener,
												 Class<Events> eventsInterface)
		throws UcpConnectionFailedException {
		eventReceiverRegistry.addEventsListener(authToken, listener, eventsInterface);

		Events eventsDistributor = eventReceiverRegistry.getEventsDistributor(eventsInterface);
		if (eventsDistributor != null) {
			try {
				registerEvents(authToken, eventsDistributor, eventsInterface);
			} catch (UcpConnectionFailedException e) {
				eventReceiverRegistry.removeEventsListener(authToken, eventsInterface);
				throw e;
			}
		}
	}

	synchronized <Events> void removeEventsListener(LoginUrlAuthToken authToken, Class<Events> eventsInterface) {
		eventReceiverRegistry.removeEventsListener(authToken, eventsInterface);

		Events eventsDistributor = eventReceiverRegistry.getEventsDistributor(eventsInterface);
		if (eventsDistributor == null) {
			unregisterEvents(eventsInterface);
		}
	}

	synchronized void removeEventsListeners(LoginUrlAuthToken authToken) {
		Collection<Class<?>> unusedEventsInterfaces = eventReceiverRegistry.removeEventsListeners(authToken);
		for (Class<?> unusedEventsInterface : unusedEventsInterfaces) {
			unregisterEvents(unusedEventsInterface);
		}
	}

	void addCallbackUrl(LoginUrlAuthToken authToken) {
		authToken.getUrl().setCallbackUrl(httpXmlRpcServer.getServerAddress());
	}

	private <Events> void registerEvents(LoginUrlAuthToken authToken, Events listener,
										 Class<Events> eventsInterface) throws UcpConnectionFailedException {
		ensureServerIsStarted();

		RpcObject<Events> rpcObject = RpcObject.createRpcObjectWithInterfaceMethods(eventsInterface, listener,
			null, LoginUrlAuthToken.class);
		httpXmlRpcServer.getRegistry().registerRpcObject(rpcObject);
	}

	private <Events> void unregisterEvents(Class<Events> eventsInterface) {
		httpXmlRpcServer.getRegistry().unregisterRpcObject(eventsInterface);

		stopServerIfNoLongerRequired();
	}

	private void ensureServerIsStarted() throws UcpConnectionFailedException {
		if (!httpXmlRpcServer.isRunning()) {
			startServer();
		}
	}

	private void stopServerIfNoLongerRequired() {
		if (httpXmlRpcServer.getRegistry().isEmpty()) {
			stopServer();
		}
	}

	private void createServer(LoginUrlAuthTokenConverter authTokenConverter, String applicationHost,
							  int applicationPort, boolean useSsl, String keyStorePath, String keyStorePassword) {
		if (useSsl) {
			httpXmlRpcServer = HttpXmlRpcServer.createHttpsServerWithHostName(applicationHost, applicationPort,
				"/", keyStorePath, keyStorePassword, keyStorePath, keyStorePassword);
		} else {
			httpXmlRpcServer = HttpXmlRpcServer.createHttpServerWithHostName(applicationHost, applicationPort, "/");
		}
		httpXmlRpcServer.setRegistry(new RpcObjectRegistry());
		httpXmlRpcServer.enableAuthentication(authTokenConverter);
		Runtime.getRuntime().addShutdownHook(new Thread() {
			@Override
			public void run() {
				try {
					httpXmlRpcServer.shutdown();
				} catch (Exception e) {
					// We are currently exiting anyway
				}
			}
		});
	}

	private void startServer() throws UcpConnectionFailedException {
		try {
			httpXmlRpcServer.startup();
		} catch (RpcException e) {
			throw new UcpConnectionFailedException(UcpConnectionFailedException.Codes.TRANSPORT_FAILURE,
				"Could not start http XML-RPC server", e);
		}
	}

	private void stopServer() {
		try {
			httpXmlRpcServer.shutdown();
		} catch (Exception e) {
			// We are currently exiting anyway
		}
	}
}
