/*
 * Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.v30.client.utils;

import de.starface.integration.uci.v30.client.UciProxyConfigurationFailedException;

import org.apache.commons.lang3.Validate;

/**
 * This is merely a container for utility methods used during configuration of UCI Proxies.
 *
 * @author tilmann
 *
 */
public final class ConfigurationUtils {
	private ConfigurationUtils() {
		// Do not make instances of this class
	}

	/**
	 * This method tries to load the class of the given name, check if it is an assignable form of
	 * the given interface class and returns an instance created using the default constructor.
	 *
	 * @param <I>
	 *        Type of the super class or interface that should be implemented by the class
	 * @param interfaceClass
	 *        class of the super class or interface that should be implemented by the class
	 * @param className
	 *        name of the class to be instantiated
	 *
	 * @return the newly created instance of the class
	 * @throws UciProxyConfigurationFailedException
	 *         if the class cannot be found, if it is not an assignable form of the interface class
	 *         or if it cannot be instantiated.
	 * @throws IllegalArgumentException
	 *         if either interfaceClass or className is null or empty
	 */
	@SuppressWarnings("unchecked")
	public static <I> I getInterfaceImplementationForClassName(String className, Class<I> interfaceClass)
		throws UciProxyConfigurationFailedException {
		Validate.notNull(interfaceClass, "interfaceClass=null");
		Validate.notEmpty(className, "className=empty");

		Class<? extends I> implementationClass;
		try {
			Class<?> clazz = Class.forName(className);
			if (!interfaceClass.isAssignableFrom(clazz)) {
				throw new UciProxyConfigurationFailedException("The class " + className
					+ " does not an assignable form of " + interfaceClass.getCanonicalName() + ".");
			} else {
				implementationClass = (Class<? extends I>) clazz;
			}
		} catch (ClassNotFoundException e) {
			throw new UciProxyConfigurationFailedException("The configured class " + className
				+ " could not be loaded.", e);
		}

		try {
			return implementationClass.newInstance();
		} catch (Exception e) {
			throw new UciProxyConfigurationFailedException("Could not instantiate the class "
				+ implementationClass + ".", e);
		}
	}
}
