package de.starface.integration.uci.java.v22.types;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueObject;
import de.starface.com.rpc.annotation.RpcValueType;
import de.starface.com.rpc.annotation.RpcValues;
import de.starface.integration.uci.java.v22.messages.requests.UciRedirectRequests;
import de.starface.integration.uci.java.v22.values.OrderDirection;
import de.starface.integration.uci.java.v22.values.RedirectSettingDestinationType;
import de.starface.integration.uci.java.v22.values.RedirectSettingProperties;
import de.starface.integration.uci.java.v22.values.RedirectSettingType;

import java.io.Serializable;
import java.util.List;

/**
 * This object holds the properties of the redirect settings as returned by
 * {@link UciRedirectRequests#getRedirectSettings(RedirectSettingType, String, RedirectSettingProperties, OrderDirection)}
 *
 * @author tilmann
 */
@RpcValueObject
public class RedirectSetting implements Serializable {
	private static final long serialVersionUID = 1L;

	/**
	 * The type of the redirect setting with one of the values of {@link RedirectSettingType}. The
	 * type together with the {@link #calledNumber} uniquely identify the redirect setting.
	 */
	@RpcValue
	private RedirectSettingType type;

	/**
	 * The incoming number for which the redirect holds. The calledNumber together with the
	 * {@link #type} uniquely identify the redirect setting.
	 */
	@RpcValue
	private String calledNumber;

	@RpcValue(minVersion = 22)
	private String calledNumberId;

	/** The id of a group if the number belongs to a user group */
	@RpcValues({@RpcValue(value = "isGroupCall", rpcType = RpcValueType.Boolean, maxVersion = 21),
		@RpcValue(minVersion = 22)})
	private String groupId = "";

	/** A flag indicating if the redirect is enabled/active */
	@RpcValues({@RpcValue(value = "isEnabled", maxVersion = 21), @RpcValue(minVersion = 22)})
	private boolean enabled;

	/**
	 * The type of the destination of the redirect setting with one of the values of
	 * {@link RedirectSettingDestinationType}
	 */
	@RpcValue
	private RedirectSettingDestinationType destinationType;

	/**
	 * The destination of the redirect. This may either be a phone number if the
	 * {@link #destinationType} is {@link RedirectSettingDestinationType#PHONENUMBER} or the id of
	 * one of the mailboxes given in {@link #mailboxes} if the {@link #destinationType} is
	 * {@link RedirectSettingDestinationType#VOICEMAIL}
	 */
	@RpcValue
	private String destination = "";

	/**
	 * A {@link List} that hold the information about the mailboxes that are valid destinations for
	 * the current redirect setting. The {@link List} contains one entry for every valid mailbox.
	 */
	@RpcValue
	private List<Mailbox> mailboxes;

	/**
	 * The timeout in seconds that will be used as a trigger for the redirect if the {@link #type}
	 * is set to {@link RedirectSettingType#TIMEOUT}.
	 */
	@RpcValue
	private int timeout;

	/** The last known selected destination voicemailbox */
	@RpcValue(minVersion = 22)
	private String lastDestinationMailbox = "";

	/** The last known selected destination phonenumber */
	@RpcValue(minVersion = 22)
	private String lastDestinationNumber = "";

	@Override
	public String toString() {
		return "RedirectSetting [type=" + type + ", calledNumber=" + calledNumber
			+ ", calledNumberId=" + calledNumberId + ", groupId=" + groupId + ", enabled="
			+ enabled + ", destinationType=" + destinationType + ", destination=" + destination
			+ ", mailboxes=" + mailboxes + ", timeout=" + timeout + ", lastDestinationMailbox="
			+ lastDestinationMailbox + ", lastDestinationNumber=" + lastDestinationNumber + "]";
	}

	public RedirectSettingType getType() {
		return type;
	}

	public void setType(RedirectSettingType type) {
		this.type = type;
	}

	public String getCalledNumber() {
		return calledNumber;
	}

	public void setCalledNumber(String calledNumber) {
		this.calledNumber = calledNumber;
	}

	public boolean isEnabled() {
		return enabled;
	}

	public void setEnabled(boolean enabled) {
		this.enabled = enabled;
	}

	public RedirectSettingDestinationType getDestinationType() {
		return destinationType;
	}

	public void setDestinationType(RedirectSettingDestinationType destinationType) {
		this.destinationType = destinationType;
	}

	public String getDestination() {
		return destination;
	}

	public void setDestination(String destination) {
		this.destination = destination;
	}

	public List<Mailbox> getMailboxes() {
		return mailboxes;
	}

	public void setMailboxes(List<Mailbox> mailboxes) {
		this.mailboxes = mailboxes;
	}

	public int getTimeout() {
		return timeout;
	}

	public void setTimeout(int timeout) {
		this.timeout = timeout;
	}

	public boolean isGroupRedirect() {
		return groupId != null && !"".equals(groupId);
	}

	public String getGroupId() {
		return groupId;
	}

	public void setGroupId(String groupId) {
		this.groupId = groupId;
	}

	public String getLastDestinationMailbox() {
		return lastDestinationMailbox;
	}

	public void setLastDestinationMailbox(String lastDestinationMailbox) {
		this.lastDestinationMailbox = lastDestinationMailbox;
	}

	public String getLastDestinationNumber() {
		return lastDestinationNumber;
	}

	public void setLastDestinationNumber(String lastDestinationNumber) {
		this.lastDestinationNumber = lastDestinationNumber;
	}

	public String getCalledNumberId() {
		return calledNumberId;
	}

	public void setCalledNumberId(String calledNumberId) {
		this.calledNumberId = calledNumberId;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((calledNumber == null) ? 0 : calledNumber.hashCode());
		result = prime * result + ((calledNumberId == null) ? 0 : calledNumberId.hashCode());
		result = prime * result + ((destination == null) ? 0 : destination.hashCode());
		result = prime * result + ((destinationType == null) ? 0 : destinationType.hashCode());
		result = prime * result + (enabled ? 1231 : 1237);
		result = prime * result + ((groupId == null) ? 0 : groupId.hashCode());
		result = prime * result
			+ ((lastDestinationMailbox == null) ? 0 : lastDestinationMailbox.hashCode());
		result = prime * result
			+ ((lastDestinationNumber == null) ? 0 : lastDestinationNumber.hashCode());
		result = prime * result + ((mailboxes == null) ? 0 : mailboxes.hashCode());
		result = prime * result + timeout;
		result = prime * result + ((type == null) ? 0 : type.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		RedirectSetting other = (RedirectSetting) obj;
		if (calledNumber == null) {
			if (other.calledNumber != null) {
				return false;
			}
		} else if (!calledNumber.equals(other.calledNumber)) {
			return false;
		}
		if (calledNumberId == null) {
			if (other.calledNumberId != null) {
				return false;
			}
		} else if (!calledNumberId.equals(other.calledNumberId)) {
			return false;
		}
		if (destination == null) {
			if (other.destination != null) {
				return false;
			}
		} else if (!destination.equals(other.destination)) {
			return false;
		}
		if (destinationType != other.destinationType) {
			return false;
		}
		if (enabled != other.enabled) {
			return false;
		}
		if (groupId == null) {
			if (other.groupId != null) {
				return false;
			}
		} else if (!groupId.equals(other.groupId)) {
			return false;
		}
		if (lastDestinationMailbox == null) {
			if (other.lastDestinationMailbox != null) {
				return false;
			}
		} else if (!lastDestinationMailbox.equals(other.lastDestinationMailbox)) {
			return false;
		}
		if (lastDestinationNumber == null) {
			if (other.lastDestinationNumber != null) {
				return false;
			}
		} else if (!lastDestinationNumber.equals(other.lastDestinationNumber)) {
			return false;
		}
		if (mailboxes == null) {
			if (other.mailboxes != null) {
				return false;
			}
		} else if (!mailboxes.equals(other.mailboxes)) {
			return false;
		}
		if (timeout != other.timeout) {
			return false;
		}
		if (type != other.type) {
			return false;
		}
		return true;
	}
}
