/*
 * $Id: UcpFaxListRequests.java 650 2011-11-29 18:33:32Z tilmann $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on 06.08.2009 by tilmann
 * 
 * $HeadURL:
 * http://192.168.2.8/repos/eai-uci/java/trunk/ucp/src/de/vertico/starface/integration/uci/
 * ucp/v21/server/UcpServerCommunicationFaxList.java $
 * 
 * $LastChangedDate: 2011-11-29 19:33:32 +0100 (Tue, 29 Nov 2011) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 650 $
 */
package de.starface.integration.uci.java.v22.ucp.messages.requests;

import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.com.rpc.services.filetransfer.common.RpcFileDownload;
import de.starface.integration.uci.java.v22.UciService;
import de.starface.integration.uci.java.v22.exceptions.UciException;
import de.starface.integration.uci.java.v22.exceptions.UciIllegalAccessException;
import de.starface.integration.uci.java.v22.exceptions.UciIllegalArgumentException;
import de.starface.integration.uci.java.v22.ucp.exceptions.UcpNotConnectedException;
import de.starface.integration.uci.java.v22.values.FaxListEntryFolder;
import de.starface.integration.uci.java.v22.values.FaxListEntryProperties;
import de.starface.integration.uci.java.v22.values.FaxListProperties;
import de.starface.integration.uci.java.v22.values.GroupRestriction;
import de.starface.integration.uci.java.v22.values.OrderDirection;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * This interface defines the fax list requests provided by the UCI server.
 *
 * @author tilmann
 */
@RpcInterface(UcpFaxListRequests.SERVICE_NAME)
@UciService(UcpFaxListRequests.SERVICE_NAME)
public interface UcpFaxListRequests {
	/** The UCI service name used for this interface */
	public static final String SERVICE_NAME = "ucp.v22.requests.faxList";

	/**
	 * Retrieve a list of received and sent faxes of the current user. The number of entries
	 * retrieved can be restricted by several parameters of this procedure call. The results are
	 * returned as {@link Map} containing the values for the keys defined in
	 * {@link FaxListProperties}.
	 *
	 * @param receivedBefore
	 *        limit returned results to faxes received/sent before the given point of time
	 * @param receivedAfter
	 *        limit returned results to faxes received/sent after the given point of time
	 * @param folderRestriction
	 *        restrict returned results to faxes in the given folder defined in
	 *        {@link FaxListEntryFolder}. If an empty String is given no restriction is used.
	 * @param groupRestriction
	 *        restrict returned results to those conducted by a specified group. Possible choices
	 *        are the String {@link GroupRestriction#NON_GROUP} to return all faxes that do not
	 *        belong to any group, the id of a group to restrict the list to the faxes belonging to
	 *        that group and the empty String to return all faxes without restriction.
	 * @param orderProperty
	 *        give the name of one of the {@link FaxListEntryProperties} to order results by the
	 *        value of the property, when an empty String is given the default
	 *        {@link FaxListEntryProperties#receptionTime} is used.
	 * @param orderDirection
	 *        give the name of one of the {@link OrderDirection}s to define the ordering. If not set
	 *        the default order direction of the property will be used
	 * @param countOffset
	 *        the offset of the first returned value in the result list after the ordering has been
	 *        applied
	 * @param countLimit
	 *        the count of items returned in the result list
	 * @return a {@link Map} containing the results with the properties defined in
	 *         {@link FaxListProperties}
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public Map<String, Object> getFaxList(Date receivedBefore, Date receivedAfter, String folderRestriction,
										  String groupRestriction, String orderProperty, String orderDirection,
										  int countOffset, int countLimit) throws UciException;

	/**
	 * Creates and returns a temporary URL that can be used to download the fax file for the next 15
	 * minutes.
	 *
	 * @param faxId
	 *        the id of the fax list entry to retrieve the file for
	 * @return The download URL as {@link String}
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public String getFaxFileUrl(String faxId) throws UciException;

	/**
	 * Prepares the download of the fax file for the given fax id. After a calling this request the
	 * file can be downloaded using the interface {@link FaxFileDownload} and the returned fileId.
	 *
	 * @param faxId
	 *        the id of the fax list entry to retrieve the file for
	 * @return The fileId that can be used to download the file from STARFACE
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public String getFaxFile(String faxId) throws UciException;

	/**
	 * This interface defines the RPC methods that can be used to download the fax files
	 */
	@RpcInterface(SERVICE_NAME + ".getFaxFile.rpcFileDownload")
	public interface FaxFileDownload extends RpcFileDownload {
		// Just mapping the procedures from the super interface with the correct service name
	}

	/**
	 * Remove a fax list entry from the list
	 *
	 * @param faxId
	 *        the id of the fax list entry to remove
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void deleteFaxListEntry(String faxId) throws UciException;

	/**
	 * Remove multiple fax list entries from the list
	 *
	 * @param faxIds
	 *        the ids of the fax list entries to remove
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void deleteFaxListEntries(List<String> faxIds) throws UciException;

	/**
	 * Move a fax list entry to another folder
	 *
	 * @param faxId
	 *        the id of the fax list entry to move
	 * @param destinationFolder
	 *        one of the following {@link FaxListEntryFolder} {@link FaxListEntryFolder#NEW},
	 *        {@link FaxListEntryFolder#OLD} or {@link FaxListEntryFolder#PRIVATE} -
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}). A
	 *         {@link UciIllegalArgumentException} will be raised if the request tries to move a fax
	 *         to the {@link FaxListEntryFolder#SENT} folder.
	 */
	public void moveFaxListEntry(String faxId, String destinationFolder) throws UciException;

	/**
	 * Move multiple fax list entries to another folder
	 *
	 * @param faxIds
	 *        the ids of the fax list entries to move
	 * @param destinationFolder
	 *        one of the following {@link FaxListEntryFolder} {@link FaxListEntryFolder#NEW},
	 *        {@link FaxListEntryFolder#OLD} or {@link FaxListEntryFolder#PRIVATE} -
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}). A
	 *         {@link UciIllegalArgumentException} will be raised if the request tries to move a fax
	 *         to the {@link FaxListEntryFolder#SENT} folder.
	 */
	public void moveFaxListEntries(List<String> faxIds, String destinationFolder) throws UciException;
}
