/*
 * $Id: UcpFunctionKeyRequests.java 659 2012-01-23 15:54:23Z tilmann $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on 06.08.2009 by tilmann
 * 
 * $HeadURL:
 * http://192.168.2.8/repos/eai-uci/java/trunk/ucp/src/de/vertico/starface/integration/uci/
 * ucp/v21/server/UcpServerCommunicationFax.java $
 * 
 * $LastChangedDate: 2012-01-23 16:54:23 +0100 (Mon, 23 Jan 2012) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 659 $
 */
package de.starface.integration.uci.java.v22.ucp.messages.requests;

import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.integration.uci.java.v22.UciService;
import de.starface.integration.uci.java.v22.exceptions.UciException;
import de.starface.integration.uci.java.v22.exceptions.UciIllegalAccessException;
import de.starface.integration.uci.java.v22.exceptions.UciIllegalArgumentException;
import de.starface.integration.uci.java.v22.ucp.exceptions.UcpNotConnectedException;
import de.starface.integration.uci.java.v22.values.CallInfoProperties;
import de.starface.integration.uci.java.v22.values.ContactInfoProperties;
import de.starface.integration.uci.java.v22.values.FunctionKeyProperties;
import de.starface.integration.uci.java.v22.values.FunctionKeyType;
import de.starface.integration.uci.java.v22.values.ImageProperties;
import de.starface.integration.uci.java.v22.values.RedirectSettingProperties;

import java.util.List;
import java.util.Map;

/**
 * This interface defines the function key requests provided by the UCI server.
 *
 * @author tilmann
 */
@RpcInterface(UcpFunctionKeyRequests.SERVICE_NAME)
@UciService(UcpFunctionKeyRequests.SERVICE_NAME)
public interface UcpFunctionKeyRequests {
	/** The UCI service name used for this interface */
	public static final String SERVICE_NAME = "ucp.v22.requests.functionKey";

	/**
	 * This request allows the UCI client to retrieve the function keys of the current user.
	 *
	 * @return A {@link List} of {@link Map} containing the function keys with the values as defined
	 *         in {@link FunctionKeyProperties}
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if the request is not allowed to the user ({@link UciIllegalAccessException}) or if
	 *         the request is placed while the UCI client is not connected (
	 *         {@link UcpNotConnectedException}).
	 */
	public List<Map<String, Object>> getFunctionKeys() throws UciException;

	/**
	 * This request allows the UCI client to virtually press a function key of the current user. A
	 * function key press usually leads to an action on the server depending on the
	 * {@link FunctionKeyType} of the key.
	 *
	 * @param functionKeyId
	 *        the id of the function key to be pressed.
	 * @param parameters
	 *        additional parameters used to modify the behavior of the action to be activated upon
	 *        the key press. The meaning of the parameters depends on the function key to be
	 *        pressed. It is documented together with the {@link FunctionKeyType}.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void pressKey(String functionKeyId, List<Object> parameters) throws UciException;

	/**
	 * This request allows the UCI client to query which redirect always settings have been
	 * activated by the user or group the key of type {@link FunctionKeyType#BUSY_LAMP_FIELD} shows
	 * the status of.
	 *
	 * @param functionKeyId
	 *        the id of the function key to obtain the settings for
	 * @return a list with the redirect settings as {@link List} of {@link Map} containing values as
	 *         defined in {@link RedirectSettingProperties}.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public List<Map<String, Object>> getRedirectSettingsForKey(String functionKeyId) throws UciException;

	/**
	 * This request allows the UCI client to query the contact information of the user or group the
	 * key of type {@link FunctionKeyType#BUSY_LAMP_FIELD} shows the status of.
	 *
	 * @param functionKeyId
	 *        the id of the function key to obtain the contact information for
	 * @return the contact information of the user or group as {@link Map} containing values as
	 *         defined in {@link ContactInfoProperties}.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public Map<String, Object> getContactInfoForKey(String functionKeyId) throws UciException;

	/**
	 * This request allows the UCI client to query information about the calls currently being held
	 * or received by the user or group the key of type {@link FunctionKeyType#BUSY_LAMP_FIELD}
	 * shows the status of.
	 *
	 * @param functionKeyId
	 *        the id of the function key to obtain the call information for
	 * @return the information about the calls in a {@link List} of {@link Map} with values as
	 *         defined in {@link CallInfoProperties}.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public List<Map<String, Object>> getCallInfoForKey(String functionKeyId) throws UciException;

	/**
	 * This request allows the UCI client to query the image of the key. A key has an image if its
	 * property {@link FunctionKeyProperties#imageHash} is not empty. Currently only keys of type
	 * {@link FunctionKeyType#BUSY_LAMP_FIELD} that show the status of a user can have an image
	 * which then is the avatar image of the user.
	 *
	 * @param functionKeyId
	 *        the id of the function key to obtain the image for
	 * @return the image of the key as {@link Map} with values as defined by {@link ImageProperties}
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public Map<String, Object> getImageForKey(String functionKeyId) throws UciException;

}
