/*
 * $Id: UciClient.java 296 2010-01-28 18:00:12Z tilmann $
 * 
 * Copyright (C) by vertico software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * $LastChangedDate: 2010-01-28 19:00:12 +0100 (Do, 28 Jan 2010) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 296 $
 */
package de.starface.integration.uci.java.v30.messages.requests;

import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.com.rpc.annotation.RpcValueTranslation;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.UciServices;
import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalAccessException;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalArgumentException;
import de.starface.integration.uci.java.v30.types.MonitorList;
import de.starface.integration.uci.java.v30.ucp.exceptions.UcpNotConnectedException;
import de.starface.integration.uci.java.v30.ucp.messages.requests.UcpMonitorListRequests;
import de.starface.integration.uci.java.v30.values.MonitorListEntryProperties;
import de.starface.integration.uci.java.v30.values.OrderDirection;

import java.io.File;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * This interface contains the UCI Communication Voicemail List Requests
 *
 * @author tilmann
 */
@UciService(UcpMonitorListRequests.SERVICE_NAME)
@RpcInterface(UcpMonitorListRequests.SERVICE_NAME)
@RpcValueTranslation(version = UciServices.UCI_VERSION, automaticFileTransfer = true)
public interface UciMonitorListRequests {
	/**
	 * Retrieve a list of monitors of the current user. The number of entries retrieved can be
	 * restricted by several parameters of this procedure call. The results are returned as
	 * {@link List} of {@link Map} containing the values for the keys defined in
	 * {@link MonitorListEntryProperties}.
	 *
	 * @param startBefore
	 *        limit returned results to monitors started before the given point of time
	 * @param startAfter
	 *        limit returned results to monitors started after the given point of time
	 * @param orderProperty
	 *        give one of the {@link MonitorListEntryProperties} to order results by the value of
	 *        the property. Allowed values are {@link MonitorListEntryProperties#monitortimestart},
	 *        {@link MonitorListEntryProperties#duration},
	 *        {@link MonitorListEntryProperties#callercallerid} and
	 *        {@link MonitorListEntryProperties#calledcallerid}. When null or an invalid property is
	 *        given the default {@link MonitorListEntryProperties#monitortimestart} is used.
	 * @param orderDirection
	 *        give one of the {@link OrderDirection}s to define the ordering. If null is given the
	 *        default order direction of the property will be used
	 * @param countOffset
	 *        the offset of the first returned value in the result list after the ordering has been
	 *        applied
	 * @param countLimit
	 *        the count of items returned in the result list
	 * @return a {@link List} containing the result elements
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public MonitorList getMonitorList(Date startAfter, Date startBefore, MonitorListEntryProperties orderProperty,
									  OrderDirection orderDirection, int countOffset, int countLimit)
		throws UciException;

	/**
	 * Remove a monitor list entry from the list
	 *
	 * @param monitorId
	 *        the id of the monitor list entry to remove
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void deleteMonitorListEntry(String monitorId) throws UciException;

	/**
	 * Remove multiple monitor list entries from the list
	 *
	 * @param monitorIds
	 *        the ids of the monitor list entries to remove
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void deleteMonitorListEntries(List<String> monitorIds) throws UciException;

	/**
	 * Creates and returns a temporary URL that can be used to download the monitor file for the
	 * next 15 minutes.
	 *
	 * @param monitorId
	 *        the id of the monitor list entry to retrieve the file for
	 * @return The download URL as {@link String}
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public String getMonitorFileUrl(String monitorId) throws UciException;

	/**
	 * Downloads and returns the monitor for the given monitor id
	 *
	 * @param monitorId
	 *        the id of the monitor list entry to retrieve the file for
	 * @return the monitor file
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public File getMonitorFile(String monitorId) throws UciException;

}
