/*
 * $Id: UcpCallRequests.java 721 2013-04-30 12:36:07Z dirk $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on 06.08.2009 by tilmann
 * 
 * $HeadURL:
 * http://192.168.2.8/repos/eai-uci/java/trunk/ucp/src/de/vertico/starface/integration/uci/
 * ucp/v21/server/UcpServerCommunicationCall.java $
 * 
 * $LastChangedDate: 2013-04-30 14:36:07 +0200 (Di, 30. Apr 2013) $ $LastChangedBy: dirk $
 * $LastChangedRevision: 721 $
 */
package de.starface.integration.uci.java.v30.ucp.messages.requests;

import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalAccessException;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalArgumentException;
import de.starface.integration.uci.java.v30.types.ConferenceCall;
import de.starface.integration.uci.java.v30.ucp.exceptions.UcpNotConnectedException;

import java.util.List;

/**
 * This interface defines the call requests provided by the UCI server.
 *
 * @author dirk
 */
@UciService(UcpConferenceCallRequests.SERVICE_NAME)
@RpcInterface(UcpConferenceCallRequests.SERVICE_NAME)
public interface UcpConferenceCallRequests {
	/** The UCI service name used for this interface */
	public static final String SERVICE_NAME = "ucp.v30.requests.conferencecall";

	/**
	 * This request tells the UCI server to hang up the call identified by the given UUID.
	 *
	 * @param callId
	 *        the UUID of the call to be hung up
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void hangupCall(String callId) throws UciException;

	/**
	 * This request tells the UCI server to transfer a call to a conference.
	 *
	 * @param roomId
	 *        a conference room id.
	 * @param uuidToTransfer
	 *        the call to be transfered to the conference.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void transferToConference(String roomId, String uuidToTransfer) throws UciException;

	/**
	 * This request tells the UCI server to join a conference.
	 *
	 * @param roomId
	 *        a conference room id.
	 * @param phone
	 *        the phone to use.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void joinToConference(String roomId, String phone) throws UciException;

	/**
	 * This request tells the UCI server to mute or unmute the call.
	 *
	 * @param uuidConferenceCall
	 *        a call that is in the conference.
	 * @param mute
	 *        set the mute option on or off on the uuidConferenceCall.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void muteToConference(String uuidConferenceCall, boolean mute) throws UciException;

	/**
	 * This request tells the UCI server that the confernce call can hear or not hear the outher
	 * participants.
	 *
	 * @param uuidConferenceCall
	 *        a call that is in the conference.
	 * @param listen
	 *        set the listen option on or off on the uuidConferenceCall.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void listenToConference(String uuidConferenceCall, boolean listen) throws UciException;

	/**
	 * This request tells the UCI server that the confernce call wants to talk.
	 *
	 * @param uuidConferenceCall
	 *        a call that is in the conference.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void wantToTalkToConference(String uuidConferenceCall) throws UciException;

	/**
	 * This request tells the UCI server to call that the phone number into the conference.
	 *
	 * @param roomId
	 *        a conference room id.
	 * @param number
	 *        the phonenumber to add to conference.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void callNumberToConference(String roomId, String number) throws UciException;

	/**
	 * This request tells the UCI server to leave the conference.
	 *
	 * @param roomId
	 *        a conference room id.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void leaveConference(String roomId) throws UciException;

	/**
	 * This request tells the UCI server to quit the conference.
	 *
	 * @param roomId
	 *        a conference room id.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void quitConference(String roomId) throws UciException;

	/**
	 * This request tells the UCI server to get all conference ids.
	 *
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public List<String> getAllConferences() throws UciException;

	/**
	 * This request tells the UCI server to get all conference call ids.
	 *
	 * @param roomId
	 *        the conference room id.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public List<ConferenceCall> getAllConferenceCallsInRoom(String roomId) throws UciException;

	/**
	 * This request tells the UCI server to set all conference participants to hear.
	 *
	 * @param roomId
	 *        the conference room id.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void allHearInRoom(String roomId) throws UciException;

	/**
	 * This request tells the UCI server to enable speaking on all participants.
	 *
	 * @param roomId
	 *        the conference room id.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void allCanSpeakInRoom(String roomId) throws UciException;

	/**
	 * This request tells the UCI server to enable speaking for one participant.
	 *
	 * @param uuidConferenceCall
	 *        a call that is in the conference.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void onlyOneCanSpeakInRoom(String uuidConferenceCall) throws UciException;

	/**
	 * This request tells the UCI server to start the conference with the given id. If no
	 * conferenceId is given a conference is startet with the given phone.
	 *
	 * @param conferenceId
	 *        id of the conference.
	 * @param phoneId
	 *        phone to dial to the conference.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void startManagedConferenceById(String conferenceId, String phoneId) throws UciException;

}
