package de.starface.integration.uci.java.v30.values;

import de.starface.integration.uci.java.v30.ucp.messages.requests.UcpFunctionKeyRequests;

import java.util.List;

/**
 * This enum defines the different types a function key can have. The constants are used for the
 * function key property {@link FunctionKeyProperties#type}.
 *
 * @author tilmann
 */
public enum FunctionKeyType {
	/**
	 * The busy lamp field function key shows the telephony state of another user. The state of this
	 * function key has the following meanings:
	 * <p>
	 * {@link FunctionKeyState#UNAVAILABLE} The other user currently has no telephones that can be
	 * called
	 * <p>
	 * {@link FunctionKeyState#INACTIVE} The other user is currently idle. The key can be pressed to
	 * call the user.
	 * <p>
	 * {@link FunctionKeyState#FLASHY} The other user is currently receiving a call and his phones
	 * are ringing. The key can be pressed to grab the incoming phone call.
	 * <p>
	 * {@link FunctionKeyState#ACTIVE} The other user is currently in an active call. The function
	 * key can still be pressed to call that user but this will only be successful if the user has
	 * call waiting indication activated.
	 * <p>
	 * {@link FunctionKeyState#INVALID} The other user's account has been removed.
	 * <p>
	 * In addition the busy lamp field function key is the only key to hold a
	 * {@link FunctionKeyProperties#userState} property that indicates the state of the user/group
	 * the key points to.
	 * <p>
	 * The first parameter of the {@link UcpFunctionKeyRequests#pressKey(String, List)} request
	 * denotes the phone of the user to be used to initiate/grab the call, with an empty string
	 * standing for the user's primary phone.
	 */
	BUSY_LAMP_FIELD, // corresponds to ACCOUNTFK on the server

	/**
	 * The quick dial function key is used to hold a single telephone number that is called when the
	 * function key is pressed.
	 * <p>
	 * The parameter of the {@link UcpFunctionKeyRequests#pressKey(String, List)} request denotes
	 * the phone of the user to be used to initiate the call, with an empty string standing for the
	 * user's primary phone.
	 */
	QUICK_DIAL, // corresponds to QUICKDIALFK on the server

	/**
	 * The group login function key can be used to log the user in/out of a defined set of groups he
	 * is a member of. The state of this function key has the following meanings:
	 * <p>
	 * {@link FunctionKeyState#INACTIVE} The user is not logged in to any of associated groups.
	 * <p>
	 * {@link FunctionKeyState#FLASHY} The user is logged in to some but not all of the associated
	 * groups.
	 * <p>
	 * {@link FunctionKeyState#ACTIVE} The user is logged in to all of the associated groups.
	 * <p>
	 * {@link FunctionKeyState#INVALID} All of the associated groups have been deleted.
	 * <p>
	 * Pressing the key leads to the active state if the state was inactive or blinking and leads to
	 * the inactive state if the state was active. The key press thus toggles the state. As an
	 * alternative if a first {@link Boolean} parameter is given to
	 * {@link UcpFunctionKeyRequests#pressKey(String, List)} this parameter determines the requested
	 * new state of the group login.
	 */
	GROUP_LOGIN, // corresponds to GROUPFK on the server

	/**
	 * The signal number function key can be used to set the associated external number as
	 * displaynumber. The state of this function key has the following meanings:
	 * <p>
	 * {@link FunctionKeyState#INACTIVE} The associated number is not used as signal number.
	 * <p>
	 * {@link FunctionKeyState#ACTIVE} The associated number is used as signal number.
	 * <p>
	 * {@link FunctionKeyState#INVALID} The associated number has been deleted.
	 */
	SIGNAL_NUMBER, // corresponds to SIGNALNUMBERFK on the server

	/**
	 * The module activation function key can be used to activate/deactivate a defined set of
	 * modules. The state of this function key has the following meanings:
	 * <p>
	 * {@link FunctionKeyState#INACTIVE} All of the associated modules are inactive.
	 * <p>
	 * {@link FunctionKeyState#FLASHY} Some but not all of the associated modules are active.
	 * <p>
	 * {@link FunctionKeyState#ACTIVE} All of the associated modules are active.
	 * <p>
	 * {@link FunctionKeyState#INVALID} All of the associated modules have been deleted.
	 * <p>
	 * Pressing the key leads to the active state if the state was inactive or blinking and leads to
	 * the inactive state if the state was active. The key press thus toggles the state. As an
	 * alternative if a first {@link Boolean} parameter is given to
	 * {@link UcpFunctionKeyRequests#pressKey(String, List)} this parameter determines the requested
	 * new state of the module activation.
	 */
	MODULE_ACTIVATION, // corresponds to MODULEINSTANCEFK on the server

	/**
	 * The do not disturb function key can be used to set/reset the user's do not disturb status.
	 * The state of this function key has the following meanings:
	 * <p>
	 * {@link FunctionKeyState#INACTIVE} The do not disturb status is inactive.
	 * <p>
	 * {@link FunctionKeyState#ACTIVE} he do not disturb status is active.
	 * <p>
	 * Pressing the key leads to the active state if the state was inactive and leads to the
	 * inactive state if the state was active. The key press thus toggles the state. As an
	 * alternative if a first {@link Boolean} parameter is given to
	 * {@link UcpFunctionKeyRequests#pressKey(String, List)} this parameter determines the requested
	 * new do not disturb status.
	 */
	DO_NOT_DISTURB, // corresponds to DNDFK on the server

	/**
	 * The completion of calls to busy subscriber (CCBS) function key is used to handle the
	 * telephony feature with the same name. The state of this function key has the following
	 * meanings:
	 * <p>
	 * {@link FunctionKeyState#INACTIVE} CCBS is currently not possible because there was no
	 * outgoing call that was signaled as busy.
	 * <p>
	 * {@link FunctionKeyState#FLASHY} CCBS has been requested and is being performed until a
	 * timeout occurs or the call has been established or the CCBS request is cleared by a new key
	 * press each leading to the {@link FunctionKeyState#INACTIVE} state.
	 * <p>
	 * {@link FunctionKeyState#ACTIVE} CCBS is currently possible because there was an outgoing call
	 * that was signaled as busy but has not been requested by the user. A key press will request
	 * CCBS leading to the {@link FunctionKeyState#FLASHY} state. A timeout with no action will lead
	 * back to the {@link FunctionKeyState#INACTIVE} state.
	 * <p>
	 * The normal key press thus toggles the state. As an alternative if a first {@link Boolean}
	 * parameter is given to {@link UcpFunctionKeyRequests#pressKey(String, List)} this parameter
	 * determines the requested new CCBS state.
	 */
	COMPLETION_OF_CALLS_TO_BUSY_SUBSCRIBER, // corresponds to CCBSFK on the server

	/**
	 * The park and orbit (P+O) function key is used to handle the telephony feature with the same
	 * name for a single associated parking lot per key. The state of this function key has the
	 * following meanings:
	 * <p>
	 * {@link FunctionKeyState#INACTIVE} The parking lot associated with this function key is empty
	 * and ready to hold a phone call. Pressing the key will forward the user's currently active
	 * call to the associated parking lot.
	 * <p>
	 * {@link FunctionKeyState#FLASHY} A phone call has been parked in this function key's
	 * associated parking lot and is been ready to be picked. The call can be picked by pressing
	 * this key giving the phone to be used to pick the call as parameter to the
	 * {@link UcpFunctionKeyRequests#pressKey(String, List)} request using the empty string for the
	 * user's primary phone.
	 */
	PARK_AND_ORBIT, // corresponds to PAOFK on the server

	/**
	 * The forward call unconditional function key can be used to activate/deactivate the
	 * unconditional forwards for all of the user's phone numbers. The state of this function key
	 * has the following meanings:
	 * <p>
	 * {@link FunctionKeyState#INACTIVE} All of the user's phone number's unconditional forwards are
	 * inactive.
	 * <p>
	 * {@link FunctionKeyState#FLASHY} Some but not all of the user's phone number's unconditional
	 * forwards are active.
	 * <p>
	 * {@link FunctionKeyState#ACTIVE} All of the user's phone number's unconditional forwards are
	 * active.
	 * <p>
	 * Pressing the key leads to the active state if the state was inactive or blinking and leads to
	 * the inactive state if the state was active. The key press thus toggles the state. As an
	 * alternative if a first {@link Boolean} parameter is given to
	 * {@link UcpFunctionKeyRequests#pressKey(String, List)} this parameter determines the requested
	 * new state of the call forwarding.
	 * <p>
	 * In addition a second {@link String} parameter holding the desired target phone number can be
	 * given. If so the forwards are set to forward to this phone number independent of their
	 * previous forward target settings.
	 */
	FORWARD_CALL_UNCONDITIONAL, // corresponds to FCUFK on the server

	/**
	 * The forward call busy function key can be used to activate/deactivate the busy forwards for
	 * all of the user's phone numbers. The state of this function key has the following meanings:
	 * <p>
	 * {@link FunctionKeyState#INACTIVE} All of the user's phone number's busy forwards are
	 * inactive.
	 * <p>
	 * {@link FunctionKeyState#FLASHY} Some but not all of the user's phone number's busy forwards
	 * are active.
	 * <p>
	 * {@link FunctionKeyState#ACTIVE} All of the user's phone number's busy forwards are active.
	 * <p>
	 * Pressing the key leads to the active state if the state was inactive or blinking and leads to
	 * the inactive state if the state was active. The key press thus toggles the state. As an
	 * alternative if a first {@link Boolean} parameter is given to
	 * {@link UcpFunctionKeyRequests#pressKey(String, List)} this parameter determines the requested
	 * new state of the call forwarding.
	 * <p>
	 * In addition a second {@link String} parameter holding the desired target phone number can be
	 * given. If so the forwards are set to forward to this phone number independent of their
	 * previous forward target settings.
	 */
	FORWARD_CALL_BUSY, // corresponds to FCBFK on the server

	/**
	 * The forward call no response function key can be used to activate/deactivate the no response
	 * forwards for all of the user's phone numbers. The state of this function key has the
	 * following meanings:
	 * <p>
	 * {@link FunctionKeyState#INACTIVE} All of the user's phone number's no response forwards are
	 * inactive.
	 * <p>
	 * {@link FunctionKeyState#FLASHY} Some but not all of the user's phone number's no response
	 * forwards are active.
	 * <p>
	 * {@link FunctionKeyState#ACTIVE} All of the user's phone number's no response forwards are
	 * active.
	 * <p>
	 * Pressing the key leads to the active state if the state was inactive or blinking and leads to
	 * the inactive state if the state was active. The key press thus toggles the state. As an
	 * alternative if a first {@link Boolean} parameter is given to
	 * {@link UcpFunctionKeyRequests#pressKey(String, List)} this parameter determines the requested
	 * new state of the call forwarding.
	 * <p>
	 * In addition a second {@link String} parameter holding the desired target phone number can be
	 * given. If so the forwards are set to forward to this phone number independent of their
	 * previous forward target settings.
	 */
	FORWARD_CALL_NO_RESPONSE, // corresponds to FCNRFK on the server

	/**
	 * The forward number unconditional function key can be used to activate/deactivate the
	 * unconditional forwards for a defined set of the user's phone numbers. The state of this
	 * function key has the following meanings:
	 * <p>
	 * {@link FunctionKeyState#INACTIVE} All of the associated phone number's unconditional forwards
	 * are inactive.
	 * <p>
	 * {@link FunctionKeyState#FLASHY} Some but not all of the associated phone number's
	 * unconditional forwards are active.
	 * <p>
	 * {@link FunctionKeyState#ACTIVE} All of the associated phone number's unconditional forwards
	 * are active.
	 * <p>
	 * {@link FunctionKeyState#INVALID} All of the associated phone numbers have been deleted.
	 * <p>
	 * Pressing the key leads to the active state if the state was inactive or blinking and leads to
	 * the inactive state if the state was active. The key press thus toggles the state. As an
	 * alternative if a first {@link Boolean} parameter is given to
	 * {@link UcpFunctionKeyRequests#pressKey(String, List)} this parameter determines the requested
	 * new state of the call forwarding.
	 */
	FORWARD_NUMBER_UNCONDITIONAL, // corresponds to FORWARDFK on the server

	/**
	 * The separator function key is used to separate function keys by blank fields or a caption.
	 * The state of this function key does not have any meaning.
	 */
	SEPARATOR,
	/**
	 * Telephone-only functionality. Stateless. Render like SEPARATOR.
	 */
	PHONE_CONTACT_SEARCH,
	/**
	 * Telephone-only functionality. Stateless. Render like SEPARATOR.
	 */
	PHONE_CONTACT_LIST,
	/**
	 * Telephone-only functionality. Stateless. Render like SEPARATOR.
	 */
	PHONE_CALLLIST_INCOMING,
	/**
	 * Telephone-only functionality. Stateless. Render like SEPARATOR.
	 */
	PHONE_CALLLIST_MISSED,
	/**
	 * Telephone-only functionality. Stateless. Render like SEPARATOR.
	 */
	PHONE_CALLLIST_OUTGOING,
	/**
	 * Telephone-only functionality. Stateless. Render like SEPARATOR.
	 */
	PHONE_GENERIC_URL,
	/**
	 * Telephone-only functionality. Stateless. Render like SEPARATOR.
	 */
	PHONE_DTMF,

	/**
	 * The forward to target function key can be used to activate/deactivate the
	 * unconditional forwards for a defined set of the user's phone numbers to a defined number or mailbox.
	 * The state of this function key has the following meanings:
	 * <p>
	 * {@link FunctionKeyState#INACTIVE} Any of the associated phone number's unconditional forwards
	 * are inactive or to another target.
	 * <p>
	 * {@link FunctionKeyState#ACTIVE} All of the associated phone number's unconditional forwards
	 * to the correct target are active.
	 * <p>
	 * {@link FunctionKeyState#INVALID} All of the associated phone numbers have been deleted.
	 * <p>
	 * Pressing the key leads to the active state if the state was inactive and leads to
	 * the inactive state if the state was active. The key press thus toggles the state. As an
	 * alternative if a first {@link Boolean} parameter is given to
	 * {@link UcpFunctionKeyRequests#pressKey(String, List)} this parameter determines the requested
	 * new state of the call forwarding.
	 */
	FORWARD_TO_TARGET
}
